@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -8,11 +8,11 @@
 
 #include "GLContext.h"
 #include "jsapi.h"
+#include "js/Warnings.h"  // JS::WarnASCII
 #include "mozilla/dom/ScriptSettings.h"
 #include "mozilla/gfx/Logging.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/Sprintf.h"
-#include "nsIDOMEvent.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsIVariant.h"
 #include "nsPrintfCString.h"
@@ -69,7 +69,7 @@ void WebGLContext::GenerateWarning(const char* fmt, va_list ap) const {
   char buf[1024];
   VsprintfLiteral(buf, fmt, ap);
 
-  // no need to print to stderr, as JS_ReportWarning takes care of this for us.
+  // JS::WarnASCII will print to stderr for us.
 
   if (!mCanvasElement) {
     return;
@@ -81,13 +81,13 @@ void WebGLContext::GenerateWarning(const char* fmt, va_list ap) const {
   }
 
   JSContext* cx = api.cx();
-  JS_ReportWarningASCII(cx, "WebGL warning: %s", buf);
+  const auto funcName = FuncName();
+  JS::WarnASCII(cx, "WebGL warning: %s: %s", funcName, buf);
   if (!ShouldGenerateWarnings()) {
-    JS_ReportWarningASCII(cx,
-                          "WebGL: No further warnings will be reported for"
-                          " this WebGL context."
-                          " (already reported %d warnings)",
-                          mAlreadyGeneratedWarnings);
+    JS::WarnASCII(cx,
+                  "WebGL: No further warnings will be reported for this WebGL "
+                  "context. (already reported %d warnings)",
+                  mAlreadyGeneratedWarnings);
   }
 }
 
@@ -118,15 +118,15 @@ void WebGLContext::GeneratePerfWarning(const char* fmt, ...) const {
 
   ////
 
-  JS_ReportWarningASCII(cx, "WebGL perf warning: %s", buf);
+  const auto funcName = FuncName();
+  JS::WarnASCII(cx, "WebGL perf warning: %s: %s", funcName, buf);
   mNumPerfWarnings++;
 
   if (!ShouldGeneratePerfWarnings()) {
-    JS_ReportWarningASCII(
-        cx,
-        "WebGL: After reporting %u, no further perf warnings will"
-        " be reported for this WebGL context.",
-        uint32_t(mNumPerfWarnings));
+    JS::WarnASCII(cx,
+                  "WebGL: After reporting %u, no further perf warnings will be "
+                  "reported for this WebGL context.",
+                  uint32_t(mNumPerfWarnings));
   }
 }
 
@@ -140,7 +140,8 @@ void WebGLContext::SynthesizeGLError(GLenum err) const {
   if (!mWebGLError) mWebGLError = err;
 }
 
-void WebGLContext::SynthesizeGLError(GLenum err, const char* fmt, ...) const {
+void WebGLContext::GenerateError(const GLenum err, const char* const fmt,
+                                 ...) const {
   va_list va;
   va_start(va, fmt);
   GenerateWarning(fmt, va);
@@ -167,15 +168,6 @@ void WebGLContext::ErrorInvalidEnumInfo(const char* info,
                           name.BeginReading());
 }
 
-void WebGLContext::ErrorInvalidEnumInfo(const char* info, const char* funcName,
-                                        GLenum enumValue) const {
-  nsCString name;
-  EnumName(enumValue, &name);
-
-  ErrorInvalidEnum("%s: %s: Invalid enum: 0x%04x (%s).", funcName, info,
-                   enumValue, name.BeginReading());
-}
-
 void WebGLContext::ErrorInvalidOperation(const char* fmt, ...) const {
   va_list va;
   va_start(va, fmt);
@@ -248,16 +240,13 @@ void WebGLContext::ErrorImplementationBug(const char* fmt, ...) const {
 }
 
 // This version is fallible and will return nullptr if unrecognized.
-static const char* GetEnumName(GLenum val) {
+const char* GetEnumName(const GLenum val, const char* const defaultRet) {
   switch (val) {
 #define XX(x)        \
   case LOCAL_GL_##x: \
     return #x
     XX(NONE);
     XX(ALPHA);
-    XX(ATC_RGB);
-    XX(ATC_RGBA_EXPLICIT_ALPHA);
-    XX(ATC_RGBA_INTERPOLATED_ALPHA);
     XX(COMPRESSED_RGBA_PVRTC_2BPPV1);
     XX(COMPRESSED_RGBA_PVRTC_4BPPV1);
     XX(COMPRESSED_RGBA_S3TC_DXT1_EXT);
@@ -574,11 +563,12 @@ static const char* GetEnumName(GLenum val) {
 #undef XX
   }
 
-  return nullptr;
+  return defaultRet;
 }
 
-/*static*/ void WebGLContext::EnumName(GLenum val, nsCString* out_name) {
-  const char* name = GetEnumName(val);
+/*static*/
+void WebGLContext::EnumName(GLenum val, nsCString* out_name) {
+  const char* name = GetEnumName(val, nullptr);
   if (name) {
     *out_name = name;
     return;
@@ -587,61 +577,20 @@ static const char* GetEnumName(GLenum val) {
   *out_name = nsPrintfCString("<enum 0x%04x>", val);
 }
 
-void WebGLContext::ErrorInvalidEnumArg(const char* funcName,
-                                       const char* argName, GLenum val) const {
+std::string EnumString(const GLenum val) {
+  const char* name = GetEnumName(val, nullptr);
+  if (name) {
+    return name;
+  }
+
+  const nsPrintfCString hex("<enum 0x%04x>", val);
+  return hex.BeginReading();
+}
+
+void WebGLContext::ErrorInvalidEnumArg(const char* argName, GLenum val) const {
   nsCString enumName;
   EnumName(val, &enumName);
-  ErrorInvalidEnum("%s: Bad `%s`: %s", funcName, argName,
-                   enumName.BeginReading());
-}
-
-bool IsCompressedTextureFormat(GLenum format) {
-  switch (format) {
-    case LOCAL_GL_COMPRESSED_RGB_S3TC_DXT1_EXT:
-    case LOCAL_GL_COMPRESSED_RGBA_S3TC_DXT1_EXT:
-    case LOCAL_GL_COMPRESSED_RGBA_S3TC_DXT3_EXT:
-    case LOCAL_GL_COMPRESSED_RGBA_S3TC_DXT5_EXT:
-    case LOCAL_GL_COMPRESSED_SRGB_S3TC_DXT1_EXT:
-    case LOCAL_GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT:
-    case LOCAL_GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT:
-    case LOCAL_GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT:
-    case LOCAL_GL_ATC_RGB:
-    case LOCAL_GL_ATC_RGBA_EXPLICIT_ALPHA:
-    case LOCAL_GL_ATC_RGBA_INTERPOLATED_ALPHA:
-    case LOCAL_GL_COMPRESSED_RGB_PVRTC_4BPPV1:
-    case LOCAL_GL_COMPRESSED_RGB_PVRTC_2BPPV1:
-    case LOCAL_GL_COMPRESSED_RGBA_PVRTC_4BPPV1:
-    case LOCAL_GL_COMPRESSED_RGBA_PVRTC_2BPPV1:
-    case LOCAL_GL_ETC1_RGB8_OES:
-    case LOCAL_GL_COMPRESSED_R11_EAC:
-    case LOCAL_GL_COMPRESSED_SIGNED_R11_EAC:
-    case LOCAL_GL_COMPRESSED_RG11_EAC:
-    case LOCAL_GL_COMPRESSED_SIGNED_RG11_EAC:
-    case LOCAL_GL_COMPRESSED_RGB8_ETC2:
-    case LOCAL_GL_COMPRESSED_SRGB8_ETC2:
-    case LOCAL_GL_COMPRESSED_RGB8_PUNCHTHROUGH_ALPHA1_ETC2:
-    case LOCAL_GL_COMPRESSED_SRGB8_PUNCHTHROUGH_ALPHA1_ETC2:
-    case LOCAL_GL_COMPRESSED_RGBA8_ETC2_EAC:
-    case LOCAL_GL_COMPRESSED_SRGB8_ALPHA8_ETC2_EAC:
-      return true;
-    default:
-      return false;
-  }
-}
-
-bool IsTextureFormatCompressed(TexInternalFormat format) {
-  return IsCompressedTextureFormat(format.get());
-}
-
-GLenum WebGLContext::GetAndFlushUnderlyingGLErrors() const {
-  // Get and clear GL error in ALL cases.
-  GLenum error = gl->fGetError();
-
-  // Only store in mUnderlyingGLError if is hasn't already recorded an
-  // error.
-  if (!mUnderlyingGLError) mUnderlyingGLError = error;
-
-  return error;
+  ErrorInvalidEnum("Bad `%s`: %s", argName, enumName.BeginReading());
 }
 
 #ifdef DEBUG
@@ -686,12 +635,12 @@ void AssertUintParamCorrect(gl::GLContext*, GLenum, GLuint) {}
 
 void WebGLContext::AssertCachedBindings() const {
 #ifdef DEBUG
-  GetAndFlushUnderlyingGLErrors();
+  gl::GLContext::LocalErrorScope errorScope(*gl);
 
   if (IsWebGL2() ||
       IsExtensionEnabled(WebGLExtensionID::OES_vertex_array_object)) {
-    GLuint bound = mBoundVertexArray ? mBoundVertexArray->GLName() : 0;
-    AssertUintParamCorrect(gl, LOCAL_GL_VERTEX_ARRAY_BINDING, bound);
+    AssertUintParamCorrect(gl, LOCAL_GL_VERTEX_ARRAY_BINDING,
+                           mBoundVertexArray->mGLName);
   }
 
   GLint stencilBits = 0;
@@ -729,7 +678,7 @@ void WebGLContext::AssertCachedBindings() const {
   bound = curBuff ? curBuff->mGLName : 0;
   AssertUintParamCorrect(gl, LOCAL_GL_ELEMENT_ARRAY_BUFFER_BINDING, bound);
 
-  MOZ_ASSERT(!GetAndFlushUnderlyingGLErrors());
+  MOZ_ASSERT(!gl::GLContext::IsBadCallError(errorScope.GetError()));
 #endif
 
   // We do not check the renderbuffer binding, because we never rely on it
@@ -738,7 +687,7 @@ void WebGLContext::AssertCachedBindings() const {
 
 void WebGLContext::AssertCachedGlobalState() const {
 #ifdef DEBUG
-  GetAndFlushUnderlyingGLErrors();
+  gl::GLContext::LocalErrorScope errorScope(*gl);
 
   ////////////////
 
@@ -809,7 +758,7 @@ void WebGLContext::AssertCachedGlobalState() const {
                            mPixelStore_PackSkipPixels);
   }
 
-  MOZ_ASSERT(!GetAndFlushUnderlyingGLErrors());
+  MOZ_ASSERT(!gl::GLContext::IsBadCallError(errorScope.GetError()));
 #endif
 }
 
