@@ -26,16 +26,16 @@
  */
 
 #ifndef WEBGLTEXELCONVERSIONS_H_
-#define WEBGLTEXELCONVERSIONS_H_
+#  define WEBGLTEXELCONVERSIONS_H_
 
-#ifdef __SUNPRO_CC
-#define __restrict
-#endif
+#  ifdef __SUNPRO_CC
+#    define __restrict
+#  endif
 
-#include "WebGLTypes.h"
-#include <stdint.h>
-#include "mozilla/Attributes.h"
-#include "mozilla/Casting.h"
+#  include "WebGLTypes.h"
+#  include <stdint.h>
+#  include "mozilla/Attributes.h"
+#  include "mozilla/Casting.h"
 
 namespace mozilla {
 
@@ -521,9 +521,9 @@ MOZ_ALWAYS_INLINE void unpack<WebGLTexelFormat::RGB565, uint16_t, uint8_t>(
   uint8_t r = (packedValue >> 11) & 0x1F;
   uint8_t g = (packedValue >> 5) & 0x3F;
   uint8_t b = packedValue & 0x1F;
-  dst[0] = (r << 3) | (r & 0x7);
-  dst[1] = (g << 2) | (g & 0x3);
-  dst[2] = (b << 3) | (b & 0x7);
+  dst[0] = uint8_t(r << 3) | (r & 0x7);
+  dst[1] = uint8_t(g << 2) | (g & 0x3);
+  dst[2] = uint8_t(b << 3) | (b & 0x7);
   dst[3] = 0xFF;
 }
 
@@ -564,10 +564,10 @@ MOZ_ALWAYS_INLINE void unpack<WebGLTexelFormat::RGBA4444, uint16_t, uint8_t>(
   uint8_t g = (packedValue >> 8) & 0x0F;
   uint8_t b = (packedValue >> 4) & 0x0F;
   uint8_t a = packedValue & 0x0F;
-  dst[0] = (r << 4) | r;
-  dst[1] = (g << 4) | g;
-  dst[2] = (b << 4) | b;
-  dst[3] = (a << 4) | a;
+  dst[0] = uint8_t(r << 4) | r;
+  dst[1] = uint8_t(g << 4) | g;
+  dst[2] = uint8_t(b << 4) | b;
+  dst[3] = uint8_t(a << 4) | a;
 }
 
 template <>
@@ -577,9 +577,9 @@ MOZ_ALWAYS_INLINE void unpack<WebGLTexelFormat::RGBA5551, uint16_t, uint8_t>(
   uint8_t r = (packedValue >> 11) & 0x1F;
   uint8_t g = (packedValue >> 6) & 0x1F;
   uint8_t b = (packedValue >> 1) & 0x1F;
-  dst[0] = (r << 3) | (r & 0x7);
-  dst[1] = (g << 3) | (g & 0x7);
-  dst[2] = (b << 3) | (b & 0x7);
+  dst[0] = uint8_t(r << 3) | (r & 0x7);
+  dst[1] = uint8_t(g << 3) | (g & 0x7);
+  dst[2] = uint8_t(b << 3) | (b & 0x7);
   dst[3] = (packedValue & 0x1) ? 0xFF : 0;
 }
 
@@ -961,8 +961,8 @@ template <>
 MOZ_ALWAYS_INLINE void
 pack<WebGLTexelFormat::RGB565, WebGLTexelPremultiplicationOp::None, uint8_t,
      uint16_t>(const uint8_t* __restrict src, uint16_t* __restrict dst) {
-  *dst = (((src[0] & 0xF8) << 8) | ((src[1] & 0xFC) << 3) |
-          ((src[2] & 0xF8) >> 3));
+  *dst = uint16_t(((src[0] & 0xF8) << 8) | ((src[1] & 0xFC) << 3) |
+                  ((src[2] & 0xF8) >> 3));
 }
 
 template <>
@@ -974,7 +974,8 @@ pack<WebGLTexelFormat::RGB565, WebGLTexelPremultiplicationOp::Premultiply,
   uint8_t srcR = static_cast<uint8_t>(src[0] * scaleFactor);
   uint8_t srcG = static_cast<uint8_t>(src[1] * scaleFactor);
   uint8_t srcB = static_cast<uint8_t>(src[2] * scaleFactor);
-  *dst = (((srcR & 0xF8) << 8) | ((srcG & 0xFC) << 3) | ((srcB & 0xF8) >> 3));
+  *dst = uint16_t(((srcR & 0xF8) << 8) | ((srcG & 0xFC) << 3) |
+                  ((srcB & 0xF8) >> 3));
 }
 
 // FIXME: this routine is lossy and must be removed.
@@ -987,7 +988,8 @@ pack<WebGLTexelFormat::RGB565, WebGLTexelPremultiplicationOp::Unpremultiply,
   uint8_t srcR = static_cast<uint8_t>(src[0] * scaleFactor);
   uint8_t srcG = static_cast<uint8_t>(src[1] * scaleFactor);
   uint8_t srcB = static_cast<uint8_t>(src[2] * scaleFactor);
-  *dst = (((srcR & 0xF8) << 8) | ((srcG & 0xFC) << 3) | ((srcB & 0xF8) >> 3));
+  *dst = uint16_t(((srcR & 0xF8) << 8) | ((srcG & 0xFC) << 3) |
+                  ((srcB & 0xF8) >> 3));
 }
 
 template <>
@@ -1121,8 +1123,8 @@ template <>
 MOZ_ALWAYS_INLINE void
 pack<WebGLTexelFormat::RGBA4444, WebGLTexelPremultiplicationOp::None, uint8_t,
      uint16_t>(const uint8_t* __restrict src, uint16_t* __restrict dst) {
-  *dst = (((src[0] & 0xF0) << 8) | ((src[1] & 0xF0) << 4) | (src[2] & 0xF0) |
-          (src[3] >> 4));
+  *dst = uint16_t(((src[0] & 0xF0) << 8) | ((src[1] & 0xF0) << 4) |
+                  (src[2] & 0xF0) | (src[3] >> 4));
 }
 
 template <>
@@ -1134,8 +1136,8 @@ pack<WebGLTexelFormat::RGBA4444, WebGLTexelPremultiplicationOp::Premultiply,
   uint8_t srcR = static_cast<uint8_t>(src[0] * scaleFactor);
   uint8_t srcG = static_cast<uint8_t>(src[1] * scaleFactor);
   uint8_t srcB = static_cast<uint8_t>(src[2] * scaleFactor);
-  *dst = (((srcR & 0xF0) << 8) | ((srcG & 0xF0) << 4) | (srcB & 0xF0) |
-          (src[3] >> 4));
+  *dst = uint16_t(((srcR & 0xF0) << 8) | ((srcG & 0xF0) << 4) | (srcB & 0xF0) |
+                  (src[3] >> 4));
 }
 
 // FIXME: this routine is lossy and must be removed.
@@ -1148,16 +1150,16 @@ pack<WebGLTexelFormat::RGBA4444, WebGLTexelPremultiplicationOp::Unpremultiply,
   uint8_t srcR = static_cast<uint8_t>(src[0] * scaleFactor);
   uint8_t srcG = static_cast<uint8_t>(src[1] * scaleFactor);
   uint8_t srcB = static_cast<uint8_t>(src[2] * scaleFactor);
-  *dst = (((srcR & 0xF0) << 8) | ((srcG & 0xF0) << 4) | (srcB & 0xF0) |
-          (src[3] >> 4));
+  *dst = uint16_t(((srcR & 0xF0) << 8) | ((srcG & 0xF0) << 4) | (srcB & 0xF0) |
+                  (src[3] >> 4));
 }
 
 template <>
 MOZ_ALWAYS_INLINE void
 pack<WebGLTexelFormat::RGBA5551, WebGLTexelPremultiplicationOp::None, uint8_t,
      uint16_t>(const uint8_t* __restrict src, uint16_t* __restrict dst) {
-  *dst = (((src[0] & 0xF8) << 8) | ((src[1] & 0xF8) << 3) |
-          ((src[2] & 0xF8) >> 2) | (src[3] >> 7));
+  *dst = uint16_t(((src[0] & 0xF8) << 8) | ((src[1] & 0xF8) << 3) |
+                  ((src[2] & 0xF8) >> 2) | (src[3] >> 7));
 }
 
 template <>
@@ -1169,8 +1171,8 @@ pack<WebGLTexelFormat::RGBA5551, WebGLTexelPremultiplicationOp::Premultiply,
   uint8_t srcR = static_cast<uint8_t>(src[0] * scaleFactor);
   uint8_t srcG = static_cast<uint8_t>(src[1] * scaleFactor);
   uint8_t srcB = static_cast<uint8_t>(src[2] * scaleFactor);
-  *dst = (((srcR & 0xF8) << 8) | ((srcG & 0xF8) << 3) | ((srcB & 0xF8) >> 2) |
-          (src[3] >> 7));
+  *dst = uint16_t(((srcR & 0xF8) << 8) | ((srcG & 0xF8) << 3) |
+                  ((srcB & 0xF8) >> 2) | (src[3] >> 7));
 }
 
 // FIXME: this routine is lossy and must be removed.
@@ -1183,8 +1185,8 @@ pack<WebGLTexelFormat::RGBA5551, WebGLTexelPremultiplicationOp::Unpremultiply,
   uint8_t srcR = static_cast<uint8_t>(src[0] * scaleFactor);
   uint8_t srcG = static_cast<uint8_t>(src[1] * scaleFactor);
   uint8_t srcB = static_cast<uint8_t>(src[2] * scaleFactor);
-  *dst = (((srcR & 0xF8) << 8) | ((srcG & 0xF8) << 3) | ((srcB & 0xF8) >> 2) |
-          (src[3] >> 7));
+  *dst = uint16_t(((srcR & 0xF8) << 8) | ((srcG & 0xF8) << 3) |
+                  ((srcB & 0xF8) >> 2) | (src[3] >> 7));
 }
 
 template <>
