@@ -5,7 +5,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "mozilla/dom/HTMLTableElement.h"
-#include "mozilla/GenericSpecifiedValuesInlines.h"
+#include "mozilla/MappedDeclarations.h"
 #include "nsAttrValueInlines.h"
 #include "nsHTMLStyleSheet.h"
 #include "nsMappedAttributes.h"
@@ -164,9 +164,9 @@ void TableRowsCollection::EnsureInitialized() {
   }
 
   mBodyStart = mRows.Length();
-  mRows.AppendElements(Move(body));
+  mRows.AppendElements(std::move(body));
   mFootStart = mRows.Length();
-  mRows.AppendElements(Move(foot));
+  mRows.AppendElements(std::move(foot));
 
   mParent->AddMutationObserver(this);
 }
@@ -191,7 +191,7 @@ void TableRowsCollection::CleanUp() {
 
 JSObject* TableRowsCollection::WrapObject(JSContext* aCx,
                                           JS::Handle<JSObject*> aGivenProto) {
-  return HTMLCollectionBinding::Wrap(aCx, this, aGivenProto);
+  return HTMLCollection_Binding::Wrap(aCx, this, aGivenProto);
 }
 
 NS_IMPL_CYCLE_COLLECTION_WRAPPERCACHE(TableRowsCollection, mRows)
@@ -251,7 +251,7 @@ void TableRowsCollection::GetSupportedNames(nsTArray<nsString>& aNames) {
       }
     }
 
-    nsGenericHTMLElement* el = nsGenericHTMLElement::FromContent(node);
+    nsGenericHTMLElement* el = nsGenericHTMLElement::FromNode(node);
     if (el) {
       const nsAttrValue* val = el->GetParsedAttr(nsGkAtoms::name);
       if (val && val->Type() == nsAttrValue::eAtom) {
@@ -499,8 +499,9 @@ void TableRowsCollection::NodeWillBeDestroyed(const nsINode* aNode) {
 /* --------------------------- HTMLTableElement ---------------------------- */
 
 HTMLTableElement::HTMLTableElement(
-    already_AddRefed<mozilla::dom::NodeInfo>& aNodeInfo)
-    : nsGenericHTMLElement(aNodeInfo), mTableInheritedAttributes(nullptr) {
+    already_AddRefed<mozilla::dom::NodeInfo>&& aNodeInfo)
+    : nsGenericHTMLElement(std::move(aNodeInfo)),
+      mTableInheritedAttributes(nullptr) {
   SetHasWeirdParserInsertionMode();
 }
 
@@ -513,7 +514,7 @@ HTMLTableElement::~HTMLTableElement() {
 
 JSObject* HTMLTableElement::WrapNode(JSContext* aCx,
                                      JS::Handle<JSObject*> aGivenProto) {
-  return HTMLTableElementBinding::Wrap(aCx, this, aGivenProto);
+  return HTMLTableElement_Binding::Wrap(aCx, this, aGivenProto);
 }
 
 NS_IMPL_CYCLE_COLLECTION_CLASS(HTMLTableElement)
@@ -588,11 +589,10 @@ already_AddRefed<nsGenericHTMLElement> HTMLTableElement::CreateTHead() {
 }
 
 void HTMLTableElement::DeleteTHead() {
-  HTMLTableSectionElement* tHead = GetTHead();
+  RefPtr<HTMLTableSectionElement> tHead = GetTHead();
   if (tHead) {
-    mozilla::ErrorResult rv;
+    mozilla::IgnoredErrorResult rv;
     nsINode::RemoveChild(*tHead, rv);
-    MOZ_ASSERT(!rv.Failed());
   }
 }
 
@@ -615,11 +615,10 @@ already_AddRefed<nsGenericHTMLElement> HTMLTableElement::CreateTFoot() {
 }
 
 void HTMLTableElement::DeleteTFoot() {
-  HTMLTableSectionElement* tFoot = GetTFoot();
+  RefPtr<HTMLTableSectionElement> tFoot = GetTFoot();
   if (tFoot) {
-    mozilla::ErrorResult rv;
+    mozilla::IgnoredErrorResult rv;
     nsINode::RemoveChild(*tFoot, rv);
-    MOZ_ASSERT(!rv.Failed());
   }
 }
 
@@ -643,11 +642,10 @@ already_AddRefed<nsGenericHTMLElement> HTMLTableElement::CreateCaption() {
 }
 
 void HTMLTableElement::DeleteCaption() {
-  HTMLTableCaptionElement* caption = GetCaption();
+  RefPtr<HTMLTableCaptionElement> caption = GetCaption();
   if (caption) {
-    mozilla::ErrorResult rv;
+    mozilla::IgnoredErrorResult rv;
     nsINode::RemoveChild(*caption, rv);
-    MOZ_ASSERT(!rv.Failed());
   }
 }
 
@@ -851,111 +849,105 @@ bool HTMLTableElement::ParseAttribute(int32_t aNamespaceID, nsAtom* aAttribute,
 }
 
 void HTMLTableElement::MapAttributesIntoRule(
-    const nsMappedAttributes* aAttributes, GenericSpecifiedValues* aData) {
+    const nsMappedAttributes* aAttributes, MappedDeclarations& aDecls) {
   // XXX Bug 211636:  This function is used by a single style rule
   // that's used to match two different type of elements -- tables, and
   // table cells.  (nsHTMLTableCellElement overrides
   // WalkContentStyleRules so that this happens.)  This violates the
   // nsIStyleRule contract, since it's the same style rule object doing
   // the mapping in two different ways.  It's also incorrect since it's
-  // testing the display type of the style context rather than checking
+  // testing the display type of the ComputedStyle rather than checking
   // which *element* it's matching (style rules should not stop matching
   // when the display type is changed).
 
-  nsCompatibility mode = aData->Document()->GetCompatibilityMode();
+  nsCompatibility mode = aDecls.Document()->GetCompatibilityMode();
 
-  if (aData->ShouldComputeStyleStruct(NS_STYLE_INHERIT_BIT(TableBorder))) {
-    // cellspacing
-    const nsAttrValue* value = aAttributes->GetAttr(nsGkAtoms::cellspacing);
-    if (value && value->Type() == nsAttrValue::eInteger &&
-        !aData->PropertyIsSet(eCSSProperty_border_spacing)) {
-      aData->SetPixelValue(eCSSProperty_border_spacing,
-                           float(value->GetIntegerValue()));
+  // cellspacing
+  const nsAttrValue* value = aAttributes->GetAttr(nsGkAtoms::cellspacing);
+  if (value && value->Type() == nsAttrValue::eInteger &&
+      !aDecls.PropertyIsSet(eCSSProperty_border_spacing)) {
+    aDecls.SetPixelValue(eCSSProperty_border_spacing,
+                         float(value->GetIntegerValue()));
+  }
+  // align; Check for enumerated type (it may be another type if
+  // illegal)
+  value = aAttributes->GetAttr(nsGkAtoms::align);
+  if (value && value->Type() == nsAttrValue::eEnum) {
+    if (value->GetEnumValue() == NS_STYLE_TEXT_ALIGN_CENTER ||
+        value->GetEnumValue() == NS_STYLE_TEXT_ALIGN_MOZ_CENTER) {
+      aDecls.SetAutoValueIfUnset(eCSSProperty_margin_left);
+      aDecls.SetAutoValueIfUnset(eCSSProperty_margin_right);
     }
   }
-  if (aData->ShouldComputeStyleStruct(NS_STYLE_INHERIT_BIT(Margin))) {
-    // align; Check for enumerated type (it may be another type if
-    // illegal)
-    const nsAttrValue* value = aAttributes->GetAttr(nsGkAtoms::align);
 
-    if (value && value->Type() == nsAttrValue::eEnum) {
-      if (value->GetEnumValue() == NS_STYLE_TEXT_ALIGN_CENTER ||
-          value->GetEnumValue() == NS_STYLE_TEXT_ALIGN_MOZ_CENTER) {
-        aData->SetAutoValueIfUnset(eCSSProperty_margin_left);
-        aData->SetAutoValueIfUnset(eCSSProperty_margin_right);
-      }
+  // hspace is mapped into left and right margin,
+  // vspace is mapped into top and bottom margins
+  // - *** Quirks Mode only ***
+  if (eCompatibility_NavQuirks == mode) {
+    value = aAttributes->GetAttr(nsGkAtoms::hspace);
+
+    if (value && value->Type() == nsAttrValue::eInteger) {
+      aDecls.SetPixelValueIfUnset(eCSSProperty_margin_left,
+                                  (float)value->GetIntegerValue());
+      aDecls.SetPixelValueIfUnset(eCSSProperty_margin_right,
+                                  (float)value->GetIntegerValue());
     }
 
-    // hspace is mapped into left and right margin,
-    // vspace is mapped into top and bottom margins
-    // - *** Quirks Mode only ***
-    if (eCompatibility_NavQuirks == mode) {
-      value = aAttributes->GetAttr(nsGkAtoms::hspace);
+    value = aAttributes->GetAttr(nsGkAtoms::vspace);
 
-      if (value && value->Type() == nsAttrValue::eInteger) {
-        aData->SetPixelValueIfUnset(eCSSProperty_margin_left,
-                                    (float)value->GetIntegerValue());
-        aData->SetPixelValueIfUnset(eCSSProperty_margin_right,
-                                    (float)value->GetIntegerValue());
-      }
-
-      value = aAttributes->GetAttr(nsGkAtoms::vspace);
-
-      if (value && value->Type() == nsAttrValue::eInteger) {
-        aData->SetPixelValueIfUnset(eCSSProperty_margin_top,
-                                    (float)value->GetIntegerValue());
-        aData->SetPixelValueIfUnset(eCSSProperty_margin_bottom,
-                                    (float)value->GetIntegerValue());
-      }
+    if (value && value->Type() == nsAttrValue::eInteger) {
+      aDecls.SetPixelValueIfUnset(eCSSProperty_margin_top,
+                                  (float)value->GetIntegerValue());
+      aDecls.SetPixelValueIfUnset(eCSSProperty_margin_bottom,
+                                  (float)value->GetIntegerValue());
     }
   }
-  if (aData->ShouldComputeStyleStruct(NS_STYLE_INHERIT_BIT(Border))) {
-    // bordercolor
-    const nsAttrValue* value = aAttributes->GetAttr(nsGkAtoms::bordercolor);
-    nscolor color;
-    if (value && !aData->ShouldIgnoreColors() && value->GetColorValue(color)) {
-      aData->SetColorValueIfUnset(eCSSProperty_border_top_color, color);
-      aData->SetColorValueIfUnset(eCSSProperty_border_left_color, color);
-      aData->SetColorValueIfUnset(eCSSProperty_border_bottom_color, color);
-      aData->SetColorValueIfUnset(eCSSProperty_border_right_color, color);
-    }
-
-    // border
-    const nsAttrValue* borderValue = aAttributes->GetAttr(nsGkAtoms::border);
-    if (borderValue) {
-      // border = 1 pixel default
-      int32_t borderThickness = 1;
-
-      if (borderValue->Type() == nsAttrValue::eInteger)
-        borderThickness = borderValue->GetIntegerValue();
-
-      // by default, set all border sides to the specified width
-      aData->SetPixelValueIfUnset(eCSSProperty_border_top_width,
-                                  (float)borderThickness);
-      aData->SetPixelValueIfUnset(eCSSProperty_border_left_width,
-                                  (float)borderThickness);
-      aData->SetPixelValueIfUnset(eCSSProperty_border_bottom_width,
-                                  (float)borderThickness);
-      aData->SetPixelValueIfUnset(eCSSProperty_border_right_width,
-                                  (float)borderThickness);
-    }
+  // bordercolor
+  value = aAttributes->GetAttr(nsGkAtoms::bordercolor);
+  nscolor color;
+  if (value && value->GetColorValue(color)) {
+    aDecls.SetColorValueIfUnset(eCSSProperty_border_top_color, color);
+    aDecls.SetColorValueIfUnset(eCSSProperty_border_left_color, color);
+    aDecls.SetColorValueIfUnset(eCSSProperty_border_bottom_color, color);
+    aDecls.SetColorValueIfUnset(eCSSProperty_border_right_color, color);
   }
-  nsGenericHTMLElement::MapImageSizeAttributesInto(aAttributes, aData);
-  nsGenericHTMLElement::MapBackgroundAttributesInto(aAttributes, aData);
-  nsGenericHTMLElement::MapCommonAttributesInto(aAttributes, aData);
+
+  // border
+  const nsAttrValue* borderValue = aAttributes->GetAttr(nsGkAtoms::border);
+  if (borderValue) {
+    // border = 1 pixel default
+    int32_t borderThickness = 1;
+
+    if (borderValue->Type() == nsAttrValue::eInteger)
+      borderThickness = borderValue->GetIntegerValue();
+
+    // by default, set all border sides to the specified width
+    aDecls.SetPixelValueIfUnset(eCSSProperty_border_top_width,
+                                (float)borderThickness);
+    aDecls.SetPixelValueIfUnset(eCSSProperty_border_left_width,
+                                (float)borderThickness);
+    aDecls.SetPixelValueIfUnset(eCSSProperty_border_bottom_width,
+                                (float)borderThickness);
+    aDecls.SetPixelValueIfUnset(eCSSProperty_border_right_width,
+                                (float)borderThickness);
+  }
+
+  nsGenericHTMLElement::MapImageSizeAttributesInto(aAttributes, aDecls);
+  nsGenericHTMLElement::MapBackgroundAttributesInto(aAttributes, aDecls);
+  nsGenericHTMLElement::MapCommonAttributesInto(aAttributes, aDecls);
 }
 
 NS_IMETHODIMP_(bool)
 HTMLTableElement::IsAttributeMapped(const nsAtom* aAttribute) const {
   static const MappedAttributeEntry attributes[] = {
-      {&nsGkAtoms::cellpadding}, {&nsGkAtoms::cellspacing},
-      {&nsGkAtoms::border},      {&nsGkAtoms::width},
-      {&nsGkAtoms::height},      {&nsGkAtoms::hspace},
-      {&nsGkAtoms::vspace},
+      {nsGkAtoms::cellpadding}, {nsGkAtoms::cellspacing},
+      {nsGkAtoms::border},      {nsGkAtoms::width},
+      {nsGkAtoms::height},      {nsGkAtoms::hspace},
+      {nsGkAtoms::vspace},
 
-      {&nsGkAtoms::bordercolor},
+      {nsGkAtoms::bordercolor},
 
-      {&nsGkAtoms::align},       {nullptr}};
+      {nsGkAtoms::align},       {nullptr}};
 
   static const MappedAttributeEntry* const map[] = {
       attributes,
@@ -972,19 +964,17 @@ nsMapRuleToAttributesFunc HTMLTableElement::GetAttributeMappingFunction()
 }
 
 static void MapInheritedTableAttributesIntoRule(
-    const nsMappedAttributes* aAttributes, GenericSpecifiedValues* aData) {
-  if (aData->ShouldComputeStyleStruct(NS_STYLE_INHERIT_BIT(Padding))) {
-    const nsAttrValue* value = aAttributes->GetAttr(nsGkAtoms::cellpadding);
-    if (value && value->Type() == nsAttrValue::eInteger) {
-      // We have cellpadding.  This will override our padding values if we
-      // don't have any set.
-      float pad = float(value->GetIntegerValue());
+    const nsMappedAttributes* aAttributes, MappedDeclarations& aDecls) {
+  const nsAttrValue* value = aAttributes->GetAttr(nsGkAtoms::cellpadding);
+  if (value && value->Type() == nsAttrValue::eInteger) {
+    // We have cellpadding.  This will override our padding values if we
+    // don't have any set.
+    float pad = float(value->GetIntegerValue());
 
-      aData->SetPixelValueIfUnset(eCSSProperty_padding_top, pad);
-      aData->SetPixelValueIfUnset(eCSSProperty_padding_right, pad);
-      aData->SetPixelValueIfUnset(eCSSProperty_padding_bottom, pad);
-      aData->SetPixelValueIfUnset(eCSSProperty_padding_left, pad);
-    }
+    aDecls.SetPixelValueIfUnset(eCSSProperty_padding_top, pad);
+    aDecls.SetPixelValueIfUnset(eCSSProperty_padding_right, pad);
+    aDecls.SetPixelValueIfUnset(eCSSProperty_padding_bottom, pad);
+    aDecls.SetPixelValueIfUnset(eCSSProperty_padding_left, pad);
   }
 }
 
@@ -996,13 +986,12 @@ void HTMLTableElement::BuildInheritedAttributes() {
   NS_ASSERTION(!mTableInheritedAttributes,
                "potential leak, plus waste of work");
   MOZ_ASSERT(NS_IsMainThread());
-  nsIDocument* document = GetComposedDoc();
+  Document* document = GetComposedDoc();
   nsHTMLStyleSheet* sheet =
       document ? document->GetAttributeStyleSheet() : nullptr;
   RefPtr<nsMappedAttributes> newAttrs;
   if (sheet) {
-    const nsAttrValue* value =
-        mAttrsAndChildren.GetAttr(nsGkAtoms::cellpadding);
+    const nsAttrValue* value = mAttrs.GetAttr(nsGkAtoms::cellpadding);
     if (value) {
       RefPtr<nsMappedAttributes> modifiableMapped =
           new nsMappedAttributes(sheet, MapInheritedTableAttributesIntoRule);
@@ -1034,13 +1023,11 @@ void HTMLTableElement::ReleaseInheritedAttributes() {
   NS_IF_RELEASE(mTableInheritedAttributes);
 }
 
-nsresult HTMLTableElement::BindToTree(nsIDocument* aDocument,
-                                      nsIContent* aParent,
-                                      nsIContent* aBindingParent,
-                                      bool aCompileEventHandlers) {
+nsresult HTMLTableElement::BindToTree(Document* aDocument, nsIContent* aParent,
+                                      nsIContent* aBindingParent) {
   ReleaseInheritedAttributes();
-  nsresult rv = nsGenericHTMLElement::BindToTree(
-      aDocument, aParent, aBindingParent, aCompileEventHandlers);
+  nsresult rv =
+      nsGenericHTMLElement::BindToTree(aDocument, aParent, aBindingParent);
   NS_ENSURE_SUCCESS(rv, rv);
   BuildInheritedAttributes();
   return NS_OK;
