@@ -39,12 +39,13 @@
 #include "mozilla/ipc/InputStreamUtils.h"
 #include "nsAutoPtr.h"
 #include "nsCOMPtr.h"
-#include "nsIDocument.h"
+#include "mozilla/dom/Document.h"
 #include "nsIObserver.h"
 #include "nsIObserverService.h"
 #include "nsIScriptError.h"
 #include "nsISupportsPrimitives.h"
 #include "nsThreadUtils.h"
+#include "nsIWeakReferenceUtils.h"
 #include "ProfilerHelpers.h"
 #include "ReportInternalError.h"
 #include "ScriptErrorHelper.h"
@@ -149,7 +150,7 @@ class IDBDatabase::Observer final : public nsIObserver {
 
 IDBDatabase::IDBDatabase(IDBOpenDBRequest* aRequest, IDBFactory* aFactory,
                          BackgroundDatabaseChild* aActor, DatabaseSpec* aSpec)
-    : IDBWrapperCache(aRequest),
+    : DOMEventTargetHelper(aRequest),
       mFactory(aFactory),
       mSpec(aSpec),
       mBackgroundActor(aActor),
@@ -183,10 +184,10 @@ already_AddRefed<IDBDatabase> IDBDatabase::Create(
 
   RefPtr<IDBDatabase> db = new IDBDatabase(aRequest, aFactory, aActor, aSpec);
 
-  db->SetScriptOwner(aRequest->GetScriptOwner());
-
   if (NS_IsMainThread()) {
-    if (nsPIDOMWindowInner* window = aFactory->GetParentObject()) {
+    nsCOMPtr<nsPIDOMWindowInner> window =
+        do_QueryInterface(aFactory->GetParentObject());
+    if (window) {
       uint64_t windowId = window->WindowID();
 
       RefPtr<Observer> observer = new Observer(db, windowId);
@@ -317,10 +318,6 @@ void IDBDatabase::RefreshSpec(bool aMayDelete) {
   }
 }
 
-nsPIDOMWindowInner* IDBDatabase::GetParentObject() const {
-  return mFactory->GetParentObject();
-}
-
 const nsString& IDBDatabase::Name() const {
   AssertIsOnOwningThread();
   MOZ_ASSERT(mSpec);
@@ -355,9 +352,9 @@ already_AddRefed<DOMStringList> IDBDatabase::ObjectStoreNames() const {
   return list.forget();
 }
 
-already_AddRefed<nsIDocument> IDBDatabase::GetOwnerDocument() const {
+already_AddRefed<Document> IDBDatabase::GetOwnerDocument() const {
   if (nsPIDOMWindowInner* window = GetOwner()) {
-    nsCOMPtr<nsIDocument> doc = window->GetExtantDoc();
+    nsCOMPtr<Document> doc = window->GetExtantDoc();
     return doc.forget();
   }
   return nullptr;
@@ -390,7 +387,11 @@ already_AddRefed<IDBObjectStore> IDBDatabase::CreateObjectStore(
   for (uint32_t count = objectStores.Length(), index = 0; index < count;
        index++) {
     if (aName == objectStores[index].metadata().name()) {
-      aRv.Throw(NS_ERROR_DOM_INDEXEDDB_CONSTRAINT_ERR);
+      aRv.ThrowDOMException(
+          NS_ERROR_DOM_INDEXEDDB_CONSTRAINT_ERR,
+          nsPrintfCString(
+              "Object store named '%s' already exists at index '%u'",
+              NS_ConvertUTF16toUTF8(aName).get(), index));
       return nullptr;
     }
   }
@@ -662,6 +663,11 @@ already_AddRefed<IDBRequest> IDBDatabase::CreateMutableFile(
     ErrorResult& aRv) {
   AssertIsOnOwningThread();
 
+  if (aName.IsEmpty()) {
+    aRv.Throw(NS_ERROR_DOM_SYNTAX_ERR);
+    return nullptr;
+  }
+
   if (QuotaManager::IsShuttingDown()) {
     IDB_REPORT_INTERNAL_ERR();
     aRv.Throw(NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
@@ -822,8 +828,7 @@ PBackgroundIDBDatabaseFileChild* IDBDatabase::GetOrCreateFileActorForBlob(
   // a) it is unique per blob, b) it is reference-counted so that we can
   // guarantee that it stays alive, and c) it doesn't hold the actual File
   // alive.
-  nsCOMPtr<nsIDOMBlob> blob = aBlob;
-  nsCOMPtr<nsIWeakReference> weakRef = do_GetWeakReference(blob);
+  nsWeakPtr weakRef = do_GetWeakReference(aBlob);
   MOZ_ASSERT(weakRef);
 
   PBackgroundIDBDatabaseFileChild* actor = nullptr;
@@ -966,7 +971,7 @@ void IDBDatabase::ExpireFileActors(bool aExpireAll) {
 
       bool shouldExpire = aExpireAll;
       if (!shouldExpire) {
-        nsCOMPtr<nsIWeakReference> weakRef = do_QueryInterface(key);
+        nsWeakPtr weakRef = do_QueryInterface(key);
         MOZ_ASSERT(weakRef);
 
         nsCOMPtr<nsISupports> referent = do_QueryReferent(weakRef);
@@ -1051,20 +1056,22 @@ void IDBDatabase::LogWarning(const char* aMessageName,
       mFactory->InnerWindowID());
 }
 
-NS_IMPL_ADDREF_INHERITED(IDBDatabase, IDBWrapperCache)
-NS_IMPL_RELEASE_INHERITED(IDBDatabase, IDBWrapperCache)
+NS_IMPL_ADDREF_INHERITED(IDBDatabase, DOMEventTargetHelper)
+NS_IMPL_RELEASE_INHERITED(IDBDatabase, DOMEventTargetHelper)
 
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(IDBDatabase)
-NS_INTERFACE_MAP_END_INHERITING(IDBWrapperCache)
+NS_INTERFACE_MAP_END_INHERITING(DOMEventTargetHelper)
 
 NS_IMPL_CYCLE_COLLECTION_CLASS(IDBDatabase)
 
-NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN_INHERITED(IDBDatabase, IDBWrapperCache)
+NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN_INHERITED(IDBDatabase,
+                                                  DOMEventTargetHelper)
   tmp->AssertIsOnOwningThread();
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mFactory)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
-NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN_INHERITED(IDBDatabase, IDBWrapperCache)
+NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN_INHERITED(IDBDatabase,
+                                                DOMEventTargetHelper)
   tmp->AssertIsOnOwningThread();
 
   // Don't unlink mFactory!
@@ -1074,11 +1081,18 @@ NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN_INHERITED(IDBDatabase, IDBWrapperCache)
   tmp->CloseInternal();
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
+void IDBDatabase::DisconnectFromOwner() {
+  InvalidateInternal();
+  DOMEventTargetHelper::DisconnectFromOwner();
+}
+
 void IDBDatabase::LastRelease() {
   AssertIsOnOwningThread();
 
   CloseInternal();
 
+  ExpireFileActors(/* aExpireAll */ true);
+
   if (mBackgroundActor) {
     mBackgroundActor->SendDeleteMeInternal();
     MOZ_ASSERT(!mBackgroundActor, "SendDeleteMeInternal should have cleared!");
@@ -1097,7 +1111,7 @@ nsresult IDBDatabase::PostHandleEvent(EventChainPostVisitor& aVisitor) {
 
 JSObject* IDBDatabase::WrapObject(JSContext* aCx,
                                   JS::Handle<JSObject*> aGivenProto) {
-  return IDBDatabaseBinding::Wrap(aCx, this, aGivenProto);
+  return IDBDatabase_Binding::Wrap(aCx, this, aGivenProto);
 }
 
 NS_IMETHODIMP
@@ -1181,7 +1195,7 @@ nsresult IDBDatabase::RenameObjectStore(int64_t aObjectStoreId,
       continue;
     }
     if (aName == objSpec.metadata().name()) {
-      return NS_ERROR_DOM_INDEXEDDB_CONSTRAINT_ERR;
+      return NS_ERROR_DOM_INDEXEDDB_RENAME_OBJECT_STORE_ERR;
     }
   }
 
@@ -1224,7 +1238,7 @@ nsresult IDBDatabase::RenameIndex(int64_t aObjectStoreId, int64_t aIndexId,
       continue;
     }
     if (aName == metadata.name()) {
-      return NS_ERROR_DOM_INDEXEDDB_CONSTRAINT_ERR;
+      return NS_ERROR_DOM_INDEXEDDB_RENAME_INDEX_ERR;
     }
   }
 
