@@ -7,16 +7,9 @@
 
 #include "nsContentUtils.h"
 
-#include "mozilla/dom/AudioStreamTrackBinding.h"
-
 namespace mozilla {
 namespace dom {
 
-JSObject* AudioStreamTrack::WrapObject(JSContext* aCx,
-                                       JS::Handle<JSObject*> aGivenProto) {
-  return AudioStreamTrackBinding::Wrap(aCx, this, aGivenProto);
-}
-
 void AudioStreamTrack::GetLabel(nsAString& aLabel, CallerType aCallerType) {
   if (nsContentUtils::ResistFingerprinting(aCallerType)) {
     aLabel.AssignLiteral("Internal Microphone");
