@@ -12,9 +12,10 @@
 namespace mozilla {
 namespace dom {
 
-MediaTrack::MediaTrack(const nsAString& aId, const nsAString& aKind,
-                       const nsAString& aLabel, const nsAString& aLanguage)
-    : DOMEventTargetHelper(),
+MediaTrack::MediaTrack(nsIGlobalObject* aOwnerGlobal, const nsAString& aId,
+                       const nsAString& aKind, const nsAString& aLabel,
+                       const nsAString& aLanguage)
+    : DOMEventTargetHelper(aOwnerGlobal),
       mId(aId),
       mKind(aKind),
       mLabel(aLabel),
@@ -31,9 +32,5 @@ NS_INTERFACE_MAP_END_INHERITING(DOMEventTargetHelper)
 
 void MediaTrack::SetTrackList(MediaTrackList* aList) { mList = aList; }
 
-void MediaTrack::Init(nsPIDOMWindowInner* aOwnerWindow) {
-  BindToOwner(aOwnerWindow);
-}
-
 }  // namespace dom
 }  // namespace mozilla
