@@ -20,7 +20,7 @@ void QueueObject::Dispatch(nsIRunnable* aRunnable) {
 }
 
 void QueueObject::Dispatch(already_AddRefed<nsIRunnable> aRunnable) {
-  mThread->Dispatch(Move(aRunnable));
+  mThread->Dispatch(std::move(aRunnable));
 }
 
 bool QueueObject::OnThread() { return mThread->IsCurrentThreadIn(); }
