@@ -37,9 +37,7 @@ TrackEncoder::TrackEncoder(TrackRate aTrackRate)
       mInitialized(false),
       mEndOfStream(false),
       mCanceled(false),
-      mCurrentTime(0),
       mInitCounter(0),
-      mNotInitDuration(0),
       mSuspended(false),
       mTrackRate(aTrackRate) {}
 
@@ -103,7 +101,7 @@ void TrackEncoder::SetWorkerThread(AbstractThread* aWorkerThread) {
   mWorkerThread = aWorkerThread;
 }
 
-void AudioTrackEncoder::Suspend(TimeStamp) {
+void AudioTrackEncoder::Suspend() {
   MOZ_ASSERT(!mWorkerThread || mWorkerThread->IsCurrentThreadIn());
   TRACK_LOG(LogLevel::Info, ("[AudioTrackEncoder %p]: Suspend(), was %s", this,
                              mSuspended ? "suspended" : "live"));
@@ -115,7 +113,7 @@ void AudioTrackEncoder::Suspend(TimeStamp) {
   mSuspended = true;
 }
 
-void AudioTrackEncoder::Resume(TimeStamp) {
+void AudioTrackEncoder::Resume() {
   MOZ_ASSERT(!mWorkerThread || mWorkerThread->IsCurrentThreadIn());
   TRACK_LOG(LogLevel::Info, ("[AudioTrackEncoder %p]: Resume(), was %s", this,
                              mSuspended ? "suspended" : "live"));
@@ -129,6 +127,7 @@ void AudioTrackEncoder::Resume(TimeStamp) {
 
 void AudioTrackEncoder::AppendAudioSegment(AudioSegment&& aSegment) {
   MOZ_ASSERT(!mWorkerThread || mWorkerThread->IsCurrentThreadIn());
+  AUTO_PROFILER_LABEL("AudioTrackEncoder::AppendAudioSegment", OTHER);
   TRACK_LOG(LogLevel::Verbose,
             ("[AudioTrackEncoder %p]: AppendAudioSegment() duration=%" PRIu64,
              this, aSegment.GetDuration()));
@@ -141,7 +140,15 @@ void AudioTrackEncoder::AppendAudioSegment(AudioSegment&& aSegment) {
     return;
   }
 
-  mIncomingBuffer.AppendFrom(&aSegment);
+  TryInit(mOutgoingBuffer, aSegment.GetDuration());
+
+  if (!mSuspended) {
+    mOutgoingBuffer.AppendFrom(&aSegment);
+  }
+
+  if (mInitialized && mOutgoingBuffer.GetDuration() >= GetPacketDuration()) {
+    OnDataAvailable();
+  }
 }
 
 void AudioTrackEncoder::TakeTrackData(AudioSegment& aSegment) {
@@ -216,20 +223,15 @@ void AudioTrackEncoder::TryInit(const AudioSegment& aSegment,
 
 void AudioTrackEncoder::Cancel() {
   MOZ_ASSERT(!mWorkerThread || mWorkerThread->IsCurrentThreadIn());
-  TRACK_LOG(LogLevel::Info,
-            ("[AudioTrackEncoder %p]: Cancel(), currentTime=%" PRIu64, this,
-             mCurrentTime));
+  TRACK_LOG(LogLevel::Info, ("[AudioTrackEncoder %p]: Cancel()", this));
   mCanceled = true;
-  mIncomingBuffer.Clear();
   mOutgoingBuffer.Clear();
 }
 
 void AudioTrackEncoder::NotifyEndOfStream() {
   MOZ_ASSERT(!mWorkerThread || mWorkerThread->IsCurrentThreadIn());
-  TRACK_LOG(
-      LogLevel::Info,
-      ("[AudioTrackEncoder %p]: NotifyEndOfStream(), currentTime=%" PRIu64,
-       this, mCurrentTime));
+  TRACK_LOG(LogLevel::Info,
+            ("[AudioTrackEncoder %p]: NotifyEndOfStream()", this));
 
   if (!mCanceled && !mInitialized) {
     // If source audio track is completely silent till the end of encoding,
@@ -239,84 +241,11 @@ void AudioTrackEncoder::NotifyEndOfStream() {
 
   mEndOfStream = true;
 
-  mIncomingBuffer.Clear();
-
   if (mInitialized && !mCanceled) {
     OnDataAvailable();
   }
 }
 
-void AudioTrackEncoder::SetStartOffset(StreamTime aStartOffset) {
-  MOZ_ASSERT(!mWorkerThread || mWorkerThread->IsCurrentThreadIn());
-  MOZ_ASSERT(mCurrentTime == 0);
-  TRACK_LOG(LogLevel::Info,
-            ("[AudioTrackEncoder %p]: SetStartOffset(), aStartOffset=%" PRIu64,
-             this, aStartOffset));
-  mIncomingBuffer.InsertNullDataAtStart(aStartOffset);
-  mCurrentTime = aStartOffset;
-}
-
-void AudioTrackEncoder::AdvanceBlockedInput(StreamTime aDuration) {
-  MOZ_ASSERT(!mWorkerThread || mWorkerThread->IsCurrentThreadIn());
-  TRACK_LOG(
-      LogLevel::Verbose,
-      ("[AudioTrackEncoder %p]: AdvanceBlockedInput(), aDuration=%" PRIu64,
-       this, aDuration));
-
-  // We call Init here so it can account for aDuration towards the Init timeout
-  TryInit(mOutgoingBuffer, aDuration);
-
-  mIncomingBuffer.InsertNullDataAtStart(aDuration);
-  mCurrentTime += aDuration;
-}
-
-void AudioTrackEncoder::AdvanceCurrentTime(StreamTime aDuration) {
-  AUTO_PROFILER_LABEL("AudioTrackEncoder::AdvanceCurrentTime", OTHER);
-
-  MOZ_ASSERT(!mWorkerThread || mWorkerThread->IsCurrentThreadIn());
-
-  if (mCanceled) {
-    return;
-  }
-
-  if (mEndOfStream) {
-    return;
-  }
-
-  TRACK_LOG(LogLevel::Verbose,
-            ("[AudioTrackEncoder %p]: AdvanceCurrentTime() %" PRIu64, this,
-             aDuration));
-
-  StreamTime currentTime = mCurrentTime + aDuration;
-
-  if (mSuspended) {
-    mCurrentTime = currentTime;
-    mIncomingBuffer.ForgetUpTo(mCurrentTime);
-    return;
-  }
-
-  if (currentTime <= mIncomingBuffer.GetDuration()) {
-    mOutgoingBuffer.AppendSlice(mIncomingBuffer, mCurrentTime, currentTime);
-
-    TryInit(mOutgoingBuffer, aDuration);
-    if (mInitialized && mOutgoingBuffer.GetDuration() >= GetPacketDuration()) {
-      OnDataAvailable();
-    }
-  } else {
-    NS_ASSERTION(false,
-                 "AudioTrackEncoder::AdvanceCurrentTime Not enough data");
-    TRACK_LOG(
-        LogLevel::Error,
-        ("[AudioTrackEncoder %p]: AdvanceCurrentTime() Not enough data. "
-         "In incoming=%" PRIu64 ", aDuration=%" PRIu64 ", currentTime=%" PRIu64,
-         this, mIncomingBuffer.GetDuration(), aDuration, currentTime));
-    OnError();
-  }
-
-  mCurrentTime = currentTime;
-  mIncomingBuffer.ForgetUpTo(mCurrentTime);
-}
-
 /*static*/
 void AudioTrackEncoder::InterleaveTrackData(AudioChunk& aChunk,
                                             int32_t aDuration,
@@ -366,13 +295,14 @@ void AudioTrackEncoder::DeInterleaveTrackData(AudioDataValue* aInput,
 size_t AudioTrackEncoder::SizeOfExcludingThis(
     mozilla::MallocSizeOf aMallocSizeOf) {
   MOZ_ASSERT(!mWorkerThread || mWorkerThread->IsCurrentThreadIn());
-  return mIncomingBuffer.SizeOfExcludingThis(aMallocSizeOf) +
-         mOutgoingBuffer.SizeOfExcludingThis(aMallocSizeOf);
+  return mOutgoingBuffer.SizeOfExcludingThis(aMallocSizeOf);
 }
 
-VideoTrackEncoder::VideoTrackEncoder(TrackRate aTrackRate,
+VideoTrackEncoder::VideoTrackEncoder(RefPtr<DriftCompensator> aDriftCompensator,
+                                     TrackRate aTrackRate,
                                      FrameDroppingMode aFrameDroppingMode)
     : TrackEncoder(aTrackRate),
+      mDriftCompensator(std::move(aDriftCompensator)),
       mFrameWidth(0),
       mFrameHeight(0),
       mDisplayWidth(0),
@@ -380,14 +310,17 @@ VideoTrackEncoder::VideoTrackEncoder(TrackRate aTrackRate,
       mEncodedTicks(0),
       mVideoBitrate(0),
       mFrameDroppingMode(aFrameDroppingMode),
-      mKeyFrameInterval(DEFAULT_KEYFRAME_INTERVAL_MS) {
+      mKeyFrameInterval(DEFAULT_KEYFRAME_INTERVAL_MS),
+      mEnabled(true) {
   mLastChunk.mDuration = 0;
 }
 
-void VideoTrackEncoder::Suspend(TimeStamp aTime) {
+void VideoTrackEncoder::Suspend(const TimeStamp& aTime) {
   MOZ_ASSERT(!mWorkerThread || mWorkerThread->IsCurrentThreadIn());
-  TRACK_LOG(LogLevel::Info, ("[VideoTrackEncoder %p]: Suspend(), was %s", this,
-                             mSuspended ? "suspended" : "live"));
+  TRACK_LOG(LogLevel::Info,
+            ("[VideoTrackEncoder %p]: Suspend() at %.3fs, was %s", this,
+             mStartTime.IsNull() ? 0.0 : (aTime - mStartTime).ToSeconds(),
+             mSuspended ? "suspended" : "live"));
 
   if (mSuspended) {
     return;
@@ -397,21 +330,25 @@ void VideoTrackEncoder::Suspend(TimeStamp aTime) {
   mSuspendTime = aTime;
 }
 
-void VideoTrackEncoder::Resume(TimeStamp aTime) {
+void VideoTrackEncoder::Resume(const TimeStamp& aTime) {
   MOZ_ASSERT(!mWorkerThread || mWorkerThread->IsCurrentThreadIn());
-  TRACK_LOG(LogLevel::Info, ("[VideoTrackEncoder %p]: Resume(), was %s", this,
-                             mSuspended ? "suspended" : "live"));
 
   if (!mSuspended) {
     return;
   }
 
+  TRACK_LOG(
+      LogLevel::Info,
+      ("[VideoTrackEncoder %p]: Resume() after %.3fs, was %s", this,
+       (aTime - mSuspendTime).ToSeconds(), mSuspended ? "suspended" : "live"));
+
   mSuspended = false;
 
   TimeDuration suspendDuration = aTime - mSuspendTime;
   if (!mLastChunk.mTimeStamp.IsNull()) {
-    VideoChunk* nextChunk = mIncomingBuffer.FindChunkContaining(mCurrentTime);
-    if (nextChunk && nextChunk->mTimeStamp < aTime) {
+    VideoChunk* nextChunk = mIncomingBuffer.FindChunkContaining(aTime);
+    MOZ_ASSERT_IF(nextChunk, nextChunk->mTimeStamp <= aTime);
+    if (nextChunk) {
       nextChunk->mTimeStamp = aTime;
     }
     mLastChunk.mTimeStamp += suspendDuration;
@@ -423,11 +360,64 @@ void VideoTrackEncoder::Resume(TimeStamp aTime) {
   mSuspendTime = TimeStamp();
 }
 
+void VideoTrackEncoder::Disable(const TimeStamp& aTime) {
+  MOZ_ASSERT(!mWorkerThread || mWorkerThread->IsCurrentThreadIn());
+  TRACK_LOG(LogLevel::Debug, ("[VideoTrackEncoder %p]: Disable()", this));
+
+  if (mStartTime.IsNull()) {
+    // We haven't started yet. No need to touch future frames.
+    mEnabled = false;
+    return;
+  }
+
+  // Advancing currentTime to process any frames in mIncomingBuffer between
+  // mCurrentTime and aTime.
+  AdvanceCurrentTime(aTime);
+  if (!mLastChunk.mTimeStamp.IsNull()) {
+    // Insert a black frame at t=aTime into mIncomingBuffer, to trigger the
+    // shift to black at the right moment.
+    VideoSegment tempSegment;
+    tempSegment.AppendFrom(&mIncomingBuffer);
+    mIncomingBuffer.AppendFrame(do_AddRef(mLastChunk.mFrame.GetImage()),
+                                mLastChunk.mFrame.GetIntrinsicSize(),
+                                mLastChunk.mFrame.GetPrincipalHandle(), true,
+                                aTime);
+    mIncomingBuffer.AppendFrom(&tempSegment);
+  }
+  mEnabled = false;
+}
+
+void VideoTrackEncoder::Enable(const TimeStamp& aTime) {
+  MOZ_ASSERT(!mWorkerThread || mWorkerThread->IsCurrentThreadIn());
+  TRACK_LOG(LogLevel::Debug, ("[VideoTrackEncoder %p]: Enable()", this));
+
+  if (mStartTime.IsNull()) {
+    // We haven't started yet. No need to touch future frames.
+    mEnabled = true;
+    return;
+  }
+
+  // Advancing currentTime to process any frames in mIncomingBuffer between
+  // mCurrentTime and aTime.
+  AdvanceCurrentTime(aTime);
+  if (!mLastChunk.mTimeStamp.IsNull()) {
+    // Insert a real frame at t=aTime into mIncomingBuffer, to trigger the
+    // shift from black at the right moment.
+    VideoSegment tempSegment;
+    tempSegment.AppendFrom(&mIncomingBuffer);
+    mIncomingBuffer.AppendFrame(do_AddRef(mLastChunk.mFrame.GetImage()),
+                                mLastChunk.mFrame.GetIntrinsicSize(),
+                                mLastChunk.mFrame.GetPrincipalHandle(),
+                                mLastChunk.mFrame.GetForceBlack(), aTime);
+    mIncomingBuffer.AppendFrom(&tempSegment);
+  }
+  mEnabled = true;
+}
+
 void VideoTrackEncoder::AppendVideoSegment(VideoSegment&& aSegment) {
   MOZ_ASSERT(!mWorkerThread || mWorkerThread->IsCurrentThreadIn());
   TRACK_LOG(LogLevel::Verbose,
-            ("[VideoTrackEncoder %p]: AppendVideoSegment() duration=%" PRIu64,
-             this, aSegment.GetDuration()));
+            ("[VideoTrackEncoder %p]: AppendVideoSegment()", this));
 
   if (mCanceled) {
     return;
@@ -437,7 +427,28 @@ void VideoTrackEncoder::AppendVideoSegment(VideoSegment&& aSegment) {
     return;
   }
 
-  mIncomingBuffer.AppendFrom(&aSegment);
+  for (VideoSegment::ConstChunkIterator iter(aSegment); !iter.IsEnded();
+       iter.Next()) {
+    if (iter->IsNull()) {
+      // A null image was sent. This is a signal from the source that we should
+      // clear any images buffered in the future.
+      mIncomingBuffer.Clear();
+      continue;  // Don't append iter, as it is null.
+    }
+    if (VideoChunk* c = mIncomingBuffer.GetLastChunk()) {
+      if (iter->mTimeStamp < c->mTimeStamp) {
+        // Time went backwards. This can happen when a MediaDecoder seeks.
+        // We need to handle this by removing any frames buffered in the future
+        // and start over at iter->mTimeStamp.
+        mIncomingBuffer.Clear();
+      }
+    }
+    mIncomingBuffer.AppendFrame(do_AddRef(iter->mFrame.GetImage()),
+                                iter->mFrame.GetIntrinsicSize(),
+                                iter->mFrame.GetPrincipalHandle(),
+                                iter->mFrame.GetForceBlack(), iter->mTimeStamp);
+  }
+  aSegment.Clear();
 }
 
 void VideoTrackEncoder::TakeTrackData(VideoSegment& aSegment) {
@@ -452,7 +463,7 @@ void VideoTrackEncoder::TakeTrackData(VideoSegment& aSegment) {
 }
 
 void VideoTrackEncoder::Init(const VideoSegment& aSegment,
-                             StreamTime aDuration) {
+                             const TimeStamp& aTime) {
   MOZ_ASSERT(!mWorkerThread || mWorkerThread->IsCurrentThreadIn());
 
   if (mInitialized) {
@@ -488,8 +499,7 @@ void VideoTrackEncoder::Init(const VideoSegment& aSegment,
     break;
   }
 
-  mNotInitDuration += aDuration;
-  if ((mNotInitDuration / mTrackRate > VIDEO_INIT_FAILED_DURATION) &&
+  if (((aTime - mStartTime).ToSeconds() > VIDEO_INIT_FAILED_DURATION) &&
       mInitCounter > 1) {
     TRACK_LOG(LogLevel::Warning,
               ("[VideoTrackEncoder %p]: No successful init for %ds.", this,
@@ -503,9 +513,7 @@ void VideoTrackEncoder::Init(const VideoSegment& aSegment,
 
 void VideoTrackEncoder::Cancel() {
   MOZ_ASSERT(!mWorkerThread || mWorkerThread->IsCurrentThreadIn());
-  TRACK_LOG(LogLevel::Info,
-            ("[VideoTrackEncoder %p]: Cancel(), currentTime=%" PRIu64, this,
-             mCurrentTime));
+  TRACK_LOG(LogLevel::Info, ("[VideoTrackEncoder %p]: Cancel()", this));
   mCanceled = true;
   mIncomingBuffer.Clear();
   mOutgoingBuffer.Clear();
@@ -528,22 +536,33 @@ void VideoTrackEncoder::NotifyEndOfStream() {
   }
 
   mEndOfStream = true;
-  TRACK_LOG(
-      LogLevel::Info,
-      ("[VideoTrackEncoder %p]: NotifyEndOfStream(), currentTime=%" PRIu64,
-       this, mCurrentTime));
+  TRACK_LOG(LogLevel::Info,
+            ("[VideoTrackEncoder %p]: NotifyEndOfStream()", this));
 
-  if (!mLastChunk.IsNull() && mLastChunk.mDuration > 0) {
+  if (!mLastChunk.IsNull()) {
     RefPtr<layers::Image> lastImage = mLastChunk.mFrame.GetImage();
-    TRACK_LOG(LogLevel::Debug,
-              ("[VideoTrackEncoder]: Appending last video frame %p, "
-               "duration=%.5f",
-               lastImage.get(),
-               FramesToTimeUnit(mLastChunk.mDuration, mTrackRate).ToSeconds()));
-    mOutgoingBuffer.AppendFrame(
-        lastImage.forget(), mLastChunk.mDuration,
-        mLastChunk.mFrame.GetIntrinsicSize(), PRINCIPAL_HANDLE_NONE,
-        mLastChunk.mFrame.GetForceBlack(), mLastChunk.mTimeStamp);
+    const TimeStamp now = TimeStamp::Now();
+    TimeStamp currentTime = mSuspended ? mSuspendTime : mCurrentTime;
+    currentTime = mDriftCompensator->GetVideoTime(now, currentTime);
+    TimeDuration absoluteEndTime = currentTime - mStartTime;
+    CheckedInt64 duration =
+        UsecsToFrames(absoluteEndTime.ToMicroseconds(), mTrackRate) -
+        mEncodedTicks;
+    if (duration.isValid() && duration.value() > 0) {
+      mEncodedTicks += duration.value();
+      TRACK_LOG(LogLevel::Debug,
+                ("[VideoTrackEncoder %p]: Appending last video frame %p at pos "
+                 "%.3fs, "
+                 "track-end=%.3fs",
+                 this, lastImage.get(),
+                 (mLastChunk.mTimeStamp - mStartTime).ToSeconds(),
+                 absoluteEndTime.ToSeconds()));
+      mOutgoingBuffer.AppendFrame(
+          lastImage.forget(), mLastChunk.mFrame.GetIntrinsicSize(),
+          PRINCIPAL_HANDLE_NONE, mLastChunk.mFrame.GetForceBlack() || !mEnabled,
+          mLastChunk.mTimeStamp);
+      mOutgoingBuffer.ExtendLastFrameBy(duration.value());
+    }
   }
 
   mIncomingBuffer.Clear();
@@ -554,34 +573,20 @@ void VideoTrackEncoder::NotifyEndOfStream() {
   }
 }
 
-void VideoTrackEncoder::SetStartOffset(StreamTime aStartOffset) {
+void VideoTrackEncoder::SetStartOffset(const TimeStamp& aStartOffset) {
   MOZ_ASSERT(!mWorkerThread || mWorkerThread->IsCurrentThreadIn());
-  MOZ_ASSERT(mCurrentTime == 0);
-  TRACK_LOG(LogLevel::Info,
-            ("[VideoTrackEncoder %p]: SetStartOffset(), aStartOffset=%" PRIu64,
-             this, aStartOffset));
-  mIncomingBuffer.InsertNullDataAtStart(aStartOffset);
+  MOZ_ASSERT(mCurrentTime.IsNull());
+  TRACK_LOG(LogLevel::Info, ("[VideoTrackEncoder %p]: SetStartOffset()", this));
+  mStartTime = aStartOffset;
   mCurrentTime = aStartOffset;
 }
 
-void VideoTrackEncoder::AdvanceBlockedInput(StreamTime aDuration) {
-  MOZ_ASSERT(!mWorkerThread || mWorkerThread->IsCurrentThreadIn());
-  TRACK_LOG(
-      LogLevel::Verbose,
-      ("[VideoTrackEncoder %p]: AdvanceBlockedInput(), aDuration=%" PRIu64,
-       this, aDuration));
-
-  // We call Init here so it can account for aDuration towards the Init timeout
-  Init(mOutgoingBuffer, aDuration);
-
-  mIncomingBuffer.InsertNullDataAtStart(aDuration);
-  mCurrentTime += aDuration;
-}
-
-void VideoTrackEncoder::AdvanceCurrentTime(StreamTime aDuration) {
+void VideoTrackEncoder::AdvanceCurrentTime(const TimeStamp& aTime) {
   AUTO_PROFILER_LABEL("VideoTrackEncoder::AdvanceCurrentTime", OTHER);
 
   MOZ_ASSERT(!mWorkerThread || mWorkerThread->IsCurrentThreadIn());
+  MOZ_ASSERT(!mStartTime.IsNull());
+  MOZ_ASSERT(!mCurrentTime.IsNull());
 
   if (mCanceled) {
     return;
@@ -591,117 +596,114 @@ void VideoTrackEncoder::AdvanceCurrentTime(StreamTime aDuration) {
     return;
   }
 
-  TRACK_LOG(LogLevel::Verbose,
-            ("[VideoTrackEncoder %p]: AdvanceCurrentTime() %" PRIu64, this,
-             aDuration));
-
-  StreamTime currentTime = mCurrentTime + aDuration;
-
   if (mSuspended) {
-    mCurrentTime = currentTime;
-    mIncomingBuffer.ForgetUpTo(mCurrentTime);
+    TRACK_LOG(
+        LogLevel::Verbose,
+        ("[VideoTrackEncoder %p]: AdvanceCurrentTime() suspended at %.3fs",
+         this, (mCurrentTime - mStartTime).ToSeconds()));
+    mCurrentTime = aTime;
+    mIncomingBuffer.ForgetUpToTime(mCurrentTime);
     return;
   }
 
+  TRACK_LOG(LogLevel::Verbose,
+            ("[VideoTrackEncoder %p]: AdvanceCurrentTime() to %.3fs", this,
+             (aTime - mStartTime).ToSeconds()));
+
+  // Grab frames within the currentTime range from the incoming buffer.
   VideoSegment tempSegment;
-  if (currentTime <= mIncomingBuffer.GetDuration()) {
-    tempSegment.AppendSlice(mIncomingBuffer, mCurrentTime, currentTime);
-  } else {
-    NS_ASSERTION(false,
-                 "VideoTrackEncoder::AdvanceCurrentTime Not enough data");
-    TRACK_LOG(
-        LogLevel::Error,
-        ("[VideoTrackEncoder %p]: AdvanceCurrentTime() Not enough data. "
-         "In incoming=%" PRIu64 ", aDuration=%" PRIu64 ", currentTime=%" PRIu64,
-         this, mIncomingBuffer.GetDuration(), aDuration, currentTime));
-    OnError();
+  {
+    VideoChunk* previousChunk = &mLastChunk;
+    auto appendDupes = [&](const TimeStamp& aUpTo) {
+      while ((aUpTo - previousChunk->mTimeStamp).ToSeconds() > 1.0) {
+        // We encode at least one frame per second, even if there are none
+        // flowing.
+        previousChunk->mTimeStamp += TimeDuration::FromSeconds(1.0);
+        tempSegment.AppendFrame(
+            do_AddRef(previousChunk->mFrame.GetImage()),
+            previousChunk->mFrame.GetIntrinsicSize(),
+            previousChunk->mFrame.GetPrincipalHandle(),
+            previousChunk->mFrame.GetForceBlack() || !mEnabled,
+            previousChunk->mTimeStamp);
+        TRACK_LOG(
+            LogLevel::Verbose,
+            ("[VideoTrackEncoder %p]: Duplicating video frame (%p) at pos %.3f",
+             this, previousChunk->mFrame.GetImage(),
+             (previousChunk->mTimeStamp - mStartTime).ToSeconds()));
+      }
+    };
+    for (VideoSegment::ChunkIterator iter(mIncomingBuffer); !iter.IsEnded();
+         iter.Next()) {
+      MOZ_ASSERT(!iter->IsNull());
+      if (!previousChunk->IsNull() &&
+          iter->mTimeStamp <= previousChunk->mTimeStamp) {
+        // This frame starts earlier than previousChunk. Skip.
+        continue;
+      }
+      if (iter->mTimeStamp >= aTime) {
+        // This frame starts in the future. Stop.
+        break;
+      }
+      if (!previousChunk->IsNull()) {
+        appendDupes(iter->mTimeStamp);
+      }
+      tempSegment.AppendFrame(
+          do_AddRef(iter->mFrame.GetImage()), iter->mFrame.GetIntrinsicSize(),
+          iter->mFrame.GetPrincipalHandle(),
+          iter->mFrame.GetForceBlack() || !mEnabled, iter->mTimeStamp);
+      TRACK_LOG(LogLevel::Verbose,
+                ("[VideoTrackEncoder %p]: Taking video frame (%p) at pos %.3f",
+                 this, iter->mFrame.GetImage(),
+                 (iter->mTimeStamp - mStartTime).ToSeconds()));
+      previousChunk = &*iter;
+    }
+    if (!previousChunk->IsNull()) {
+      appendDupes(aTime);
+    }
   }
+  mCurrentTime = aTime;
+  mIncomingBuffer.ForgetUpToTime(mCurrentTime);
 
-  mCurrentTime = currentTime;
-  mIncomingBuffer.ForgetUpTo(mCurrentTime);
-
+  // Convert tempSegment timestamps to durations and add chunks with known
+  // duration to mOutgoingBuffer.
+  const TimeStamp now = TimeStamp::Now();
   bool chunkAppended = false;
-
-  // Convert tempSegment timestamps to durations and add it to mOutgoingBuffer.
-  VideoSegment::ConstChunkIterator iter(tempSegment);
-  for (; !iter.IsEnded(); iter.Next()) {
+  for (VideoSegment::ConstChunkIterator iter(tempSegment); !iter.IsEnded();
+       iter.Next()) {
     VideoChunk chunk = *iter;
 
     if (mLastChunk.mTimeStamp.IsNull()) {
-      if (chunk.IsNull()) {
-        // The start of this track is frameless. We need to track the time
-        // it takes to get the first frame.
-        mLastChunk.mDuration += chunk.mDuration;
-        continue;
-      }
+      // This is the first real chunk in the track. Make it start at the
+      // beginning of the track.
+      MOZ_ASSERT(!iter->mTimeStamp.IsNull());
 
-      // This is the first real chunk in the track. Use its timestamp as the
-      // starting point for this track.
-      MOZ_ASSERT(!chunk.mTimeStamp.IsNull());
-      const StreamTime nullDuration = mLastChunk.mDuration;
-      mLastChunk = chunk;
-      chunk.mDuration = 0;
+      TRACK_LOG(
+          LogLevel::Verbose,
+          ("[VideoTrackEncoder %p]: Got the first video frame (%p) at pos %.3f "
+           "(moving it to beginning)",
+           this, iter->mFrame.GetImage(),
+           (iter->mTimeStamp - mStartTime).ToSeconds()));
 
-      TRACK_LOG(LogLevel::Verbose,
-                ("[VideoTrackEncoder]: Got first video chunk after %" PRId64
-                 " ticks.",
-                 nullDuration));
-      // Adapt to the time before the first frame. This extends the first frame
-      // from [start, end] to [0, end], but it'll do for now.
-      auto diff = FramesToTimeUnit(nullDuration, mTrackRate);
-      if (!diff.IsValid()) {
-        NS_ERROR("null duration overflow");
-        return;
-      }
-
-      mLastChunk.mTimeStamp -= diff.ToTimeDuration();
-      mLastChunk.mDuration += nullDuration;
+      mLastChunk = *iter;
+      mLastChunk.mTimeStamp = mStartTime;
+      continue;
     }
 
     MOZ_ASSERT(!mLastChunk.IsNull());
-    if (mLastChunk.CanCombineWithFollowing(chunk) || chunk.IsNull()) {
-      TRACK_LOG(LogLevel::Verbose,
-                ("[VideoTrackEncoder]: Got dupe or null chunk."));
-      // This is the same frame as before (or null). We extend the last chunk
-      // with its duration.
-      mLastChunk.mDuration += chunk.mDuration;
+    MOZ_ASSERT(!chunk.IsNull());
 
-      if (mLastChunk.mDuration < mTrackRate) {
-        TRACK_LOG(LogLevel::Verbose, ("[VideoTrackEncoder]: Ignoring dupe/null "
-                                      "chunk of duration %" PRId64,
-                                      chunk.mDuration));
-        continue;
-      }
-
-      TRACK_LOG(LogLevel::Verbose,
-                ("[VideoTrackEncoder]: Chunk >1 second. duration=%" PRId64 ", "
-                 "trackRate=%" PRId32,
-                 mLastChunk.mDuration, mTrackRate));
-
-      // If we have gotten dupes for over a second, we force send one
-      // to the encoder to make sure there is some output.
-      chunk.mTimeStamp = mLastChunk.mTimeStamp + TimeDuration::FromSeconds(1);
-      chunk.mDuration = mLastChunk.mDuration - mTrackRate;
-      mLastChunk.mDuration = mTrackRate;
-
-      if (chunk.IsNull()) {
-        // Ensure that we don't pass null to the encoder by making mLastChunk
-        // null later on.
-        chunk.mFrame = mLastChunk.mFrame;
-      }
-    }
-
-    if (mStartTime.IsNull()) {
-      mStartTime = mLastChunk.mTimeStamp;
-    }
-
-    TimeDuration relativeTime = chunk.mTimeStamp - mStartTime;
-    RefPtr<layers::Image> lastImage = mLastChunk.mFrame.GetImage();
+    TimeDuration absoluteEndTime =
+        mDriftCompensator->GetVideoTime(now, chunk.mTimeStamp) - mStartTime;
     TRACK_LOG(LogLevel::Verbose,
-              ("[VideoTrackEncoder]: Appending video frame %p, at pos %.5fs",
-               lastImage.get(), relativeTime.ToSeconds()));
+              ("[VideoTrackEncoder %p]: Appending video frame %p, at pos %.3fs "
+               "until %.3fs",
+               this, mLastChunk.mFrame.GetImage(),
+               (mDriftCompensator->GetVideoTime(now, mLastChunk.mTimeStamp) -
+                mStartTime)
+                   .ToSeconds(),
+               absoluteEndTime.ToSeconds()));
     CheckedInt64 duration =
-        UsecsToFrames(relativeTime.ToMicroseconds(), mTrackRate) -
+        UsecsToFrames(absoluteEndTime.ToMicroseconds(), mTrackRate) -
         mEncodedTicks;
     if (!duration.isValid()) {
       NS_ERROR("Duration overflow");
@@ -709,29 +711,35 @@ void VideoTrackEncoder::AdvanceCurrentTime(StreamTime aDuration) {
     }
 
     if (duration.value() <= 0) {
-      // The timestamp for mLastChunk is newer than for chunk.
-      // This means the durations reported from MediaStreamGraph for
-      // mLastChunk were larger than the timestamp diff - and durations were
-      // used to trigger the 1-second frame above. This could happen due to
-      // drift or underruns in the graph.
-      TRACK_LOG(LogLevel::Warning,
-                ("[VideoTrackEncoder]: Underrun detected. Diff=%" PRId64,
-                 duration.value()));
-      chunk.mTimeStamp = mLastChunk.mTimeStamp;
-    } else {
-      mEncodedTicks += duration.value();
-      mOutgoingBuffer.AppendFrame(
-          lastImage.forget(), duration.value(),
-          mLastChunk.mFrame.GetIntrinsicSize(), PRINCIPAL_HANDLE_NONE,
-          mLastChunk.mFrame.GetForceBlack(), mLastChunk.mTimeStamp);
-      chunkAppended = true;
+      // A frame either started before the last frame (can happen when
+      // multiple frames are added before SetStartOffset), or
+      // two frames were so close together that they ended up at the same
+      // position. We handle both cases by ignoring the previous frame.
+
+      TRACK_LOG(LogLevel::Verbose,
+                ("[VideoTrackEncoder %p]: Duration from frame %p to frame %p "
+                 "is %" PRId64 ". Ignoring %p",
+                 this, mLastChunk.mFrame.GetImage(), iter->mFrame.GetImage(),
+                 duration.value(), mLastChunk.mFrame.GetImage()));
+
+      TimeStamp t = mLastChunk.mTimeStamp;
+      mLastChunk = *iter;
+      mLastChunk.mTimeStamp = t;
+      continue;
     }
 
+    mEncodedTicks += duration.value();
+    mOutgoingBuffer.AppendFrame(
+        do_AddRef(mLastChunk.mFrame.GetImage()),
+        mLastChunk.mFrame.GetIntrinsicSize(), PRINCIPAL_HANDLE_NONE,
+        mLastChunk.mFrame.GetForceBlack() || !mEnabled, mLastChunk.mTimeStamp);
+    mOutgoingBuffer.ExtendLastFrameBy(duration.value());
+    chunkAppended = true;
     mLastChunk = chunk;
   }
 
   if (chunkAppended) {
-    Init(mOutgoingBuffer, aDuration);
+    Init(mOutgoingBuffer, mCurrentTime);
     if (mInitialized) {
       OnDataAvailable();
     }
