@@ -70,7 +70,7 @@ class HLSDemuxer::HLSDemuxerCallbacksSupport
   using NativeCallbacks::AttachNative;
   using NativeCallbacks::DisposeNative;
 
-  HLSDemuxerCallbacksSupport(HLSDemuxer* aDemuxer)
+  explicit HLSDemuxerCallbacksSupport(HLSDemuxer* aDemuxer)
       : mMutex("HLSDemuxerCallbacksSupport"), mDemuxer(aDemuxer) {
     MOZ_ASSERT(mDemuxer);
   }
@@ -125,9 +125,8 @@ class HLSDemuxer::HLSDemuxerCallbacksSupport
 };
 
 HLSDemuxer::HLSDemuxer(int aPlayerId)
-    : mTaskQueue(
-          new AutoTaskQueue(GetMediaThreadPool(MediaThreadType::PLAYBACK),
-                            /* aSupportsTailDispatch = */ false)) {
+    : mTaskQueue(new TaskQueue(GetMediaThreadPool(MediaThreadType::PLAYBACK),
+                               /* aSupportsTailDispatch = */ false)) {
   MOZ_ASSERT(NS_IsMainThread());
   HLSDemuxerCallbacksSupport::Init();
   mJavaCallbacks = GeckoHLSDemuxerWrapper::Callbacks::New();
@@ -232,7 +231,7 @@ HLSTrackDemuxer::HLSTrackDemuxer(HLSDemuxer* aParent,
     : mParent(aParent),
       mType(aType),
       mMutex("HLSTrackDemuxer"),
-      mTrackInfo(Move(aTrackInfo)) {
+      mTrackInfo(std::move(aTrackInfo)) {
   // Only support audio and video track currently.
   MOZ_ASSERT(mType == TrackInfo::kVideoTrack ||
              mType == TrackInfo::kAudioTrack);
@@ -311,7 +310,7 @@ RefPtr<HLSTrackDemuxer::SamplesPromise> HLSTrackDemuxer::DoGetSamples(
   }
 
   for (auto&& demuxedSample : sampleObjectArray) {
-    java::GeckoHLSSample::LocalRef sample(Move(demuxedSample));
+    java::GeckoHLSSample::LocalRef sample(std::move(demuxedSample));
     if (sample->IsEOS()) {
       HLS_DEBUG("HLSTrackDemuxer", "Met BUFFER_FLAG_END_OF_STREAM.");
       if (samples->mSamples.IsEmpty()) {
@@ -345,43 +344,52 @@ void HLSTrackDemuxer::UpdateMediaInfo(int index) {
   jni::Object::LocalRef infoObj = nullptr;
   if (mType == TrackType::kAudioTrack) {
     infoObj = mParent->mHLSDemuxerWrapper->GetAudioInfo(index);
+    if (!infoObj) {
+      NS_WARNING("Failed to get audio info from Java wrapper");
+      return;
+    }
     auto* audioInfo = mTrackInfo->GetAsAudioInfo();
-    if (infoObj && audioInfo) {
-      HLS_DEBUG("HLSTrackDemuxer", "Update audio info (%d)", index);
-      java::GeckoAudioInfo::LocalRef audioInfoObj(Move(infoObj));
-      audioInfo->mRate = audioInfoObj->Rate();
-      audioInfo->mChannels = audioInfoObj->Channels();
-      audioInfo->mProfile = audioInfoObj->Profile();
-      audioInfo->mBitDepth = audioInfoObj->BitDepth();
-      audioInfo->mMimeType =
-          NS_ConvertUTF16toUTF8(audioInfoObj->MimeType()->ToString());
-      audioInfo->mDuration =
-          TimeUnit::FromMicroseconds(audioInfoObj->Duration());
-      auto&& csd = audioInfoObj->CodecSpecificData()->GetElements();
+    MOZ_ASSERT(audioInfo != nullptr);
+    HLS_DEBUG("HLSTrackDemuxer", "Update audio info (%d)", index);
+    java::GeckoAudioInfo::LocalRef audioInfoObj(std::move(infoObj));
+    audioInfo->mRate = audioInfoObj->Rate();
+    audioInfo->mChannels = audioInfoObj->Channels();
+    audioInfo->mProfile = audioInfoObj->Profile();
+    audioInfo->mBitDepth = audioInfoObj->BitDepth();
+    audioInfo->mMimeType =
+        NS_ConvertUTF16toUTF8(audioInfoObj->MimeType()->ToString());
+    audioInfo->mDuration =
+        TimeUnit::FromMicroseconds(audioInfoObj->Duration());
+    jni::ByteArray::LocalRef csdBytes = audioInfoObj->CodecSpecificData();
+    if (csdBytes) {
+      auto&& csd = csdBytes->GetElements();
       audioInfo->mCodecSpecificConfig->Clear();
       audioInfo->mCodecSpecificConfig->AppendElements(
           reinterpret_cast<uint8_t*>(&csd[0]), csd.Length());
     }
   } else {
     infoObj = mParent->mHLSDemuxerWrapper->GetVideoInfo(index);
-    auto* videoInfo = mTrackInfo->GetAsVideoInfo();
-    if (infoObj && videoInfo) {
-      java::GeckoVideoInfo::LocalRef videoInfoObj(Move(infoObj));
-      videoInfo->mStereoMode = getStereoMode(videoInfoObj->StereoMode());
-      videoInfo->mRotation = getVideoInfoRotation(videoInfoObj->Rotation());
-      videoInfo->mImage.width = videoInfoObj->DisplayWidth();
-      videoInfo->mImage.height = videoInfoObj->DisplayHeight();
-      videoInfo->mDisplay.width = videoInfoObj->PictureWidth();
-      videoInfo->mDisplay.height = videoInfoObj->PictureHeight();
-      videoInfo->mMimeType =
-          NS_ConvertUTF16toUTF8(videoInfoObj->MimeType()->ToString());
-      videoInfo->mDuration =
-          TimeUnit::FromMicroseconds(videoInfoObj->Duration());
-      HLS_DEBUG("HLSTrackDemuxer",
-                "Update video info (%d) / I(%dx%d) / D(%dx%d)", index,
-                videoInfo->mImage.width, videoInfo->mImage.height,
-                videoInfo->mDisplay.width, videoInfo->mDisplay.height);
+    if (!infoObj) {
+      NS_WARNING("Failed to get video info from Java wrapper");
+      return;
     }
+    auto* videoInfo = mTrackInfo->GetAsVideoInfo();
+    MOZ_ASSERT(videoInfo != nullptr);
+    java::GeckoVideoInfo::LocalRef videoInfoObj(std::move(infoObj));
+    videoInfo->mStereoMode = getStereoMode(videoInfoObj->StereoMode());
+    videoInfo->mRotation = getVideoInfoRotation(videoInfoObj->Rotation());
+    videoInfo->mImage.width = videoInfoObj->DisplayWidth();
+    videoInfo->mImage.height = videoInfoObj->DisplayHeight();
+    videoInfo->mDisplay.width = videoInfoObj->PictureWidth();
+    videoInfo->mDisplay.height = videoInfoObj->PictureHeight();
+    videoInfo->mMimeType =
+        NS_ConvertUTF16toUTF8(videoInfoObj->MimeType()->ToString());
+    videoInfo->mDuration =
+        TimeUnit::FromMicroseconds(videoInfoObj->Duration());
+    HLS_DEBUG("HLSTrackDemuxer",
+              "Update video info (%d) / I(%dx%d) / D(%dx%d)", index,
+              videoInfo->mImage.width, videoInfo->mImage.height,
+              videoInfo->mDisplay.width, videoInfo->mDisplay.height);
   }
 }
 
@@ -395,13 +403,20 @@ CryptoSample HLSTrackDemuxer::ExtractCryptoSample(
   char const* msg = "";
   do {
     HLS_DEBUG("HLSTrackDemuxer", "Sample has Crypto Info");
-    crypto.mValid = true;
+
     int32_t mode = 0;
     if (NS_FAILED(aCryptoInfo->Mode(&mode))) {
       msg = "Error when extracting encryption mode.";
       break;
     }
-    crypto.mMode = mode;
+    // We currently only handle ctr mode.
+    if (mode != java::sdk::MediaCodec::CRYPTO_MODE_AES_CTR) {
+      msg = "Error: unexpected encryption mode.";
+      break;
+    }
+
+    crypto.mCryptoScheme = CryptoScheme::Cenc;
+
     mozilla::jni::ByteArray::LocalRef ivData;
     if (NS_FAILED(aCryptoInfo->Iv(&ivData))) {
       msg = "Error when extracting encryption IV.";
@@ -552,7 +567,7 @@ HLSTrackDemuxer::DoSkipToNextRandomAccessPoint(const TimeUnit& aTimeThreshold) {
       break;
     }
     parsed++;
-    java::GeckoHLSSample::LocalRef sample(Move(sampleObjectArray[0]));
+    java::GeckoHLSSample::LocalRef sample(std::move(sampleObjectArray[0]));
     if (sample->IsEOS()) {
       result = NS_ERROR_DOM_MEDIA_END_OF_STREAM;
       break;
