@@ -98,7 +98,7 @@ already_AddRefed<MediaRawData> SampleIterator::GetNext() {
   sample->mOffset = s->mByteRange.mStart;
   sample->mKeyframe = s->mSync;
 
-  nsAutoPtr<MediaRawDataWriter> writer(sample->CreateWriter());
+  UniquePtr<MediaRawDataWriter> writer(sample->CreateWriter());
   // Do the blocking read
   if (!writer->SetSize(s->mByteRange.Length())) {
     return nullptr;
@@ -111,71 +111,130 @@ already_AddRefed<MediaRawData> SampleIterator::GetNext() {
     return nullptr;
   }
 
-  if (mCurrentSample == 0 && mIndex->mMoofParser) {
-    const nsTArray<Moof>& moofs = mIndex->mMoofParser->Moofs();
-    MOZ_ASSERT(mCurrentMoof < moofs.Length());
+  MoofParser* moofParser = mIndex->mMoofParser.get();
+  if (!moofParser) {
+    // File is not fragmented, we can't have crypto, just early return.
+    Next();
+    return sample.forget();
+  }
+
+  SampleDescriptionEntry* sampleDescriptionEntry = GetSampleDescriptionEntry();
+  if (!sampleDescriptionEntry) {
+    // For the file to be valid the tfhd must reference a sample description
+    // entry.
+    return nullptr;
+  }
+
+  // Match scheme type box to enum representation.
+  CryptoScheme cryptoScheme = CryptoScheme::None;
+  // If a fragment references a sample description without crypto info them we
+  // treat it as unencrypted, even if other fragments may be encrypted.
+  if (sampleDescriptionEntry->mIsEncryptedEntry) {
+    if (!moofParser->mSinf.IsValid()) {
+      // The sample description entry says this sample is encrypted, but we
+      // don't have a relevant sinf box, this shouldn't happen, so bail.
+      return nullptr;
+    }
+    if (moofParser->mSinf.mDefaultEncryptionType == AtomType("cenc")) {
+      cryptoScheme = CryptoScheme::Cenc;
+      writer->mCrypto.mCryptoScheme = CryptoScheme::Cenc;
+    } else if (moofParser->mSinf.mDefaultEncryptionType == AtomType("cbcs")) {
+      cryptoScheme = CryptoScheme::Cbcs;
+      writer->mCrypto.mCryptoScheme = CryptoScheme::Cbcs;
+    } else {
+      MOZ_ASSERT_UNREACHABLE(
+          "Sample description entry reports sample is encrypted, but no "
+          "scheme, or an unsupported shceme is in use!");
+      return nullptr;
+    }
+  }
+
+  // We need to check if this moof has init data the CDM expects us to surface.
+  // This should happen when handling the first sample, even if that sample
+  // isn't encrypted (samples later in the moof may be).
+  if (mCurrentSample == 0) {
+    const nsTArray<Moof>& moofs = moofParser->Moofs();
     const Moof* currentMoof = &moofs[mCurrentMoof];
     if (!currentMoof->mPsshes.IsEmpty()) {
       // This Moof contained crypto init data. Report that. We only report
       // the init data on the Moof's first sample, to avoid reporting it more
       // than once per Moof.
-      writer->mCrypto.mValid = true;
       writer->mCrypto.mInitDatas.AppendElements(currentMoof->mPsshes);
       writer->mCrypto.mInitDataType = NS_LITERAL_STRING("cenc");
     }
   }
 
-  if (!s->mCencRange.IsEmpty()) {
-    MoofParser* parser = mIndex->mMoofParser.get();
-
-    if (!parser || !parser->mSinf.IsValid()) {
-      return nullptr;
-    }
-
-    uint8_t ivSize = parser->mSinf.mDefaultIVSize;
-
-    // The size comes from an 8 bit field
-    AutoTArray<uint8_t, 256> cenc;
-    cenc.SetLength(s->mCencRange.Length());
-    if (!mIndex->mSource->ReadAt(s->mCencRange.mStart, cenc.Elements(),
-                                 cenc.Length(), &bytesRead) ||
-        bytesRead != cenc.Length()) {
-      return nullptr;
-    }
-    BufferReader reader(cenc);
-    writer->mCrypto.mValid = true;
-    writer->mCrypto.mIVSize = ivSize;
+  if (sampleDescriptionEntry->mIsEncryptedEntry) {
+    writer->mCrypto.mCryptoScheme = cryptoScheme;
 
+    MOZ_ASSERT(writer->mCrypto.mKeyId.IsEmpty(),
+               "Sample should not already have a key ID");
+    MOZ_ASSERT(writer->mCrypto.mConstantIV.IsEmpty(),
+               "Sample should not already have a constant IV");
     CencSampleEncryptionInfoEntry* sampleInfo = GetSampleEncryptionEntry();
     if (sampleInfo) {
+      // Use sample group information if present, this supersedes track level
+      // information.
       writer->mCrypto.mKeyId.AppendElements(sampleInfo->mKeyId);
+      writer->mCrypto.mIVSize = sampleInfo->mIVSize;
+      writer->mCrypto.mCryptByteBlock = sampleInfo->mCryptByteBlock;
+      writer->mCrypto.mSkipByteBlock = sampleInfo->mSkipByteBlock;
+      writer->mCrypto.mConstantIV.AppendElements(sampleInfo->mConsantIV);
+    } else {
+      // Use the crypto info from track metadata
+      writer->mCrypto.mKeyId.AppendElements(moofParser->mSinf.mDefaultKeyID,
+                                            16);
+      writer->mCrypto.mIVSize = moofParser->mSinf.mDefaultIVSize;
+      writer->mCrypto.mCryptByteBlock =
+          moofParser->mSinf.mDefaultCryptByteBlock;
+      writer->mCrypto.mSkipByteBlock = moofParser->mSinf.mDefaultSkipByteBlock;
+      writer->mCrypto.mConstantIV.AppendElements(
+          moofParser->mSinf.mDefaultConstantIV);
     }
 
-    if (!reader.ReadArray(writer->mCrypto.mIV, ivSize)) {
-      return nullptr;
-    }
-
-    auto res = reader.ReadU16();
-    if (res.isOk() && res.unwrap() > 0) {
-      uint16_t count = res.unwrap();
-
-      if (reader.Remaining() < count * 6) {
+    MOZ_ASSERT((writer->mCrypto.mIVSize == 0 &&
+                !writer->mCrypto.mConstantIV.IsEmpty()) ||
+                   !s->mCencRange.IsEmpty(),
+               "Crypto information should contain either a constant IV, or "
+               "have auxiliary information that will contain an IV");
+    // Parse auxiliary information if present
+    if (!s->mCencRange.IsEmpty()) {
+      // The size comes from an 8 bit field
+      AutoTArray<uint8_t, 256> cencAuxInfo;
+      cencAuxInfo.SetLength(s->mCencRange.Length());
+      if (!mIndex->mSource->ReadAt(s->mCencRange.mStart, cencAuxInfo.Elements(),
+                                   cencAuxInfo.Length(), &bytesRead) ||
+          bytesRead != cencAuxInfo.Length()) {
+        return nullptr;
+      }
+      BufferReader reader(cencAuxInfo);
+      if (!reader.ReadArray(writer->mCrypto.mIV, writer->mCrypto.mIVSize)) {
         return nullptr;
       }
 
-      for (size_t i = 0; i < count; i++) {
-        auto res_16 = reader.ReadU16();
-        auto res_32 = reader.ReadU32();
-        if (res_16.isErr() || res_32.isErr()) {
+      // Parse the auxiliary information for subsample information
+      auto res = reader.ReadU16();
+      if (res.isOk() && res.unwrap() > 0) {
+        uint16_t count = res.unwrap();
+
+        if (reader.Remaining() < count * 6) {
           return nullptr;
         }
-        writer->mCrypto.mPlainSizes.AppendElement(res_16.unwrap());
-        writer->mCrypto.mEncryptedSizes.AppendElement(res_32.unwrap());
+
+        for (size_t i = 0; i < count; i++) {
+          auto res_16 = reader.ReadU16();
+          auto res_32 = reader.ReadU32();
+          if (res_16.isErr() || res_32.isErr()) {
+            return nullptr;
+          }
+          writer->mCrypto.mPlainSizes.AppendElement(res_16.unwrap());
+          writer->mCrypto.mEncryptedSizes.AppendElement(res_32.unwrap());
+        }
+      } else {
+        // No subsample information means the entire sample is encrypted.
+        writer->mCrypto.mPlainSizes.AppendElement(0);
+        writer->mCrypto.mEncryptedSizes.AppendElement(sample->Size());
       }
-    } else {
-      // No subsample information means the entire sample is encrypted.
-      writer->mCrypto.mPlainSizes.AppendElement(0);
-      writer->mCrypto.mEncryptedSizes.AppendElement(sample->Size());
     }
   }
 
@@ -184,6 +243,22 @@ already_AddRefed<MediaRawData> SampleIterator::GetNext() {
   return sample.forget();
 }
 
+SampleDescriptionEntry* SampleIterator::GetSampleDescriptionEntry() {
+  nsTArray<Moof>& moofs = mIndex->mMoofParser->Moofs();
+  Moof& currentMoof = moofs[mCurrentMoof];
+  uint32_t sampleDescriptionIndex =
+      currentMoof.mTfhd.mDefaultSampleDescriptionIndex;
+  // Mp4 indices start at 1, shift down 1 so we index our array correctly.
+  sampleDescriptionIndex--;
+  FallibleTArray<SampleDescriptionEntry>& sampleDescriptions =
+      mIndex->mMoofParser->mSampleDescriptions;
+  if (sampleDescriptionIndex >= sampleDescriptions.Length()) {
+    // The sample description index is invalid, the mp4 is malformed. Bail out.
+    return nullptr;
+  }
+  return &sampleDescriptions[sampleDescriptionIndex];
+}
+
 CencSampleEncryptionInfoEntry* SampleIterator::GetSampleEncryptionEntry() {
   nsTArray<Moof>& moofs = mIndex->mMoofParser->Moofs();
   Moof* currentMoof = &moofs[mCurrentMoof];
@@ -307,7 +382,7 @@ Index::Index(const IndiceWrapper& aIndices, ByteStream* aSource,
              uint32_t aTrackId, bool aIsAudio)
     : mSource(aSource), mIsAudio(aIsAudio) {
   if (!aIndices.Length()) {
-    mMoofParser = new MoofParser(aSource, aTrackId, aIsAudio);
+    mMoofParser = new MoofParser(aSource, AsVariant(aTrackId), aIsAudio);
   } else {
     if (!mIndex.SetCapacity(aIndices.Length(), fallible)) {
       // OOM.
@@ -330,7 +405,6 @@ Index::Index(const IndiceWrapper& aIndices, ByteStream* aSource,
       if (!haveSync) {
         continue;
       }
-
       Sample sample;
       sample.mByteRange =
           MediaByteRange(indice.start_offset, indice.end_offset);
@@ -479,9 +553,11 @@ TimeIntervals Index::ConvertByteRangesToTimeRanges(
         }
       }
       if (end > start) {
-        timeRanges += TimeInterval(
-            TimeUnit::FromMicroseconds(mDataOffset[start].mTime.start),
-            TimeUnit::FromMicroseconds(mDataOffset[end - 1].mTime.end));
+        for (uint32_t i = start; i < end; i++) {
+          timeRanges += TimeInterval(
+              TimeUnit::FromMicroseconds(mDataOffset[i].mTime.start),
+              TimeUnit::FromMicroseconds(mDataOffset[i].mTime.end));
+        }
       }
       if (end < mDataOffset.Length()) {
         // Find samples in partial block contained in the byte range.
