@@ -2,16 +2,17 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "H264.h"
+#include <cmath>
+#include <limits>
+#include "AnnexB.h"
+#include "BitReader.h"
+#include "BitWriter.h"
+#include "BufferReader.h"
+#include "ByteWriter.h"
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/PodOperations.h"
 #include "mozilla/ResultExtensions.h"
-#include "BitReader.h"
-#include "BufferReader.h"
-#include "ByteWriter.h"
-#include "AnnexB.h"
-#include "H264.h"
-#include <limits>
-#include <cmath>
 
 #define READSE(var, min, max)     \
   {                               \
@@ -99,50 +100,6 @@ static void scaling_list(BitReader& aBr, uint8_t (&aScalingList)[N],
   detail::scaling_list(aBr, aScalingList, N, aDefaultList, nullptr);
 }
 
-static uint32_t GetBitLength(const mozilla::MediaByteBuffer* aNAL) {
-  size_t size = aNAL->Length();
-
-  while (size > 0 && aNAL->ElementAt(size - 1) == 0) {
-    size--;
-  }
-
-  if (!size) {
-    return 0;
-  }
-
-  if (size > UINT32_MAX / 8) {
-    // We can't represent it, we'll use as much as we can.
-    return UINT32_MAX;
-  }
-
-  uint8_t v = aNAL->ElementAt(size - 1);
-  size *= 8;
-
-  // Remove the stop bit and following trailing zeros.
-  if (v) {
-    // Count the consecutive zero bits (trailing) on the right by binary search.
-    // Adapted from Matt Whitlock algorithm to only bother with 8 bits integers.
-    uint32_t c;
-    if (v & 1) {
-      // Special case for odd v (assumed to happen half of the time).
-      c = 0;
-    } else {
-      c = 1;
-      if ((v & 0xf) == 0) {
-        v >>= 4;
-        c += 4;
-      }
-      if ((v & 0x3) == 0) {
-        v >>= 2;
-        c += 2;
-      }
-      c -= v & 0x1;
-    }
-    size -= c + 1;
-  }
-  return size;
-}
-
 SPSData::SPSData() {
   PodZero(this);
   // Default values when they aren't defined as per ITU-T H.264 (2014/02).
@@ -163,6 +120,205 @@ bool SPSData::operator!=(const SPSData& aOther) const {
   return !(operator==(aOther));
 }
 
+// Described in ISO 23001-8:2016
+// Table 2
+enum class PrimaryID : uint8_t {
+  INVALID = 0,
+  BT709 = 1,
+  UNSPECIFIED = 2,
+  BT470M = 4,
+  BT470BG = 5,
+  SMPTE170M = 6,
+  SMPTE240M = 7,
+  FILM = 8,
+  BT2020 = 9,
+  SMPTEST428_1 = 10,
+  SMPTEST431_2 = 11,
+  SMPTEST432_1 = 12,
+  EBU_3213_E = 22
+};
+
+// Table 3
+enum class TransferID : uint8_t {
+  INVALID = 0,
+  BT709 = 1,
+  UNSPECIFIED = 2,
+  GAMMA22 = 4,
+  GAMMA28 = 5,
+  SMPTE170M = 6,
+  SMPTE240M = 7,
+  LINEAR = 8,
+  LOG = 9,
+  LOG_SQRT = 10,
+  IEC61966_2_4 = 11,
+  BT1361_ECG = 12,
+  IEC61966_2_1 = 13,
+  BT2020_10 = 14,
+  BT2020_12 = 15,
+  SMPTEST2084 = 16,
+  SMPTEST428_1 = 17,
+
+  // Not yet standardized
+  ARIB_STD_B67 = 18,  // AKA hybrid-log gamma, HLG.
+};
+
+// Table 4
+enum class MatrixID : uint8_t {
+  RGB = 0,
+  BT709 = 1,
+  UNSPECIFIED = 2,
+  FCC = 4,
+  BT470BG = 5,
+  SMPTE170M = 6,
+  SMPTE240M = 7,
+  YCOCG = 8,
+  BT2020_NCL = 9,
+  BT2020_CL = 10,
+  YDZDX = 11,
+  INVALID = 255,
+};
+
+static PrimaryID GetPrimaryID(int aPrimary) {
+  if (aPrimary < 1 || aPrimary > 22 || aPrimary == 3) {
+    return PrimaryID::INVALID;
+  }
+  if (aPrimary > 12 && aPrimary < 22) {
+    return PrimaryID::INVALID;
+  }
+  return static_cast<PrimaryID>(aPrimary);
+}
+
+static TransferID GetTransferID(int aTransfer) {
+  if (aTransfer < 1 || aTransfer > 18 || aTransfer == 3) {
+    return TransferID::INVALID;
+  }
+  return static_cast<TransferID>(aTransfer);
+}
+
+static MatrixID GetMatrixID(int aMatrix) {
+  if (aMatrix < 0 || aMatrix > 11 || aMatrix == 3) {
+    return MatrixID::INVALID;
+  }
+  return static_cast<MatrixID>(aMatrix);
+}
+
+gfx::YUVColorSpace SPSData::ColorSpace() const {
+  // Bitfield, note that guesses with higher values take precedence over
+  // guesses with lower values.
+  enum Guess {
+    GUESS_BT601 = 1 << 0,
+    GUESS_BT709 = 1 << 1,
+    GUESS_BT2020 = 1 << 2,
+  };
+
+  uint32_t guess = 0;
+
+  switch (GetPrimaryID(colour_primaries)) {
+    case PrimaryID::BT709:
+      guess |= GUESS_BT709;
+      break;
+    case PrimaryID::BT470M:
+    case PrimaryID::BT470BG:
+    case PrimaryID::SMPTE170M:
+    case PrimaryID::SMPTE240M:
+      guess |= GUESS_BT601;
+      break;
+    case PrimaryID::BT2020:
+      guess |= GUESS_BT2020;
+      break;
+    case PrimaryID::FILM:
+    case PrimaryID::SMPTEST428_1:
+    case PrimaryID::SMPTEST431_2:
+    case PrimaryID::SMPTEST432_1:
+    case PrimaryID::EBU_3213_E:
+    case PrimaryID::INVALID:
+    case PrimaryID::UNSPECIFIED:
+      break;
+  }
+
+  switch (GetTransferID(transfer_characteristics)) {
+    case TransferID::BT709:
+      guess |= GUESS_BT709;
+      break;
+    case TransferID::GAMMA22:
+    case TransferID::GAMMA28:
+    case TransferID::SMPTE170M:
+    case TransferID::SMPTE240M:
+      guess |= GUESS_BT601;
+      break;
+    case TransferID::BT2020_10:
+    case TransferID::BT2020_12:
+      guess |= GUESS_BT2020;
+      break;
+    case TransferID::LINEAR:
+    case TransferID::LOG:
+    case TransferID::LOG_SQRT:
+    case TransferID::IEC61966_2_4:
+    case TransferID::BT1361_ECG:
+    case TransferID::IEC61966_2_1:
+    case TransferID::SMPTEST2084:
+    case TransferID::SMPTEST428_1:
+    case TransferID::ARIB_STD_B67:
+    case TransferID::INVALID:
+    case TransferID::UNSPECIFIED:
+      break;
+  }
+
+  switch (GetMatrixID(matrix_coefficients)) {
+    case MatrixID::BT709:
+      guess |= GUESS_BT709;
+      break;
+    case MatrixID::BT470BG:
+    case MatrixID::SMPTE170M:
+    case MatrixID::SMPTE240M:
+      guess |= GUESS_BT601;
+      break;
+    case MatrixID::BT2020_NCL:
+    case MatrixID::BT2020_CL:
+      guess |= GUESS_BT2020;
+      break;
+    case MatrixID::RGB:
+    case MatrixID::FCC:
+    case MatrixID::YCOCG:
+    case MatrixID::YDZDX:
+    case MatrixID::INVALID:
+    case MatrixID::UNSPECIFIED:
+      break;
+  }
+
+  // Removes lowest bit until only a single bit remains.
+  while (guess & (guess - 1)) {
+    guess &= guess - 1;
+  }
+  if (!guess) {
+    // A better default to BT601 which should die a slow death.
+    guess = GUESS_BT709;
+  }
+
+  switch (guess) {
+    case GUESS_BT601:
+      return gfx::YUVColorSpace::BT601;
+    case GUESS_BT709:
+      return gfx::YUVColorSpace::BT709;
+    case GUESS_BT2020:
+      return gfx::YUVColorSpace::BT2020;
+    default:
+      MOZ_ASSERT_UNREACHABLE(
+          "not possible to get here but makes compiler happy");
+      return gfx::YUVColorSpace::UNKNOWN;
+  }
+}
+
+gfx::ColorDepth SPSData::ColorDepth() const {
+  if (bit_depth_luma_minus8 != 0 && bit_depth_luma_minus8 != 2 &&
+      bit_depth_luma_minus8 != 4) {
+    // We don't know what that is, just assume 8 bits to prevent decoding
+    // regressions if we ever encounter those.
+    return gfx::ColorDepth::COLOR_8;
+  }
+  return gfx::ColorDepthForBitDepth(bit_depth_luma_minus8 + 8);
+}
+
 // SPSNAL and SPSNALIterator do not own their data.
 class SPSNAL {
  public:
@@ -174,7 +330,7 @@ class SPSNAL {
     }
     mDecodedNAL = H264::DecodeNALUnit(aPtr, aLength);
     if (mDecodedNAL) {
-      mLength = GetBitLength(mDecodedNAL);
+      mLength = BitReader::GetBitLength(mDecodedNAL);
     }
   }
 
@@ -343,18 +499,53 @@ class SPSNALIterator {
   return rbsp.forget();
 }
 
+// The reverse of DecodeNALUnit. To allow the distinction between Annex B (that
+// uses 0x000001 as marker) and AVCC, the pattern 0x00 0x00 0x0n (where n is
+// between 0 and 3) can't be found in the bytestream. A 0x03 byte is inserted
+// after the second 0. Eg. 0x00 0x00 0x00 becomes 0x00 0x00 0x03 0x00
+/* static */ already_AddRefed<mozilla::MediaByteBuffer> H264::EncodeNALUnit(
+    const uint8_t* aNAL, size_t aLength) {
+  MOZ_ASSERT(aNAL);
+  RefPtr<MediaByteBuffer> rbsp = new MediaByteBuffer();
+  BufferReader reader(aNAL, aLength);
+
+  auto res = reader.ReadU8();
+  if (res.isErr()) {
+    return rbsp.forget();
+  }
+  rbsp->AppendElement(res.unwrap());
+
+  res = reader.ReadU8();
+  if (res.isErr()) {
+    return rbsp.forget();
+  }
+  rbsp->AppendElement(res.unwrap());
+
+  while ((res = reader.ReadU8()).isOk()) {
+    uint8_t val = res.unwrap();
+    if (val <= 0x03 && rbsp->ElementAt(rbsp->Length() - 2) == 0 &&
+        rbsp->ElementAt(rbsp->Length() - 1) == 0) {
+      rbsp->AppendElement(0x03);
+    }
+    rbsp->AppendElement(val);
+  }
+  return rbsp.forget();
+}
+
 static int32_t ConditionDimension(float aValue) {
   // This will exclude NaNs and too-big values.
-  if (aValue > 1.0 && aValue <= INT32_MAX) return int32_t(aValue);
+  if (aValue > 1.0 && aValue <= INT32_MAX / 2) {
+    return int32_t(aValue);
+  }
   return 0;
 }
 
-/* static */ bool H264::DecodeSPS(const mozilla::MediaByteBuffer* aSPS,
-                                  SPSData& aDest) {
+/* static */
+bool H264::DecodeSPS(const mozilla::MediaByteBuffer* aSPS, SPSData& aDest) {
   if (!aSPS) {
     return false;
   }
-  BitReader br(aSPS, GetBitLength(aSPS));
+  BitReader br(aSPS, BitReader::GetBitLength(aSPS));
 
   aDest.profile_idc = br.ReadBits(8);
   aDest.constraint_set0_flag = br.ReadBit();
@@ -523,7 +714,8 @@ static int32_t ConditionDimension(float aValue) {
   return true;
 }
 
-/* static */ bool H264::vui_parameters(BitReader& aBr, SPSData& aDest) {
+/* static */
+bool H264::vui_parameters(BitReader& aBr, SPSData& aDest) {
   aDest.aspect_ratio_info_present_flag = aBr.ReadBit();
   if (aDest.aspect_ratio_info_present_flag) {
     aDest.aspect_ratio_idc = aBr.ReadBits(8);
@@ -700,8 +892,9 @@ static int32_t ConditionDimension(float aValue) {
   return true;
 }
 
-/* static */ bool H264::DecodeSPSFromExtraData(
-    const mozilla::MediaByteBuffer* aExtraData, SPSData& aDest) {
+/* static */
+bool H264::DecodeSPSFromExtraData(const mozilla::MediaByteBuffer* aExtraData,
+                                  SPSData& aDest) {
   SPSNALIterator it(aExtraData);
   if (!it) {
     return false;
@@ -709,7 +902,8 @@ static int32_t ConditionDimension(float aValue) {
   return (*it).GetSPSData(aDest);
 }
 
-/* static */ bool H264::EnsureSPSIsSane(SPSData& aSPS) {
+/* static */
+bool H264::EnsureSPSIsSane(SPSData& aSPS) {
   bool valid = true;
   static const float default_aspect = 4.0f / 3.0f;
   if (aSPS.sample_ratio <= 0.0f || aSPS.sample_ratio > 6.0f) {
@@ -729,8 +923,8 @@ static int32_t ConditionDimension(float aValue) {
   return valid;
 }
 
-/* static */ uint32_t H264::ComputeMaxRefFrames(
-    const mozilla::MediaByteBuffer* aExtraData) {
+/* static */
+uint32_t H264::ComputeMaxRefFrames(const mozilla::MediaByteBuffer* aExtraData) {
   uint32_t maxRefFrames = 4;
   // Retrieve video dimensions from H264 SPS NAL.
   SPSData spsdata;
@@ -814,7 +1008,7 @@ static int32_t ConditionDimension(float aValue) {
   int nalLenSize = ((*aSample->mExtraData)[4] & 3) + 1;
 
   size_t sampleSize = aSample->Size();
-  if (aSample->mCrypto.mValid) {
+  if (aSample->mCrypto.IsEncrypted()) {
     // The content is encrypted, we can only parse the non-encrypted data.
     MOZ_ASSERT(aSample->mCrypto.mPlainSizes.Length() > 0);
     if (aSample->mCrypto.mPlainSizes.Length() == 0 ||
@@ -917,11 +1111,13 @@ static int32_t ConditionDimension(float aValue) {
   return extradata.forget();
 }
 
-/* static */ bool H264::HasSPS(const mozilla::MediaByteBuffer* aExtraData) {
+/* static */
+bool H264::HasSPS(const mozilla::MediaByteBuffer* aExtraData) {
   return NumSPS(aExtraData) > 0;
 }
 
-/* static */ uint8_t H264::NumSPS(const mozilla::MediaByteBuffer* aExtraData) {
+/* static */
+uint8_t H264::NumSPS(const mozilla::MediaByteBuffer* aExtraData) {
   if (!aExtraData || aExtraData->IsEmpty()) {
     return 0;
   }
@@ -937,9 +1133,9 @@ static int32_t ConditionDimension(float aValue) {
   return res.unwrap() & 0x1f;
 }
 
-/* static */ bool H264::CompareExtraData(
-    const mozilla::MediaByteBuffer* aExtraData1,
-    const mozilla::MediaByteBuffer* aExtraData2) {
+/* static */
+bool H264::CompareExtraData(const mozilla::MediaByteBuffer* aExtraData1,
+                            const mozilla::MediaByteBuffer* aExtraData2) {
   if (aExtraData1 == aExtraData2) {
     return true;
   }
@@ -978,8 +1174,9 @@ static inline Result<Ok, nsresult> ReadSEIInt(BufferReader& aBr,
   return Ok();
 }
 
-/* static */ bool H264::DecodeRecoverySEI(const mozilla::MediaByteBuffer* aSEI,
-                                          SEIRecoveryData& aDest) {
+/* static */
+bool H264::DecodeRecoverySEI(const mozilla::MediaByteBuffer* aSEI,
+                             SEIRecoveryData& aDest) {
   if (!aSEI) {
     return false;
   }
@@ -1025,6 +1222,99 @@ static inline Result<Ok, nsresult> ReadSEIInt(BufferReader& aBr,
   return false;
 }
 
+/*static */ already_AddRefed<mozilla::MediaByteBuffer> H264::CreateExtraData(
+    uint8_t aProfile, uint8_t aConstraints, uint8_t aLevel,
+    const gfx::IntSize& aSize) {
+  // SPS of a 144p video.
+  const uint8_t originSPS[] = {0x4d, 0x40, 0x0c, 0xe8, 0x80, 0x80, 0x9d,
+                               0x80, 0xb5, 0x01, 0x01, 0x01, 0x40, 0x00,
+                               0x00, 0x00, 0x40, 0x00, 0x00, 0x0f, 0x03,
+                               0xc5, 0x0a, 0x44, 0x80};
+
+  RefPtr<MediaByteBuffer> extraData = new MediaByteBuffer();
+  extraData->AppendElements(originSPS, sizeof(originSPS));
+  BitReader br(extraData, BitReader::GetBitLength(extraData));
+
+  RefPtr<MediaByteBuffer> sps = new MediaByteBuffer();
+  BitWriter bw(sps);
+
+  br.ReadBits(8);  // Skip original profile_idc
+  bw.WriteU8(aProfile);
+  br.ReadBits(8);  // Skip original constraint flags && reserved_zero_2bits
+  aConstraints =
+      aConstraints & ~0x3;  // Ensure reserved_zero_2bits are set to 0
+  bw.WriteBits(aConstraints, 8);
+  br.ReadBits(8);  // Skip original level_idc
+  bw.WriteU8(aLevel);
+  bw.WriteUE(br.ReadUE());  // seq_parameter_set_id (0 stored on 1 bit)
+
+  if (aProfile == 100 || aProfile == 110 || aProfile == 122 ||
+      aProfile == 244 || aProfile == 44 || aProfile == 83 || aProfile == 86 ||
+      aProfile == 118 || aProfile == 128 || aProfile == 138 ||
+      aProfile == 139 || aProfile == 134) {
+    bw.WriteUE(1);  // chroma_format_idc -> always set to 4:2:0 chroma format
+    bw.WriteUE(0);  // bit_depth_luma_minus8 -> always 8 bits here
+    bw.WriteUE(0);  // bit_depth_chroma_minus8 -> always 8 bits here
+  }
+
+  bw.WriteBits(br.ReadBits(11),
+               11);  // log2_max_frame_num to gaps_in_frame_num_allowed_flag
+
+  // skip over original exp-golomb encoded width/height
+  br.ReadUE();  // skip width
+  br.ReadUE();  // skip height
+  uint32_t width = aSize.width;
+  uint32_t widthNeeded = width % 16 != 0 ? (width / 16 + 1) * 16 : width;
+  uint32_t height = aSize.height;
+  uint32_t heightNeeded = height % 16 != 0 ? (height / 16 + 1) * 16 : height;
+  bw.WriteUE(widthNeeded / 16 - 1);
+  bw.WriteUE(heightNeeded / 16 - 1);
+  bw.WriteBit(br.ReadBit());  // write frame_mbs_only_flag
+  bw.WriteBit(br.ReadBit());  // write direct_8x8_inference_flag;
+  if (widthNeeded != width || heightNeeded != height) {
+    // Write cropping value
+    bw.WriteBit(true);                        // skip frame_cropping_flag
+    bw.WriteUE(0);                            // frame_crop_left_offset
+    bw.WriteUE((widthNeeded - width) / 2);    // frame_crop_right_offset
+    bw.WriteUE(0);                            // frame_crop_top_offset
+    bw.WriteUE((heightNeeded - height) / 2);  // frame_crop_bottom_offset
+  } else {
+    bw.WriteBit(false);  // skip frame_cropping_flag
+  }
+  br.ReadBit();  // skip frame_cropping_flag;
+  // Write the remainings of the original sps (vui_parameters which sets an
+  // aspect ration of 1.0)
+  while (br.BitsLeft()) {
+    bw.WriteBit(br.ReadBit());
+  }
+  bw.CloseWithRbspTrailing();
+
+  RefPtr<MediaByteBuffer> encodedSPS =
+      EncodeNALUnit(sps->Elements(), sps->Length());
+  extraData->Clear();
+  extraData->AppendElement(1);
+  extraData->AppendElement(aProfile);
+  extraData->AppendElement(aConstraints);
+  extraData->AppendElement(aLevel);
+  extraData->AppendElement(3);  // nalLENSize-1
+  extraData->AppendElement(1);  // numPPS
+  uint8_t c[2];
+  mozilla::BigEndian::writeUint16(&c[0], encodedSPS->Length() + 1);
+  extraData->AppendElements(c, 2);
+  extraData->AppendElement((0x00 << 7) | (0x3 << 5) | H264_NAL_SPS);
+  extraData->AppendElements(*encodedSPS);
+
+  const uint8_t PPS[] = {0xeb, 0xef, 0x20};
+
+  extraData->AppendElement(1);  // numPPS
+  mozilla::BigEndian::writeUint16(&c[0], sizeof(PPS) + 1);
+  extraData->AppendElements(c, 2);
+  extraData->AppendElement((0x00 << 7) | (0x3 << 5) | H264_NAL_PPS);
+  extraData->AppendElements(PPS, sizeof(PPS));
+
+  return extraData.forget();
+}
+
 #undef READUE
 #undef READSE
 
