@@ -32,33 +32,40 @@ using media::TimeUnit;
 
 namespace mozilla {
 
-class RemoteVideoDecoder : public RemoteDataDecoder {
+// Hold a reference to the output buffer until we're ready to release it back to
+// the Java codec (for rendering or not).
+class RenderOrReleaseOutput {
  public:
-  // Hold an output buffer and render it to the surface when the frame is sent
-  // to compositor, or release it if not presented.
-  class RenderOrReleaseOutput : public VideoData::Listener {
-   public:
-    RenderOrReleaseOutput(java::CodecProxy::Param aCodec,
-                          java::Sample::Param aSample)
-        : mCodec(aCodec), mSample(aSample) {}
+  RenderOrReleaseOutput(CodecProxy::Param aCodec, Sample::Param aSample)
+      : mCodec(aCodec), mSample(aSample) {}
 
-    ~RenderOrReleaseOutput() { ReleaseOutput(false); }
+  virtual ~RenderOrReleaseOutput() { ReleaseOutput(false); }
 
-    void OnSentToCompositor() override {
-      ReleaseOutput(true);
+ protected:
+  void ReleaseOutput(bool aToRender) {
+    if (mCodec && mSample) {
+      mCodec->ReleaseOutput(mSample, aToRender);
       mCodec = nullptr;
       mSample = nullptr;
     }
+  }
 
-   private:
-    void ReleaseOutput(bool aToRender) {
-      if (mCodec && mSample) {
-        mCodec->ReleaseOutput(mSample, aToRender);
-      }
-    }
+ private:
+  CodecProxy::GlobalRef mCodec;
+  Sample::GlobalRef mSample;
+};
 
-    java::CodecProxy::GlobalRef mCodec;
-    java::Sample::GlobalRef mSample;
+class RemoteVideoDecoder : public RemoteDataDecoder {
+ public:
+  // Render the output to the surface when the frame is sent
+  // to compositor, or release it if not presented.
+  class CompositeListener : private RenderOrReleaseOutput,
+                            public VideoData::Listener {
+   public:
+    CompositeListener(CodecProxy::Param aCodec, Sample::Param aSample)
+        : RenderOrReleaseOutput(aCodec, aSample) {}
+
+    void OnSentToCompositor() override { ReleaseOutput(true); }
   };
 
   class InputInfo {
@@ -78,63 +85,18 @@ class RemoteVideoDecoder : public RemoteDataDecoder {
 
   class CallbacksSupport final : public JavaCallbacksSupport {
    public:
-    CallbacksSupport(RemoteVideoDecoder* aDecoder) : mDecoder(aDecoder) {}
+    explicit CallbacksSupport(RemoteVideoDecoder* aDecoder)
+        : mDecoder(aDecoder) {}
 
     void HandleInput(int64_t aTimestamp, bool aProcessed) override {
       mDecoder->UpdateInputStatus(aTimestamp, aProcessed);
     }
 
-    void HandleOutput(Sample::Param aSample) override {
-      UniquePtr<VideoData::Listener> releaseSample(
-          new RenderOrReleaseOutput(mDecoder->mJavaDecoder, aSample));
-
-      BufferInfo::LocalRef info = aSample->Info();
-
-      int32_t flags;
-      bool ok = NS_SUCCEEDED(info->Flags(&flags));
-
-      int32_t offset;
-      ok &= NS_SUCCEEDED(info->Offset(&offset));
-
-      int32_t size;
-      ok &= NS_SUCCEEDED(info->Size(&size));
-
-      int64_t presentationTimeUs;
-      ok &= NS_SUCCEEDED(info->PresentationTimeUs(&presentationTimeUs));
-
-      if (!ok) {
-        HandleError(MediaResult(NS_ERROR_DOM_MEDIA_FATAL_ERR,
-                                RESULT_DETAIL("VideoCallBack::HandleOutput")));
-        return;
-      }
-
-      InputInfo inputInfo;
-      ok = mDecoder->mInputInfos.Find(presentationTimeUs, inputInfo);
-      bool isEOS = !!(flags & MediaCodec::BUFFER_FLAG_END_OF_STREAM);
-      if (!ok && !isEOS) {
-        // Ignore output with no corresponding input.
-        return;
-      }
-
-      if (ok && (size > 0 || presentationTimeUs >= 0)) {
-        RefPtr<layers::Image> img = new SurfaceTextureImage(
-            mDecoder->mSurfaceHandle, inputInfo.mImageSize,
-            false /* NOT continuous */, gl::OriginPos::BottomLeft);
-
-        RefPtr<VideoData> v = VideoData::CreateFromImage(
-            inputInfo.mDisplaySize, offset,
-            TimeUnit::FromMicroseconds(presentationTimeUs),
-            TimeUnit::FromMicroseconds(inputInfo.mDurationUs), img,
-            !!(flags & MediaCodec::BUFFER_FLAG_SYNC_FRAME),
-            TimeUnit::FromMicroseconds(presentationTimeUs));
-
-        v->SetListener(Move(releaseSample));
-        mDecoder->UpdateOutputStatus(Move(v));
-      }
-
-      if (isEOS) {
-        mDecoder->DrainComplete();
-      }
+    void HandleOutput(Sample::Param aSample,
+                      java::SampleBuffer::Param aBuffer) override {
+      MOZ_ASSERT(!aBuffer, "Video sample should be bufferless");
+      // aSample will be implicitly converted into a GlobalRef.
+      mDecoder->ProcessOutput(std::move(aSample));
     }
 
     void HandleError(const MediaResult& aError) override {
@@ -160,6 +122,12 @@ class RemoteVideoDecoder : public RemoteDataDecoder {
   }
 
   RefPtr<InitPromise> Init() override {
+    BufferInfo::LocalRef bufferInfo;
+    if (NS_FAILED(BufferInfo::New(&bufferInfo)) || !bufferInfo) {
+      return InitPromise::CreateAndReject(NS_ERROR_OUT_OF_MEMORY, __func__);
+    }
+    mInputBufferInfo = bufferInfo;
+
     mSurface = GeckoSurface::LocalRef(SurfaceAllocator::AcquireSurface(
         mConfig.mImage.width, mConfig.mImage.height, false));
     if (!mSurface) {
@@ -173,6 +141,10 @@ class RemoteVideoDecoder : public RemoteDataDecoder {
     JavaCallbacksSupport::Init();
 
     mJavaCallbacks = CodecProxy::NativeCallbacks::New();
+    if (!mJavaCallbacks) {
+      return InitPromise::CreateAndReject(NS_ERROR_DOM_MEDIA_FATAL_ERR,
+                                          __func__);
+    }
     JavaCallbacksSupport::AttachNative(
         mJavaCallbacks, mozilla::MakeUnique<CallbacksSupport>(this));
 
@@ -185,25 +157,35 @@ class RemoteVideoDecoder : public RemoteDataDecoder {
     }
     mIsCodecSupportAdaptivePlayback =
         mJavaDecoder->IsAdaptivePlaybackSupported();
+    mIsHardwareAccelerated = mJavaDecoder->IsHardwareAccelerated();
     return InitPromise::CreateAndResolve(TrackInfo::kVideoTrack, __func__);
   }
 
   RefPtr<MediaDataDecoder::FlushPromise> Flush() override {
-    mInputInfos.Clear();
-    mSeekTarget.reset();
-    return RemoteDataDecoder::Flush();
+    RefPtr<RemoteVideoDecoder> self = this;
+    return InvokeAsync(mTaskQueue, __func__, [self, this]() {
+      mInputInfos.Clear();
+      mSeekTarget.reset();
+      mLatestOutputTime.reset();
+      return RemoteDataDecoder::ProcessFlush();
+    });
   }
 
   RefPtr<MediaDataDecoder::DecodePromise> Decode(
       MediaRawData* aSample) override {
-    const VideoInfo* config =
-        aSample->mTrackInfo ? aSample->mTrackInfo->GetAsVideoInfo() : &mConfig;
-    MOZ_ASSERT(config);
+    RefPtr<RemoteVideoDecoder> self = this;
+    RefPtr<MediaRawData> sample = aSample;
+    return InvokeAsync(mTaskQueue, __func__, [self, sample]() {
+      const VideoInfo* config = sample->mTrackInfo
+                                    ? sample->mTrackInfo->GetAsVideoInfo()
+                                    : &self->mConfig;
+      MOZ_ASSERT(config);
 
-    InputInfo info(aSample->mDuration.ToMicroseconds(), config->mImage,
-                   config->mDisplay);
-    mInputInfos.Insert(aSample->mTime.ToMicroseconds(), info);
-    return RemoteDataDecoder::Decode(aSample);
+      InputInfo info(sample->mDuration.ToMicroseconds(), config->mImage,
+                     config->mDisplay);
+      self->mInputInfos.Insert(sample->mTime.ToMicroseconds(), info);
+      return self->RemoteDataDecoder::ProcessDecode(sample);
+    });
   }
 
   bool SupportDecoderRecycling() const override {
@@ -211,28 +193,131 @@ class RemoteVideoDecoder : public RemoteDataDecoder {
   }
 
   void SetSeekThreshold(const TimeUnit& aTime) override {
-    mSeekTarget = Some(aTime);
+    RefPtr<RemoteVideoDecoder> self = this;
+    nsCOMPtr<nsIRunnable> runnable = NS_NewRunnableFunction(
+        "RemoteVideoDecoder::SetSeekThreshold", [self, aTime]() {
+          if (aTime.IsValid()) {
+            self->mSeekTarget = Some(aTime);
+          } else {
+            self->mSeekTarget.reset();
+          }
+        });
+    nsresult rv = mTaskQueue->Dispatch(runnable.forget());
+    MOZ_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv));
+    Unused << rv;
   }
 
   bool IsUsefulData(const RefPtr<MediaData>& aSample) override {
     AssertOnTaskQueue();
-    if (!mSeekTarget) {
-      return true;
+
+    if (mLatestOutputTime && aSample->mTime < mLatestOutputTime.value()) {
+      return false;
     }
-    if (aSample->GetEndTime() > mSeekTarget.value()) {
-      mSeekTarget.reset();
-      return true;
+
+    const TimeUnit endTime = aSample->GetEndTime();
+    if (mSeekTarget && endTime <= mSeekTarget.value()) {
+      return false;
     }
-    return false;
+
+    mSeekTarget.reset();
+    mLatestOutputTime = Some(endTime);
+    return true;
+  }
+
+  bool IsHardwareAccelerated(nsACString& aFailureReason) const override {
+    return mIsHardwareAccelerated;
+  }
+
+  ConversionRequired NeedsConversion() const override {
+    return ConversionRequired::kNeedAnnexB;
   }
 
  private:
+  // Param and LocalRef are only valid for the duration of a JNI method call.
+  // Use GlobalRef as the parameter type to keep the Java object referenced
+  // until running.
+  void ProcessOutput(Sample::GlobalRef&& aSample) {
+    if (!mTaskQueue->IsCurrentThreadIn()) {
+      nsresult rv = mTaskQueue->Dispatch(NewRunnableMethod<Sample::GlobalRef&&>(
+          "RemoteVideoDecoder::ProcessOutput", this,
+          &RemoteVideoDecoder::ProcessOutput, std::move(aSample)));
+      MOZ_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv));
+      Unused << rv;
+      return;
+    }
+
+    AssertOnTaskQueue();
+    if (GetState() == State::SHUTDOWN) {
+      aSample->Dispose();
+      return;
+    }
+
+    UniquePtr<VideoData::Listener> releaseSample(
+        new CompositeListener(mJavaDecoder, aSample));
+
+    BufferInfo::LocalRef info = aSample->Info();
+    MOZ_ASSERT(info);
+
+    int32_t flags;
+    bool ok = NS_SUCCEEDED(info->Flags(&flags));
+
+    int32_t offset;
+    ok &= NS_SUCCEEDED(info->Offset(&offset));
+
+    int32_t size;
+    ok &= NS_SUCCEEDED(info->Size(&size));
+
+    int64_t presentationTimeUs;
+    ok &= NS_SUCCEEDED(info->PresentationTimeUs(&presentationTimeUs));
+
+    if (!ok) {
+      Error(MediaResult(NS_ERROR_DOM_MEDIA_FATAL_ERR,
+                        RESULT_DETAIL("VideoCallBack::HandleOutput")));
+      return;
+    }
+
+    InputInfo inputInfo;
+    ok = mInputInfos.Find(presentationTimeUs, inputInfo);
+    bool isEOS = !!(flags & MediaCodec::BUFFER_FLAG_END_OF_STREAM);
+    if (!ok && !isEOS) {
+      // Ignore output with no corresponding input.
+      return;
+    }
+
+    if (ok && (size > 0 || presentationTimeUs >= 0)) {
+      RefPtr<layers::Image> img = new SurfaceTextureImage(
+          mSurfaceHandle, inputInfo.mImageSize, false /* NOT continuous */,
+          gl::OriginPos::BottomLeft, mConfig.HasAlpha());
+
+      RefPtr<VideoData> v = VideoData::CreateFromImage(
+          inputInfo.mDisplaySize, offset,
+          TimeUnit::FromMicroseconds(presentationTimeUs),
+          TimeUnit::FromMicroseconds(inputInfo.mDurationUs), img,
+          !!(flags & MediaCodec::BUFFER_FLAG_SYNC_FRAME),
+          TimeUnit::FromMicroseconds(presentationTimeUs));
+
+      v->SetListener(std::move(releaseSample));
+      RemoteDataDecoder::UpdateOutputStatus(std::move(v));
+    }
+
+    if (isEOS) {
+      DrainComplete();
+    }
+  }
+
   const VideoInfo mConfig;
   GeckoSurface::GlobalRef mSurface;
   AndroidSurfaceTextureHandle mSurfaceHandle;
-  SimpleMap<InputInfo> mInputInfos;
+  // Only accessed on reader's task queue.
   bool mIsCodecSupportAdaptivePlayback = false;
+  // Can be accessed on any thread, but only written on during init.
+  bool mIsHardwareAccelerated = false;
+  // Accessed on mTaskQueue and reader's TaskQueue. SimpleMap however is
+  // thread-safe, so it's okay to do so.
+  SimpleMap<InputInfo> mInputInfos;
+  // Only accessed on the TaskQueue.
   Maybe<TimeUnit> mSeekTarget;
+  Maybe<TimeUnit> mLatestOutputTime;
 };
 
 class RemoteAudioDecoder : public RemoteDataDecoder {
@@ -240,8 +325,7 @@ class RemoteAudioDecoder : public RemoteDataDecoder {
   RemoteAudioDecoder(const AudioInfo& aConfig, MediaFormat::Param aFormat,
                      const nsString& aDrmStubId, TaskQueue* aTaskQueue)
       : RemoteDataDecoder(MediaData::Type::AUDIO_DATA, aConfig.mMimeType,
-                          aFormat, aDrmStubId, aTaskQueue),
-        mConfig(aConfig) {
+                          aFormat, aDrmStubId, aTaskQueue) {
     JNIEnv* const env = jni::GetEnvForThread();
 
     bool formatHasCSD = false;
@@ -258,10 +342,20 @@ class RemoteAudioDecoder : public RemoteDataDecoder {
   }
 
   RefPtr<InitPromise> Init() override {
+    BufferInfo::LocalRef bufferInfo;
+    if (NS_FAILED(BufferInfo::New(&bufferInfo)) || !bufferInfo) {
+      return InitPromise::CreateAndReject(NS_ERROR_OUT_OF_MEMORY, __func__);
+    }
+    mInputBufferInfo = bufferInfo;
+
     // Register native methods.
     JavaCallbacksSupport::Init();
 
     mJavaCallbacks = CodecProxy::NativeCallbacks::New();
+    if (!mJavaCallbacks) {
+      return InitPromise::CreateAndReject(NS_ERROR_DOM_MEDIA_FATAL_ERR,
+                                          __func__);
+    }
     JavaCallbacksSupport::AttachNative(
         mJavaCallbacks, mozilla::MakeUnique<CallbacksSupport>(this));
 
@@ -275,83 +369,60 @@ class RemoteAudioDecoder : public RemoteDataDecoder {
     return InitPromise::CreateAndResolve(TrackInfo::kAudioTrack, __func__);
   }
 
-  ConversionRequired NeedsConversion() const override {
-    return ConversionRequired::kNeedAnnexB;
+  RefPtr<FlushPromise> Flush() override {
+    RefPtr<RemoteAudioDecoder> self = this;
+    return InvokeAsync(mTaskQueue, __func__, [self]() {
+      self->mFirstDemuxedSampleTime.reset();
+      return self->RemoteDataDecoder::ProcessFlush();
+    });
+  }
+
+  RefPtr<DecodePromise> Decode(MediaRawData* aSample) override {
+    RefPtr<RemoteAudioDecoder> self = this;
+    RefPtr<MediaRawData> sample = aSample;
+    return InvokeAsync(mTaskQueue, __func__, [self, sample]() {
+      if (!self->mFirstDemuxedSampleTime) {
+        MOZ_ASSERT(sample->mTime.IsValid());
+        self->mFirstDemuxedSampleTime.emplace(sample->mTime);
+      }
+      return self->RemoteDataDecoder::ProcessDecode(sample);
+    });
   }
 
  private:
   class CallbacksSupport final : public JavaCallbacksSupport {
    public:
-    CallbacksSupport(RemoteAudioDecoder* aDecoder) : mDecoder(aDecoder) {}
+    explicit CallbacksSupport(RemoteAudioDecoder* aDecoder)
+        : mDecoder(aDecoder) {}
 
     void HandleInput(int64_t aTimestamp, bool aProcessed) override {
       mDecoder->UpdateInputStatus(aTimestamp, aProcessed);
     }
 
-    void HandleOutput(Sample::Param aSample) override {
-      BufferInfo::LocalRef info = aSample->Info();
-
-      int32_t flags;
-      bool ok = NS_SUCCEEDED(info->Flags(&flags));
-
-      int32_t offset;
-      ok &= NS_SUCCEEDED(info->Offset(&offset));
-
-      int64_t presentationTimeUs;
-      ok &= NS_SUCCEEDED(info->PresentationTimeUs(&presentationTimeUs));
-
-      int32_t size;
-      ok &= NS_SUCCEEDED(info->Size(&size));
-
-      if (!ok) {
-        HandleError(MediaResult(NS_ERROR_DOM_MEDIA_FATAL_ERR,
-                                RESULT_DETAIL("AudioCallBack::HandleOutput")));
-        return;
-      }
-
-      if (size > 0) {
-#ifdef MOZ_SAMPLE_TYPE_S16
-        const int32_t numSamples = size / 2;
-#else
-#error We only support 16-bit integer PCM
-#endif
-
-        const int32_t numFrames = numSamples / mOutputChannels;
-        AlignedAudioBuffer audio(numSamples);
-        if (!audio) {
-          mDecoder->Error(MediaResult(NS_ERROR_OUT_OF_MEMORY, __func__));
-          return;
-        }
-
-        jni::ByteBuffer::LocalRef dest =
-            jni::ByteBuffer::New(audio.get(), size);
-        aSample->WriteToByteBuffer(dest);
-
-        RefPtr<AudioData> data = new AudioData(
-            0, TimeUnit::FromMicroseconds(presentationTimeUs),
-            FramesToTimeUnit(numFrames, mOutputSampleRate), numFrames,
-            Move(audio), mOutputChannels, mOutputSampleRate);
-
-        mDecoder->UpdateOutputStatus(Move(data));
-      }
-
-      if ((flags & MediaCodec::BUFFER_FLAG_END_OF_STREAM) != 0) {
-        mDecoder->DrainComplete();
-      }
+    void HandleOutput(Sample::Param aSample,
+                      java::SampleBuffer::Param aBuffer) override {
+      MOZ_ASSERT(aBuffer, "Audio sample should have buffer");
+      // aSample will be implicitly converted into a GlobalRef.
+      mDecoder->ProcessOutput(std::move(aSample), std::move(aBuffer));
     }
 
     void HandleOutputFormatChanged(MediaFormat::Param aFormat) override {
-      aFormat->GetInteger(NS_LITERAL_STRING("channel-count"), &mOutputChannels);
-      AudioConfig::ChannelLayout layout(mOutputChannels);
+      int32_t outputChannels = 0;
+      aFormat->GetInteger(NS_LITERAL_STRING("channel-count"), &outputChannels);
+      AudioConfig::ChannelLayout layout(outputChannels);
       if (!layout.IsValid()) {
         mDecoder->Error(MediaResult(
             NS_ERROR_DOM_MEDIA_FATAL_ERR,
-            RESULT_DETAIL("Invalid channel layout:%d", mOutputChannels)));
+            RESULT_DETAIL("Invalid channel layout:%d", outputChannels)));
         return;
       }
-      aFormat->GetInteger(NS_LITERAL_STRING("sample-rate"), &mOutputSampleRate);
+
+      int32_t sampleRate = 0;
+      aFormat->GetInteger(NS_LITERAL_STRING("sample-rate"), &sampleRate);
       LOG("Audio output format changed: channels:%d sample rate:%d",
-          mOutputChannels, mOutputSampleRate);
+          outputChannels, sampleRate);
+
+      mDecoder->ProcessOutputFormatChange(outputChannels, sampleRate);
     }
 
     void HandleError(const MediaResult& aError) override {
@@ -360,11 +431,120 @@ class RemoteAudioDecoder : public RemoteDataDecoder {
 
    private:
     RemoteAudioDecoder* mDecoder;
-    int32_t mOutputChannels;
-    int32_t mOutputSampleRate;
   };
 
-  const AudioInfo mConfig;
+  bool IsSampleTimeSmallerThanFirstDemuxedSampleTime(int64_t aTime) const {
+    return mFirstDemuxedSampleTime->ToMicroseconds() > aTime;
+  }
+
+  bool ShouldDiscardSample(int64_t aSession) const {
+    AssertOnTaskQueue();
+    // HandleOutput() runs on Android binder thread pool and could be preempted
+    // by RemoteDateDecoder task queue. That means ProcessOutput() could be
+    // scheduled after ProcessShutdown() or ProcessFlush(). We won't need the
+    // sample which is returned after calling Shutdown() and Flush(). We can
+    // check mFirstDemuxedSampleTime to know whether the Flush() has been
+    // called, becasue it would be reset in Flush().
+    return GetState() == State::SHUTDOWN || !mFirstDemuxedSampleTime ||
+           mSession != aSession;
+  }
+
+  // Param and LocalRef are only valid for the duration of a JNI method call.
+  // Use GlobalRef as the parameter type to keep the Java object referenced
+  // until running.
+  void ProcessOutput(Sample::GlobalRef&& aSample,
+                     SampleBuffer::GlobalRef&& aBuffer) {
+    if (!mTaskQueue->IsCurrentThreadIn()) {
+      nsresult rv = mTaskQueue->Dispatch(
+          NewRunnableMethod<Sample::GlobalRef&&, SampleBuffer::GlobalRef&&>(
+              "RemoteAudioDecoder::ProcessOutput", this,
+              &RemoteAudioDecoder::ProcessOutput, std::move(aSample),
+              std::move(aBuffer)));
+      MOZ_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv));
+      Unused << rv;
+      return;
+    }
+
+    AssertOnTaskQueue();
+
+    if (ShouldDiscardSample(aSample->Session()) || !aBuffer->IsValid()) {
+      aSample->Dispose();
+      return;
+    }
+
+    RenderOrReleaseOutput autoRelease(mJavaDecoder, aSample);
+
+    BufferInfo::LocalRef info = aSample->Info();
+    MOZ_ASSERT(info);
+
+    int32_t flags = 0;
+    bool ok = NS_SUCCEEDED(info->Flags(&flags));
+    bool isEOS = !!(flags & MediaCodec::BUFFER_FLAG_END_OF_STREAM);
+
+    int32_t offset;
+    ok &= NS_SUCCEEDED(info->Offset(&offset));
+
+    int64_t presentationTimeUs;
+    ok &= NS_SUCCEEDED(info->PresentationTimeUs(&presentationTimeUs));
+
+    int32_t size;
+    ok &= NS_SUCCEEDED(info->Size(&size));
+
+    if (!ok ||
+        (IsSampleTimeSmallerThanFirstDemuxedSampleTime(presentationTimeUs) &&
+         !isEOS)) {
+      Error(MediaResult(NS_ERROR_DOM_MEDIA_FATAL_ERR, __func__));
+      return;
+    }
+
+    if (size > 0) {
+#ifdef MOZ_SAMPLE_TYPE_S16
+      const int32_t numSamples = size / 2;
+#else
+#  error We only support 16-bit integer PCM
+#endif
+
+      AlignedAudioBuffer audio(numSamples);
+      if (!audio) {
+        Error(MediaResult(NS_ERROR_OUT_OF_MEMORY, __func__));
+        return;
+      }
+
+      jni::ByteBuffer::LocalRef dest = jni::ByteBuffer::New(audio.get(), size);
+      aBuffer->WriteToByteBuffer(dest, offset, size);
+
+      RefPtr<AudioData> data =
+          new AudioData(0, TimeUnit::FromMicroseconds(presentationTimeUs),
+                        std::move(audio), mOutputChannels, mOutputSampleRate);
+
+      UpdateOutputStatus(std::move(data));
+    }
+
+    if (isEOS) {
+      DrainComplete();
+    }
+  }
+
+  void ProcessOutputFormatChange(int32_t aChannels, int32_t aSampleRate) {
+    if (!mTaskQueue->IsCurrentThreadIn()) {
+      nsresult rv = mTaskQueue->Dispatch(NewRunnableMethod<int32_t, int32_t>(
+          "RemoteAudioDecoder::ProcessOutputFormatChange", this,
+          &RemoteAudioDecoder::ProcessOutputFormatChange, aChannels,
+          aSampleRate));
+      MOZ_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv));
+      Unused << rv;
+      return;
+    }
+
+    AssertOnTaskQueue();
+
+    mOutputChannels = aChannels;
+    mOutputSampleRate = aSampleRate;
+  }
+
+  int32_t mOutputChannels;
+  int32_t mOutputSampleRate;
+  Maybe<TimeUnit> mFirstDemuxedSampleTime;
 };
 
 already_AddRefed<MediaDataDecoder> RemoteDataDecoder::CreateAudioDecoder(
@@ -392,7 +572,7 @@ already_AddRefed<MediaDataDecoder> RemoteDataDecoder::CreateVideoDecoder(
   MediaFormat::LocalRef format;
   NS_ENSURE_SUCCESS(MediaFormat::CreateVideoFormat(
                         TranslateMimeType(config.mMimeType),
-                        config.mDisplay.width, config.mDisplay.height, &format),
+                        config.mImage.width, config.mImage.height, &format),
                     nullptr);
 
   RefPtr<MediaDataDecoder> decoder =
@@ -413,49 +593,51 @@ RemoteDataDecoder::RemoteDataDecoder(MediaData::Type aType,
       mFormat(aFormat),
       mDrmStubId(aDrmStubId),
       mTaskQueue(aTaskQueue),
+      mSession(0),
       mNumPendingInputs(0) {}
 
 RefPtr<MediaDataDecoder::FlushPromise> RemoteDataDecoder::Flush() {
   RefPtr<RemoteDataDecoder> self = this;
-  return InvokeAsync(mTaskQueue, __func__, [self, this]() {
-    mDecodedData.Clear();
-    mNumPendingInputs = 0;
-    mDecodePromise.RejectIfExists(NS_ERROR_DOM_MEDIA_CANCELED, __func__);
-    mDrainPromise.RejectIfExists(NS_ERROR_DOM_MEDIA_CANCELED, __func__);
-    mDrainStatus = DrainStatus::DRAINED;
-    mJavaDecoder->Flush();
-    return FlushPromise::CreateAndResolve(true, __func__);
-  });
+  return InvokeAsync(mTaskQueue, this, __func__,
+                     &RemoteDataDecoder::ProcessFlush);
+}
+
+RefPtr<MediaDataDecoder::FlushPromise> RemoteDataDecoder::ProcessFlush() {
+  AssertOnTaskQueue();
+
+  mDecodedData = DecodedData();
+  UpdatePendingInputStatus(PendingOp::CLEAR);
+  mDecodePromise.RejectIfExists(NS_ERROR_DOM_MEDIA_CANCELED, __func__);
+  mDrainPromise.RejectIfExists(NS_ERROR_DOM_MEDIA_CANCELED, __func__);
+  SetState(State::DRAINED);
+  mJavaDecoder->Flush();
+  return FlushPromise::CreateAndResolve(true, __func__);
 }
 
 RefPtr<MediaDataDecoder::DecodePromise> RemoteDataDecoder::Drain() {
   RefPtr<RemoteDataDecoder> self = this;
   return InvokeAsync(mTaskQueue, __func__, [self, this]() {
-    if (mShutdown) {
+    if (GetState() == State::SHUTDOWN) {
       return DecodePromise::CreateAndReject(NS_ERROR_DOM_MEDIA_CANCELED,
                                             __func__);
     }
     RefPtr<DecodePromise> p = mDrainPromise.Ensure(__func__);
-    if (mDrainStatus == DrainStatus::DRAINED) {
+    if (GetState() == State::DRAINED) {
       // There's no operation to perform other than returning any already
       // decoded data.
       ReturnDecodedData();
       return p;
     }
 
-    if (mDrainStatus == DrainStatus::DRAINING) {
+    if (GetState() == State::DRAINING) {
       // Draining operation already pending, let it complete its course.
       return p;
     }
 
-    BufferInfo::LocalRef bufferInfo;
-    nsresult rv = BufferInfo::New(&bufferInfo);
-    if (NS_FAILED(rv)) {
-      return DecodePromise::CreateAndReject(NS_ERROR_OUT_OF_MEMORY, __func__);
-    }
-    mDrainStatus = DrainStatus::DRAINING;
-    bufferInfo->Set(0, 0, -1, MediaCodec::BUFFER_FLAG_END_OF_STREAM);
-    mJavaDecoder->Input(nullptr, bufferInfo, nullptr);
+    SetState(State::DRAINING);
+    self->mInputBufferInfo->Set(0, 0, -1,
+                                MediaCodec::BUFFER_FLAG_END_OF_STREAM);
+    mSession = mJavaDecoder->Input(nullptr, self->mInputBufferInfo, nullptr);
     return p;
   });
 }
@@ -469,7 +651,7 @@ RefPtr<ShutdownPromise> RemoteDataDecoder::Shutdown() {
 
 RefPtr<ShutdownPromise> RemoteDataDecoder::ProcessShutdown() {
   AssertOnTaskQueue();
-  mShutdown = true;
+  SetState(State::SHUTDOWN);
   if (mJavaDecoder) {
     mJavaDecoder->Release();
     mJavaDecoder = nullptr;
@@ -490,7 +672,7 @@ static CryptoInfo::LocalRef GetCryptoInfoFromSample(
     const MediaRawData* aSample) {
   auto& cryptoObj = aSample->mCrypto;
 
-  if (!cryptoObj.mValid) {
+  if (!cryptoObj.IsEncrypted()) {
     return nullptr;
   }
 
@@ -546,30 +728,44 @@ static CryptoInfo::LocalRef GetCryptoInfoFromSample(
 
 RefPtr<MediaDataDecoder::DecodePromise> RemoteDataDecoder::Decode(
     MediaRawData* aSample) {
-  MOZ_ASSERT(aSample != nullptr);
-
   RefPtr<RemoteDataDecoder> self = this;
   RefPtr<MediaRawData> sample = aSample;
-  return InvokeAsync(mTaskQueue, __func__, [self, sample]() {
-    jni::ByteBuffer::LocalRef bytes = jni::ByteBuffer::New(
-        const_cast<uint8_t*>(sample->Data()), sample->Size());
+  return InvokeAsync(mTaskQueue, __func__,
+                     [self, sample]() { return self->ProcessDecode(sample); });
+}
 
-    BufferInfo::LocalRef bufferInfo;
-    nsresult rv = BufferInfo::New(&bufferInfo);
-    if (NS_FAILED(rv)) {
-      return DecodePromise::CreateAndReject(
-          MediaResult(NS_ERROR_OUT_OF_MEMORY, __func__), __func__);
-    }
-    bufferInfo->Set(0, sample->Size(), sample->mTime.ToMicroseconds(), 0);
+RefPtr<MediaDataDecoder::DecodePromise> RemoteDataDecoder::ProcessDecode(
+    MediaRawData* aSample) {
+  AssertOnTaskQueue();
+  MOZ_ASSERT(aSample != nullptr);
+  jni::ByteBuffer::LocalRef bytes = jni::ByteBuffer::New(
+      const_cast<uint8_t*>(aSample->Data()), aSample->Size());
 
-    self->mDrainStatus = DrainStatus::DRAINABLE;
-    return self->mJavaDecoder->Input(bytes, bufferInfo,
-                                     GetCryptoInfoFromSample(sample))
-               ? self->mDecodePromise.Ensure(__func__)
-               : DecodePromise::CreateAndReject(
-                     MediaResult(NS_ERROR_OUT_OF_MEMORY, __func__), __func__);
+  SetState(State::DRAINABLE);
+  mInputBufferInfo->Set(0, aSample->Size(), aSample->mTime.ToMicroseconds(), 0);
+  int64_t session = mJavaDecoder->Input(bytes, mInputBufferInfo,
+                                        GetCryptoInfoFromSample(aSample));
+  if (session == java::CodecProxy::INVALID_SESSION) {
+    return DecodePromise::CreateAndReject(
+        MediaResult(NS_ERROR_OUT_OF_MEMORY, __func__), __func__);
+  }
+  mSession = session;
+  return mDecodePromise.Ensure(__func__);
+}
 
-  });
+void RemoteDataDecoder::UpdatePendingInputStatus(PendingOp aOp) {
+  AssertOnTaskQueue();
+  switch (aOp) {
+    case PendingOp::INCREASE:
+      mNumPendingInputs++;
+      break;
+    case PendingOp::DECREASE:
+      mNumPendingInputs--;
+      break;
+    case PendingOp::CLEAR:
+      mNumPendingInputs = 0;
+      break;
+  }
 }
 
 void RemoteDataDecoder::UpdateInputStatus(int64_t aTimestamp, bool aProcessed) {
@@ -582,56 +778,45 @@ void RemoteDataDecoder::UpdateInputStatus(int64_t aTimestamp, bool aProcessed) {
     return;
   }
   AssertOnTaskQueue();
-  if (mShutdown) {
+  if (GetState() == State::SHUTDOWN) {
     return;
   }
 
   if (!aProcessed) {
-    mNumPendingInputs++;
-  } else if (mNumPendingInputs > 0) {
-    mNumPendingInputs--;
+    UpdatePendingInputStatus(PendingOp::INCREASE);
+  } else if (HasPendingInputs()) {
+    UpdatePendingInputStatus(PendingOp::DECREASE);
   }
 
-  if (mNumPendingInputs ==
-          0 ||  // Input has been processed, request the next one.
+  if (!HasPendingInputs() ||  // Input has been processed, request the next one.
       !mDecodedData.IsEmpty()) {  // Previous output arrived before Decode().
     ReturnDecodedData();
   }
 }
 
 void RemoteDataDecoder::UpdateOutputStatus(RefPtr<MediaData>&& aSample) {
-  if (!mTaskQueue->IsCurrentThreadIn()) {
-    nsresult rv =
-        mTaskQueue->Dispatch(NewRunnableMethod<const RefPtr<MediaData>>(
-            "RemoteDataDecoder::UpdateOutputStatus", this,
-            &RemoteDataDecoder::UpdateOutputStatus, Move(aSample)));
-    MOZ_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv));
-    Unused << rv;
-    return;
-  }
   AssertOnTaskQueue();
-  if (mShutdown) {
+  if (GetState() == State::SHUTDOWN) {
     return;
   }
   if (IsUsefulData(aSample)) {
-    mDecodedData.AppendElement(Move(aSample));
+    mDecodedData.AppendElement(std::move(aSample));
   }
   ReturnDecodedData();
 }
 
 void RemoteDataDecoder::ReturnDecodedData() {
   AssertOnTaskQueue();
-  MOZ_ASSERT(!mShutdown);
+  MOZ_ASSERT(GetState() != State::SHUTDOWN);
 
   // We only want to clear mDecodedData when we have resolved the promises.
   if (!mDecodePromise.IsEmpty()) {
-    mDecodePromise.Resolve(mDecodedData, __func__);
-    mDecodedData.Clear();
+    mDecodePromise.Resolve(std::move(mDecodedData), __func__);
+    mDecodedData = DecodedData();
   } else if (!mDrainPromise.IsEmpty() &&
-             (!mDecodedData.IsEmpty() ||
-              mDrainStatus == DrainStatus::DRAINED)) {
-    mDrainPromise.Resolve(mDecodedData, __func__);
-    mDecodedData.Clear();
+             (!mDecodedData.IsEmpty() || GetState() == State::DRAINED)) {
+    mDrainPromise.Resolve(std::move(mDecodedData), __func__);
+    mDecodedData = DecodedData();
   }
 }
 
@@ -645,10 +830,10 @@ void RemoteDataDecoder::DrainComplete() {
     return;
   }
   AssertOnTaskQueue();
-  if (mShutdown) {
+  if (GetState() == State::SHUTDOWN) {
     return;
   }
-  mDrainStatus = DrainStatus::DRAINED;
+  SetState(State::DRAINED);
   ReturnDecodedData();
   // Make decoder accept input again.
   mJavaDecoder->Flush();
@@ -663,7 +848,7 @@ void RemoteDataDecoder::Error(const MediaResult& aError) {
     return;
   }
   AssertOnTaskQueue();
-  if (mShutdown) {
+  if (GetState() == State::SHUTDOWN) {
     return;
   }
   mDecodePromise.RejectIfExists(aError, __func__);
