@@ -15,8 +15,8 @@
 #include "mozilla/IntegerPrintfMacros.h"
 
 #ifdef LOG
-#undef LOG
-#undef LOGL
+#  undef LOG
+#  undef LOGL
 #endif
 
 #define LOG(arg, ...)                                                  \
@@ -92,9 +92,11 @@ class MediaDataHelper {
 };
 
 OmxDataDecoder::OmxDataDecoder(const TrackInfo& aTrackInfo,
+                               TaskQueue* aTaskQueue,
                                layers::ImageContainer* aImageContainer)
     : mOmxTaskQueue(
           CreateMediaDecodeTaskQueue("OmxDataDecoder::mOmxTaskQueue")),
+      mTaskQueue(aTaskQueue),
       mImageContainer(aImageContainer),
       mWatchManager(this, mOmxTaskQueue),
       mOmxState(OMX_STATETYPE::OMX_StateInvalid, "OmxDataDecoder::mOmxState"),
@@ -122,13 +124,9 @@ void OmxDataDecoder::EndOfStream() {
   RefPtr<OmxDataDecoder> self = this;
   mOmxLayer->SendCommand(OMX_CommandFlush, OMX_ALL, nullptr)
       ->Then(mOmxTaskQueue, __func__,
-             [self, this]() {
-               mDrainPromise.ResolveIfExists(mDecodedData, __func__);
-               mDecodedData.Clear();
-             },
-             [self, this]() {
-               mDrainPromise.ResolveIfExists(mDecodedData, __func__);
-               mDecodedData.Clear();
+             [self, this](OmxCommandPromise::ResolveOrRejectValue&& aValue) {
+               mDrainPromise.ResolveIfExists(std::move(mDecodedData), __func__);
+               mDecodedData = DecodedData();
              });
 }
 
@@ -141,15 +139,16 @@ RefPtr<MediaDataDecoder::InitPromise> OmxDataDecoder::Init() {
 
     RefPtr<InitPromise> p = mInitPromise.Ensure(__func__);
     mOmxLayer->Init(mTrackInfo.get())
-        ->Then(mOmxTaskQueue, __func__,
-               [self, this]() {
-                 // Omx state should be OMX_StateIdle.
-                 mOmxState = mOmxLayer->GetState();
-                 MOZ_ASSERT(mOmxState != OMX_StateIdle);
-               },
-               [self, this]() {
-                 RejectInitPromise(NS_ERROR_DOM_MEDIA_FATAL_ERR, __func__);
-               });
+        ->Then(
+            mOmxTaskQueue, __func__,
+            [self, this]() {
+              // Omx state should be OMX_StateIdle.
+              mOmxState = mOmxLayer->GetState();
+              MOZ_ASSERT(mOmxState != OMX_StateIdle);
+            },
+            [self, this]() {
+              RejectInitPromise(NS_ERROR_DOM_MEDIA_FATAL_ERR, __func__);
+            });
     return p;
   });
 }
@@ -163,7 +162,7 @@ RefPtr<MediaDataDecoder::DecodePromise> OmxDataDecoder::Decode(
   RefPtr<MediaRawData> sample = aSample;
   return InvokeAsync(mOmxTaskQueue, __func__, [self, this, sample]() {
     RefPtr<DecodePromise> p = mDecodePromise.Ensure(__func__);
-    mMediaRawDatas.AppendElement(Move(sample));
+    mMediaRawDatas.AppendElement(std::move(sample));
 
     // Start to fill/empty buffers.
     if (mOmxState == OMX_StateIdle || mOmxState == OMX_StateExecuting) {
@@ -213,61 +212,68 @@ RefPtr<ShutdownPromise> OmxDataDecoder::DoAsyncShutdown() {
   // Flush to all ports, so all buffers can be returned from component.
   RefPtr<OmxDataDecoder> self = this;
   mOmxLayer->SendCommand(OMX_CommandFlush, OMX_ALL, nullptr)
-      ->Then(mOmxTaskQueue, __func__,
-             [self]() -> RefPtr<OmxCommandPromise> {
-               LOGL("DoAsyncShutdown: flush complete");
-               return self->mOmxLayer->SendCommand(OMX_CommandStateSet,
-                                                   OMX_StateIdle, nullptr);
-             },
-             [self](const OmxCommandFailureHolder& aError) {
-               self->mOmxLayer->Shutdown();
-               return OmxCommandPromise::CreateAndReject(aError, __func__);
-             })
-      ->Then(mOmxTaskQueue, __func__,
-             [self]() -> RefPtr<OmxCommandPromise> {
-               RefPtr<OmxCommandPromise> p = self->mOmxLayer->SendCommand(
-                   OMX_CommandStateSet, OMX_StateLoaded, nullptr);
-
-               // According to spec 3.1.1.2.2.1:
-               // OMX_StateLoaded needs to be sent before releasing buffers.
-               // And state transition from OMX_StateIdle to OMX_StateLoaded
-               // is completed when all of the buffers have been removed
-               // from the component.
-               // Here the buffer promises are not resolved due to displaying
-               // in layer, it needs to wait before the layer returns the
-               // buffers.
-               LOGL("DoAsyncShutdown: releasing buffers...");
-               self->ReleaseBuffers(OMX_DirInput);
-               self->ReleaseBuffers(OMX_DirOutput);
-
-               return p;
-             },
-             [self](const OmxCommandFailureHolder& aError) {
-               self->mOmxLayer->Shutdown();
-               return OmxCommandPromise::CreateAndReject(aError, __func__);
-             })
       ->Then(
           mOmxTaskQueue, __func__,
-          [self]() {
+          [self]() -> RefPtr<OmxCommandPromise> {
+            LOGL("DoAsyncShutdown: flush complete");
+            return self->mOmxLayer->SendCommand(OMX_CommandStateSet,
+                                                OMX_StateIdle, nullptr);
+          },
+          [self](const OmxCommandFailureHolder& aError) {
+            self->mOmxLayer->Shutdown();
+            return OmxCommandPromise::CreateAndReject(aError, __func__);
+          })
+      ->Then(
+          mOmxTaskQueue, __func__,
+          [self]() -> RefPtr<OmxCommandPromise> {
+            RefPtr<OmxCommandPromise> p = self->mOmxLayer->SendCommand(
+                OMX_CommandStateSet, OMX_StateLoaded, nullptr);
+
+            // According to spec 3.1.1.2.2.1:
+            // OMX_StateLoaded needs to be sent before releasing buffers.
+            // And state transition from OMX_StateIdle to OMX_StateLoaded
+            // is completed when all of the buffers have been removed
+            // from the component.
+            // Here the buffer promises are not resolved due to displaying
+            // in layer, it needs to wait before the layer returns the
+            // buffers.
+            LOGL("DoAsyncShutdown: releasing buffers...");
+            self->ReleaseBuffers(OMX_DirInput);
+            self->ReleaseBuffers(OMX_DirOutput);
+
+            return p;
+          },
+          [self](const OmxCommandFailureHolder& aError) {
+            self->mOmxLayer->Shutdown();
+            return OmxCommandPromise::CreateAndReject(aError, __func__);
+          })
+      ->Then(
+          mOmxTaskQueue, __func__,
+          [self]() -> RefPtr<ShutdownPromise> {
             LOGL(
                 "DoAsyncShutdown: OMX_StateLoaded, it is safe to shutdown omx");
             self->mOmxLayer->Shutdown();
             self->mWatchManager.Shutdown();
             self->mOmxLayer = nullptr;
             self->mMediaDataHelper = nullptr;
-
             self->mShuttingDown = false;
+            return ShutdownPromise::CreateAndResolve(true, __func__);
+          },
+          [self]() -> RefPtr<ShutdownPromise> {
+            self->mOmxLayer->Shutdown();
+            self->mWatchManager.Shutdown();
+            self->mOmxLayer = nullptr;
+            self->mMediaDataHelper = nullptr;
+            return ShutdownPromise::CreateAndReject(false, __func__);
+          })
+      ->Then(
+          mTaskQueue, __func__,
+          [self]() {
             self->mOmxTaskQueue->BeginShutdown();
             self->mOmxTaskQueue->AwaitShutdownAndIdle();
             self->mShutdownPromise.Resolve(true, __func__);
           },
           [self]() {
-            self->mOmxLayer->Shutdown();
-            self->mWatchManager.Shutdown();
-            self->mOmxLayer = nullptr;
-            self->mMediaDataHelper = nullptr;
-
-            self->mShuttingDown = false;
             self->mOmxTaskQueue->BeginShutdown();
             self->mOmxTaskQueue->AwaitShutdownAndIdle();
             self->mShutdownPromise.Resolve(true, __func__);
@@ -323,23 +329,24 @@ void OmxDataDecoder::Output(BufferData* aData) {
 
     RefPtr<OmxDataDecoder> self = this;
     RefPtr<BufferData> buffer = aData;
-    p->Then(mOmxTaskQueue, __func__,
-            [self, buffer]() {
-              MOZ_RELEASE_ASSERT(buffer->mStatus ==
-                                 BufferData::BufferStatus::OMX_CLIENT_OUTPUT);
-              buffer->mStatus = BufferData::BufferStatus::FREE;
-              self->FillAndEmptyBuffers();
-            },
-            [buffer]() {
-              MOZ_RELEASE_ASSERT(buffer->mStatus ==
-                                 BufferData::BufferStatus::OMX_CLIENT_OUTPUT);
-              buffer->mStatus = BufferData::BufferStatus::FREE;
-            });
+    p->Then(
+        mOmxTaskQueue, __func__,
+        [self, buffer]() {
+          MOZ_RELEASE_ASSERT(buffer->mStatus ==
+                             BufferData::BufferStatus::OMX_CLIENT_OUTPUT);
+          buffer->mStatus = BufferData::BufferStatus::FREE;
+          self->FillAndEmptyBuffers();
+        },
+        [buffer]() {
+          MOZ_RELEASE_ASSERT(buffer->mStatus ==
+                             BufferData::BufferStatus::OMX_CLIENT_OUTPUT);
+          buffer->mStatus = BufferData::BufferStatus::FREE;
+        });
   } else {
     aData->mStatus = BufferData::BufferStatus::FREE;
   }
 
-  mDecodedData.AppendElement(Move(data));
+  mDecodedData.AppendElement(std::move(data));
 }
 
 void OmxDataDecoder::FillBufferFailure(OmxBufferFailureHolder aFailureHolder) {
@@ -369,8 +376,8 @@ void OmxDataDecoder::EmptyBufferDone(BufferData* aData) {
             return;
           }
 
-          mDecodePromise.ResolveIfExists(mDecodedData, __func__);
-          mDecodedData.Clear();
+          mDecodePromise.ResolveIfExists(std::move(mDecodedData), __func__);
+          mDecodedData = DecodedData();
         });
 
     nsresult rv = mOmxTaskQueue->Dispatch(r.forget());
@@ -387,7 +394,7 @@ void OmxDataDecoder::NotifyError(OMX_ERRORTYPE aOmxError, const char* aLine,
                                  const MediaResult& aError) {
   LOG("NotifyError %d (%s) at %s", static_cast<int>(aOmxError),
       aError.ErrorName().get(), aLine);
-  mDecodedData.Clear();
+  mDecodedData = DecodedData();
   mDecodePromise.RejectIfExists(aError, __func__);
   mDrainPromise.RejectIfExists(aError, __func__);
   mFlushPromise.RejectIfExists(aError, __func__);
@@ -512,16 +519,16 @@ void OmxDataDecoder::OmxStateRunner() {
     // Send OpenMax state command to OMX_StateIdle.
     RefPtr<OmxDataDecoder> self = this;
     mOmxLayer->SendCommand(OMX_CommandStateSet, OMX_StateIdle, nullptr)
-        ->Then(mOmxTaskQueue, __func__,
-               [self]() {
-                 // Current state should be OMX_StateIdle.
-                 self->mOmxState = self->mOmxLayer->GetState();
-                 MOZ_ASSERT(self->mOmxState == OMX_StateIdle);
-               },
-               [self]() {
-                 self->RejectInitPromise(NS_ERROR_DOM_MEDIA_FATAL_ERR,
-                                         __func__);
-               });
+        ->Then(
+            mOmxTaskQueue, __func__,
+            [self]() {
+              // Current state should be OMX_StateIdle.
+              self->mOmxState = self->mOmxLayer->GetState();
+              MOZ_ASSERT(self->mOmxState == OMX_StateIdle);
+            },
+            [self]() {
+              self->RejectInitPromise(NS_ERROR_DOM_MEDIA_FATAL_ERR, __func__);
+            });
 
     // Allocate input and output buffers.
     OMX_DIRTYPE types[] = {OMX_DIRTYPE::OMX_DirInput,
@@ -536,17 +543,17 @@ void OmxDataDecoder::OmxStateRunner() {
   } else if (mOmxState == OMX_StateIdle) {
     RefPtr<OmxDataDecoder> self = this;
     mOmxLayer->SendCommand(OMX_CommandStateSet, OMX_StateExecuting, nullptr)
-        ->Then(mOmxTaskQueue, __func__,
-               [self]() {
-                 self->mOmxState = self->mOmxLayer->GetState();
-                 MOZ_ASSERT(self->mOmxState == OMX_StateExecuting);
+        ->Then(
+            mOmxTaskQueue, __func__,
+            [self]() {
+              self->mOmxState = self->mOmxLayer->GetState();
+              MOZ_ASSERT(self->mOmxState == OMX_StateExecuting);
 
-                 self->ResolveInitPromise(__func__);
-               },
-               [self]() {
-                 self->RejectInitPromise(NS_ERROR_DOM_MEDIA_FATAL_ERR,
-                                         __func__);
-               });
+              self->ResolveInitPromise(__func__);
+            },
+            [self]() {
+              self->RejectInitPromise(NS_ERROR_DOM_MEDIA_FATAL_ERR, __func__);
+            });
   } else if (mOmxState == OMX_StateExecuting) {
     // Configure codec once it gets OMX_StateExecuting state.
     FillCodecConfigDataToOmx();
@@ -691,8 +698,8 @@ OmxDataDecoder::CollectBufferPromises(OMX_DIRTYPE aType) {
     return OmxBufferPromise::All(mOmxTaskQueue, promises);
   }
 
-  nsTArray<BufferData*> headers;
-  return OmxBufferPromise::AllPromiseType::CreateAndResolve(headers, __func__);
+  return OmxBufferPromise::AllPromiseType::CreateAndResolve(
+      nsTArray<BufferData*>(), __func__);
 }
 
 void OmxDataDecoder::PortSettingsChanged() {
@@ -726,35 +733,36 @@ void OmxDataDecoder::PortSettingsChanged() {
     LOG("PortSettingsChanged: disable port %lu", def.nPortIndex);
     mOmxLayer
         ->SendCommand(OMX_CommandPortDisable, mPortSettingsChanged, nullptr)
-        ->Then(mOmxTaskQueue, __func__,
-               [self, def]() -> RefPtr<OmxCommandPromise> {
-                 // 3. enable port.
-                 // Send enable port command.
-                 RefPtr<OmxCommandPromise> p = self->mOmxLayer->SendCommand(
-                     OMX_CommandPortEnable, self->mPortSettingsChanged,
-                     nullptr);
+        ->Then(
+            mOmxTaskQueue, __func__,
+            [self, def]() -> RefPtr<OmxCommandPromise> {
+              // 3. enable port.
+              // Send enable port command.
+              RefPtr<OmxCommandPromise> p = self->mOmxLayer->SendCommand(
+                  OMX_CommandPortEnable, self->mPortSettingsChanged, nullptr);
 
-                 // 4. allocate port buffers.
-                 // Allocate new port buffers.
-                 nsresult rv = self->AllocateBuffers(def.eDir);
-                 if (NS_FAILED(rv)) {
-                   self->NotifyError(OMX_ErrorUndefined, __func__);
-                 }
+              // 4. allocate port buffers.
+              // Allocate new port buffers.
+              nsresult rv = self->AllocateBuffers(def.eDir);
+              if (NS_FAILED(rv)) {
+                self->NotifyError(OMX_ErrorUndefined, __func__);
+              }
 
-                 return p;
-               },
-               [self](const OmxCommandFailureHolder& aError) {
-                 self->NotifyError(OMX_ErrorUndefined, __func__);
-                 return OmxCommandPromise::CreateAndReject(aError, __func__);
-               })
-        ->Then(mOmxTaskQueue, __func__,
-               [self]() {
-                 LOGL("PortSettingsChanged: port settings changed complete");
-                 // finish port setting changed.
-                 self->mPortSettingsChanged = -1;
-                 self->FillAndEmptyBuffers();
-               },
-               [self]() { self->NotifyError(OMX_ErrorUndefined, __func__); });
+              return p;
+            },
+            [self](const OmxCommandFailureHolder& aError) {
+              self->NotifyError(OMX_ErrorUndefined, __func__);
+              return OmxCommandPromise::CreateAndReject(aError, __func__);
+            })
+        ->Then(
+            mOmxTaskQueue, __func__,
+            [self]() {
+              LOGL("PortSettingsChanged: port settings changed complete");
+              // finish port setting changed.
+              self->mPortSettingsChanged = -1;
+              self->FillAndEmptyBuffers();
+            },
+            [self]() { self->NotifyError(OMX_ErrorUndefined, __func__); });
 
     // 2. wait for port buffers return to client and then release these buffers.
     //
@@ -789,7 +797,7 @@ void OmxDataDecoder::SendEosBuffer() {
 RefPtr<MediaDataDecoder::FlushPromise> OmxDataDecoder::DoFlush() {
   MOZ_ASSERT(mOmxTaskQueue->IsCurrentThreadIn());
 
-  mDecodedData.Clear();
+  mDecodedData = DecodedData();
   mDecodePromise.RejectIfExists(NS_ERROR_DOM_MEDIA_CANCELED, __func__);
   mDrainPromise.RejectIfExists(NS_ERROR_DOM_MEDIA_CANCELED, __func__);
 
