@@ -18,14 +18,13 @@
 #include "mozilla/Preferences.h"
 #include "nsIPermissionManager.h"
 #include "nsThreadUtils.h"
-#include "nsXPCOM.h"
 #include "nsNetUtil.h"
 
 #include "webrtc/common_video/libyuv/include/webrtc_libyuv.h"
 
 #if defined(_WIN32)
-#include <process.h>
-#define getpid() _getpid()
+#  include <process.h>
+#  define getpid() _getpid()
 #endif
 
 #undef LOG
@@ -38,6 +37,8 @@ mozilla::LazyLogModule gCamerasParentLog("CamerasParent");
 #define LOG_ENABLED() MOZ_LOG_TEST(gCamerasParentLog, mozilla::LogLevel::Debug)
 
 namespace mozilla {
+using media::GetShutdownBarrier;
+using media::NewRunnableFrom;
 namespace camera {
 
 std::map<uint32_t, const char*> sDeviceUniqueIDs;
@@ -105,14 +106,13 @@ void InputObserver::OnDeviceChange() {
   MOZ_ASSERT(mParent);
 
   RefPtr<InputObserver> self(this);
-  RefPtr<nsIRunnable> ipc_runnable =
-      media::NewRunnableFrom([self]() -> nsresult {
-        if (self->mParent->IsShuttingDown()) {
-          return NS_ERROR_FAILURE;
-        }
-        Unused << self->mParent->SendDeviceChange();
-        return NS_OK;
-      });
+  RefPtr<nsIRunnable> ipc_runnable = NewRunnableFrom([self]() {
+    if (self->mParent->IsShuttingDown()) {
+      return NS_ERROR_FAILURE;
+    }
+    Unused << self->mParent->SendDeviceChange();
+    return NS_OK;
+  });
 
   nsIEventTarget* target = mParent->GetBackgroundEventTarget();
   MOZ_ASSERT(target != nullptr);
@@ -128,7 +128,8 @@ class DeliverFrameRunnable : public mozilla::Runnable {
         mParent(aParent),
         mCapEngine(aEngine),
         mStreamId(aStreamId),
-        mProperties(aProperties) {
+        mProperties(aProperties),
+        mResult(0) {
     // No ShmemBuffer (of the right size) was available, so make an
     // extra buffer here.  We have no idea when we are going to run and
     // it will be potentially long after the webrtc frame callback has
@@ -147,8 +148,9 @@ class DeliverFrameRunnable : public mozilla::Runnable {
         mParent(aParent),
         mCapEngine(aEngine),
         mStreamId(aStreamId),
-        mBuffer(Move(aBuffer)),
-        mProperties(aProperties){};
+        mBuffer(std::move(aBuffer)),
+        mProperties(aProperties),
+        mResult(0){};
 
   NS_IMETHOD Run() override {
     if (mParent->IsShuttingDown()) {
@@ -156,7 +158,7 @@ class DeliverFrameRunnable : public mozilla::Runnable {
       mResult = 0;
       return NS_OK;
     }
-    if (!mParent->DeliverFrameOverIPC(mCapEngine, mStreamId, Move(mBuffer),
+    if (!mParent->DeliverFrameOverIPC(mCapEngine, mStreamId, std::move(mBuffer),
                                       mAlternateBuffer.get(), mProperties)) {
       mResult = -1;
     } else {
@@ -177,20 +179,9 @@ class DeliverFrameRunnable : public mozilla::Runnable {
   int mResult;
 };
 
-NS_IMPL_ISUPPORTS(CamerasParent, nsIObserver)
+NS_IMPL_ISUPPORTS(CamerasParent, nsIAsyncShutdownBlocker)
 
-NS_IMETHODIMP
-CamerasParent::Observe(nsISupports* aSubject, const char* aTopic,
-                       const char16_t* aData) {
-  MOZ_ASSERT(!strcmp(aTopic, NS_XPCOM_WILL_SHUTDOWN_OBSERVER_ID));
-  nsCOMPtr<nsIObserverService> obs = services::GetObserverService();
-  MOZ_ASSERT(obs);
-  obs->RemoveObserver(this, NS_XPCOM_WILL_SHUTDOWN_OBSERVER_ID);
-  StopVideoCapture();
-  return NS_OK;
-}
-
-nsresult CamerasParent::DispatchToVideoCaptureThread(Runnable* event) {
+nsresult CamerasParent::DispatchToVideoCaptureThread(RefPtr<Runnable> event) {
   // Don't try to dispatch if we're already on the right thread.
   // There's a potential deadlock because the sThreadMonitor is likely
   // to be taken already.
@@ -205,8 +196,7 @@ nsresult CamerasParent::DispatchToVideoCaptureThread(Runnable* event) {
   if (!sVideoCaptureThread || !sVideoCaptureThread->IsRunning()) {
     return NS_ERROR_FAILURE;
   }
-  RefPtr<Runnable> addrefedEvent = event;
-  sVideoCaptureThread->message_loop()->PostTask(addrefedEvent.forget());
+  sVideoCaptureThread->message_loop()->PostTask(event.forget());
   return NS_OK;
 }
 
@@ -216,43 +206,39 @@ void CamerasParent::StopVideoCapture() {
   // from PBackground (when the Actor shuts down).
   // Shut down the WebRTC stack (on the capture thread)
   RefPtr<CamerasParent> self(this);
-  RefPtr<Runnable> webrtc_runnable =
-      media::NewRunnableFrom([self]() -> nsresult {
+  DebugOnly<nsresult> rv =
+      DispatchToVideoCaptureThread(NewRunnableFrom([self]() {
         MonitorAutoLock lock(*(self->sThreadMonitor));
         self->CloseEngines();
-        self->sThreadMonitor->NotifyAll();
-        return NS_OK;
-      });
-  DebugOnly<nsresult> rv = DispatchToVideoCaptureThread(webrtc_runnable);
+        // After closing the WebRTC stack, clean up the
+        // VideoCapture thread.
+        base::Thread* thread = nullptr;
+        if (sNumOfOpenCamerasParentEngines == 0 && self->sVideoCaptureThread) {
+          thread = self->sVideoCaptureThread;
+          self->sVideoCaptureThread = nullptr;
+        }
+        nsresult rv = NS_DispatchToMainThread(NewRunnableFrom([self, thread]() {
+          if (thread) {
+            if (thread->IsRunning()) {
+              thread->Stop();
+            }
+            delete thread;
+          }
+          nsresult rv = GetShutdownBarrier()->RemoveBlocker(self);
+          MOZ_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv));
+          Unused << rv;
+          return NS_OK;
+        }));
+        if (NS_FAILED(rv)) {
+          LOG(("Could not dispatch VideoCaptureThread destruction"));
+        }
+        return rv;
+      }));
 #ifdef DEBUG
   // It's ok for the dispatch to fail if the cleanup it has to do
   // has been done already.
   MOZ_ASSERT(NS_SUCCEEDED(rv) || !mWebRTCAlive);
 #endif
-  // Hold here until the WebRTC thread is gone. We need to dispatch
-  // the thread deletion *now*, or there will be no more possibility
-  // to get to the main thread.
-  MonitorAutoLock lock(*sThreadMonitor);
-  while (mWebRTCAlive) {
-    sThreadMonitor->Wait();
-  }
-  // After closing the WebRTC stack, clean up the
-  // VideoCapture thread.
-  if (sNumOfOpenCamerasParentEngines == 0 && self->sVideoCaptureThread) {
-    base::Thread* thread = self->sVideoCaptureThread;
-    self->sVideoCaptureThread = nullptr;
-    RefPtr<Runnable> threadShutdown =
-        media::NewRunnableFrom([thread]() -> nsresult {
-          if (thread->IsRunning()) {
-            thread->Stop();
-          }
-          delete thread;
-          return NS_OK;
-        });
-    if (NS_FAILED(NS_DispatchToMainThread(threadShutdown))) {
-      LOG(("Could not dispatch VideoCaptureThread destruction"));
-    }
-  }
 }
 
 int CamerasParent::DeliverFrameOverIPC(CaptureEngine capEng, uint32_t aStreamId,
@@ -276,14 +262,15 @@ int CamerasParent::DeliverFrameOverIPC(CaptureEngine capEng, uint32_t aStreamId,
     // get() and Size() check for proper alignment of the segment
     memcpy(shMemBuff.GetBytes(), altbuffer, aProps.bufferSize());
 
-    if (!SendDeliverFrame(capEng, aStreamId, shMemBuff.Get(), aProps)) {
+    if (!SendDeliverFrame(capEng, aStreamId, std::move(shMemBuff.Get()),
+                          aProps)) {
       return -1;
     }
   } else {
     MOZ_ASSERT(buffer.Valid());
     // ShmemBuffer was available, we're all good. A single copy happened
     // in the original webrtc callback.
-    if (!SendDeliverFrame(capEng, aStreamId, buffer.Get(), aProps)) {
+    if (!SendDeliverFrame(capEng, aStreamId, std::move(buffer.Get()), aProps)) {
       return -1;
     }
   }
@@ -313,7 +300,7 @@ void CallbackHelper::OnFrame(const webrtc::VideoFrame& aVideoFrame) {
     VideoFrameUtils::CopyVideoFrameBuffers(
         shMemBuffer.GetBytes(), properties.bufferSize(), aVideoFrame);
     runnable = new DeliverFrameRunnable(mParent, mCapEngine, mStreamId,
-                                        Move(shMemBuffer), properties);
+                                        std::move(shMemBuffer), properties);
   }
   if (!runnable) {
     runnable = new DeliverFrameRunnable(mParent, mCapEngine, mStreamId,
@@ -352,10 +339,6 @@ bool CamerasParent::SetupEngine(CaptureEngine aCapEngine) {
         captureDeviceInfo = MakeUnique<webrtc::CaptureDeviceInfo>(
             webrtc::CaptureDeviceType::Window);
         break;
-      case AppEngine:
-        captureDeviceInfo = MakeUnique<webrtc::CaptureDeviceInfo>(
-            webrtc::CaptureDeviceType::Application);
-        break;
       case CameraEngine:
         captureDeviceInfo = MakeUnique<webrtc::CaptureDeviceInfo>(
             webrtc::CaptureDeviceType::Camera);
@@ -367,7 +350,7 @@ bool CamerasParent::SetupEngine(CaptureEngine aCapEngine) {
     }
 
     config->Set<webrtc::CaptureDeviceInfo>(captureDeviceInfo.release());
-    engine = VideoEngine::Create(Move(config));
+    engine = VideoEngine::Create(std::move(config));
 
     if (!engine) {
       LOG(("VideoEngine::Create failed"));
@@ -452,34 +435,31 @@ mozilla::ipc::IPCResult CamerasParent::RecvNumberOfCaptureDevices(
   LOG((__PRETTY_FUNCTION__));
   LOG(("CaptureEngine=%d", aCapEngine));
   RefPtr<CamerasParent> self(this);
-  RefPtr<Runnable> webrtc_runnable =
-      media::NewRunnableFrom([self, aCapEngine]() -> nsresult {
-        int num = -1;
-        if (auto engine = self->EnsureInitialized(aCapEngine)) {
-          if (auto devInfo = engine->GetOrCreateVideoCaptureDeviceInfo()) {
-            num = devInfo->NumberOfDevices();
-          }
-        }
-        RefPtr<nsIRunnable> ipc_runnable =
-            media::NewRunnableFrom([self, num]() -> nsresult {
-              if (!self->mChildIsAlive) {
-                return NS_ERROR_FAILURE;
-              }
+  RefPtr<Runnable> webrtc_runnable = NewRunnableFrom([self, aCapEngine]() {
+    int num = -1;
+    if (auto engine = self->EnsureInitialized(aCapEngine)) {
+      if (auto devInfo = engine->GetOrCreateVideoCaptureDeviceInfo()) {
+        num = devInfo->NumberOfDevices();
+      }
+    }
+    RefPtr<nsIRunnable> ipc_runnable = NewRunnableFrom([self, num]() {
+      if (!self->mChildIsAlive) {
+        return NS_ERROR_FAILURE;
+      }
 
-              if (num < 0) {
-                LOG(("RecvNumberOfCaptureDevices couldn't find devices"));
-                Unused << self->SendReplyFailure();
-                return NS_ERROR_FAILURE;
-              }
+      if (num < 0) {
+        LOG(("RecvNumberOfCaptureDevices couldn't find devices"));
+        Unused << self->SendReplyFailure();
+        return NS_ERROR_FAILURE;
+      }
 
-              LOG(("RecvNumberOfCaptureDevices: %d", num));
-              Unused << self->SendReplyNumberOfCaptureDevices(num);
-              return NS_OK;
-            });
-        self->mPBackgroundEventTarget->Dispatch(ipc_runnable,
-                                                NS_DISPATCH_NORMAL);
-        return NS_OK;
-      });
+      LOG(("RecvNumberOfCaptureDevices: %d", num));
+      Unused << self->SendReplyNumberOfCaptureDevices(num);
+      return NS_OK;
+    });
+    self->mPBackgroundEventTarget->Dispatch(ipc_runnable, NS_DISPATCH_NORMAL);
+    return NS_OK;
+  });
   DispatchToVideoCaptureThread(webrtc_runnable);
   return IPC_OK();
 }
@@ -489,30 +469,27 @@ mozilla::ipc::IPCResult CamerasParent::RecvEnsureInitialized(
   LOG((__PRETTY_FUNCTION__));
 
   RefPtr<CamerasParent> self(this);
-  RefPtr<Runnable> webrtc_runnable =
-      media::NewRunnableFrom([self, aCapEngine]() -> nsresult {
-        bool result = self->EnsureInitialized(aCapEngine);
+  RefPtr<Runnable> webrtc_runnable = NewRunnableFrom([self, aCapEngine]() {
+    bool result = self->EnsureInitialized(aCapEngine);
 
-        RefPtr<nsIRunnable> ipc_runnable =
-            media::NewRunnableFrom([self, result]() -> nsresult {
-              if (!self->mChildIsAlive) {
-                return NS_ERROR_FAILURE;
-              }
+    RefPtr<nsIRunnable> ipc_runnable = NewRunnableFrom([self, result]() {
+      if (!self->mChildIsAlive) {
+        return NS_ERROR_FAILURE;
+      }
 
-              if (!result) {
-                LOG(("RecvEnsureInitialized failed"));
-                Unused << self->SendReplyFailure();
-                return NS_ERROR_FAILURE;
-              }
+      if (!result) {
+        LOG(("RecvEnsureInitialized failed"));
+        Unused << self->SendReplyFailure();
+        return NS_ERROR_FAILURE;
+      }
 
-              LOG(("RecvEnsureInitialized succeeded"));
-              Unused << self->SendReplySuccess();
-              return NS_OK;
-            });
-        self->mPBackgroundEventTarget->Dispatch(ipc_runnable,
-                                                NS_DISPATCH_NORMAL);
-        return NS_OK;
-      });
+      LOG(("RecvEnsureInitialized succeeded"));
+      Unused << self->SendReplySuccess();
+      return NS_OK;
+    });
+    self->mPBackgroundEventTarget->Dispatch(ipc_runnable, NS_DISPATCH_NORMAL);
+    return NS_OK;
+  });
   DispatchToVideoCaptureThread(webrtc_runnable);
   return IPC_OK();
 }
@@ -523,34 +500,32 @@ mozilla::ipc::IPCResult CamerasParent::RecvNumberOfCapabilities(
   LOG(("Getting caps for %s", unique_id.get()));
 
   RefPtr<CamerasParent> self(this);
-  RefPtr<Runnable> webrtc_runnable =
-      media::NewRunnableFrom([self, unique_id, aCapEngine]() -> nsresult {
-        int num = -1;
-        if (auto engine = self->EnsureInitialized(aCapEngine)) {
-          if (auto devInfo = engine->GetOrCreateVideoCaptureDeviceInfo()) {
-            num = devInfo->NumberOfCapabilities(unique_id.get());
-          }
-        }
-        RefPtr<nsIRunnable> ipc_runnable =
-            media::NewRunnableFrom([self, num]() -> nsresult {
-              if (!self->mChildIsAlive) {
-                return NS_ERROR_FAILURE;
-              }
+  RefPtr<Runnable> webrtc_runnable = NewRunnableFrom([self, unique_id,
+                                                      aCapEngine]() {
+    int num = -1;
+    if (auto engine = self->EnsureInitialized(aCapEngine)) {
+      if (auto devInfo = engine->GetOrCreateVideoCaptureDeviceInfo()) {
+        num = devInfo->NumberOfCapabilities(unique_id.get());
+      }
+    }
+    RefPtr<nsIRunnable> ipc_runnable = NewRunnableFrom([self, num]() {
+      if (!self->mChildIsAlive) {
+        return NS_ERROR_FAILURE;
+      }
 
-              if (num < 0) {
-                LOG(("RecvNumberOfCapabilities couldn't find capabilities"));
-                Unused << self->SendReplyFailure();
-                return NS_ERROR_FAILURE;
-              }
+      if (num < 0) {
+        LOG(("RecvNumberOfCapabilities couldn't find capabilities"));
+        Unused << self->SendReplyFailure();
+        return NS_ERROR_FAILURE;
+      }
 
-              LOG(("RecvNumberOfCapabilities: %d", num));
-              Unused << self->SendReplyNumberOfCapabilities(num);
-              return NS_OK;
-            });
-        self->mPBackgroundEventTarget->Dispatch(ipc_runnable,
-                                                NS_DISPATCH_NORMAL);
-        return NS_OK;
-      });
+      LOG(("RecvNumberOfCapabilities: %d", num));
+      Unused << self->SendReplyNumberOfCapabilities(num);
+      return NS_OK;
+    });
+    self->mPBackgroundEventTarget->Dispatch(ipc_runnable, NS_DISPATCH_NORMAL);
+    return NS_OK;
+  });
   DispatchToVideoCaptureThread(webrtc_runnable);
   return IPC_OK();
 }
@@ -562,52 +537,49 @@ mozilla::ipc::IPCResult CamerasParent::RecvGetCaptureCapability(
   LOG(("RecvGetCaptureCapability: %s %d", unique_id.get(), num));
 
   RefPtr<CamerasParent> self(this);
-  RefPtr<Runnable> webrtc_runnable =
-      media::NewRunnableFrom([self, unique_id, aCapEngine, num]() -> nsresult {
-        webrtc::VideoCaptureCapability webrtcCaps;
-        int error = -1;
-        if (auto engine = self->EnsureInitialized(aCapEngine)) {
-          if (auto devInfo = engine->GetOrCreateVideoCaptureDeviceInfo()) {
-            error = devInfo->GetCapability(unique_id.get(), num, webrtcCaps);
-          }
+  RefPtr<Runnable> webrtc_runnable = NewRunnableFrom([self, unique_id,
+                                                      aCapEngine, num]() {
+    webrtc::VideoCaptureCapability webrtcCaps;
+    int error = -1;
+    if (auto engine = self->EnsureInitialized(aCapEngine)) {
+      if (auto devInfo = engine->GetOrCreateVideoCaptureDeviceInfo()) {
+        error = devInfo->GetCapability(unique_id.get(), num, webrtcCaps);
+      }
 
-          if (!error && aCapEngine == CameraEngine) {
-            auto iter = self->mAllCandidateCapabilities.find(unique_id);
-            if (iter == self->mAllCandidateCapabilities.end()) {
-              std::map<uint32_t, webrtc::VideoCaptureCapability>
-                  candidateCapabilities;
-              candidateCapabilities.emplace(num, webrtcCaps);
-              self->mAllCandidateCapabilities.emplace(nsCString(unique_id),
-                                                      candidateCapabilities);
-            } else {
-              (iter->second).emplace(num, webrtcCaps);
-            }
-          }
+      if (!error && aCapEngine == CameraEngine) {
+        auto iter = self->mAllCandidateCapabilities.find(unique_id);
+        if (iter == self->mAllCandidateCapabilities.end()) {
+          std::map<uint32_t, webrtc::VideoCaptureCapability>
+              candidateCapabilities;
+          candidateCapabilities.emplace(num, webrtcCaps);
+          self->mAllCandidateCapabilities.emplace(nsCString(unique_id),
+                                                  candidateCapabilities);
+        } else {
+          (iter->second).emplace(num, webrtcCaps);
         }
-        RefPtr<nsIRunnable> ipc_runnable =
-            media::NewRunnableFrom([self, webrtcCaps, error]() -> nsresult {
-              if (!self->mChildIsAlive) {
-                return NS_ERROR_FAILURE;
-              }
-              VideoCaptureCapability capCap(
-                  webrtcCaps.width, webrtcCaps.height, webrtcCaps.maxFPS,
-                  webrtcCaps.expectedCaptureDelay, webrtcCaps.rawType,
-                  webrtcCaps.codecType, webrtcCaps.interlaced);
-              LOG(("Capability: %u %u %u %u %d %d", webrtcCaps.width,
-                   webrtcCaps.height, webrtcCaps.maxFPS,
-                   webrtcCaps.expectedCaptureDelay, webrtcCaps.rawType,
-                   webrtcCaps.codecType));
-              if (error) {
-                Unused << self->SendReplyFailure();
-                return NS_ERROR_FAILURE;
-              }
-              Unused << self->SendReplyGetCaptureCapability(capCap);
-              return NS_OK;
-            });
-        self->mPBackgroundEventTarget->Dispatch(ipc_runnable,
-                                                NS_DISPATCH_NORMAL);
-        return NS_OK;
-      });
+      }
+    }
+    RefPtr<nsIRunnable> ipc_runnable =
+        NewRunnableFrom([self, webrtcCaps, error]() {
+          if (!self->mChildIsAlive) {
+            return NS_ERROR_FAILURE;
+          }
+          VideoCaptureCapability capCap(
+              webrtcCaps.width, webrtcCaps.height, webrtcCaps.maxFPS,
+              static_cast<int>(webrtcCaps.videoType), webrtcCaps.interlaced);
+          LOG(("Capability: %u %u %u %d %d", webrtcCaps.width,
+               webrtcCaps.height, webrtcCaps.maxFPS,
+               static_cast<int>(webrtcCaps.videoType), webrtcCaps.interlaced));
+          if (error) {
+            Unused << self->SendReplyFailure();
+            return NS_ERROR_FAILURE;
+          }
+          Unused << self->SendReplyGetCaptureCapability(capCap);
+          return NS_OK;
+        });
+    self->mPBackgroundEventTarget->Dispatch(ipc_runnable, NS_DISPATCH_NORMAL);
+    return NS_OK;
+  });
   DispatchToVideoCaptureThread(webrtc_runnable);
   return IPC_OK();
 }
@@ -617,46 +589,45 @@ mozilla::ipc::IPCResult CamerasParent::RecvGetCaptureDevice(
   LOG((__PRETTY_FUNCTION__));
 
   RefPtr<CamerasParent> self(this);
-  RefPtr<Runnable> webrtc_runnable =
-      media::NewRunnableFrom([self, aCapEngine, aListNumber]() -> nsresult {
-        char deviceName[MediaEngineSource::kMaxDeviceNameLength];
-        char deviceUniqueId[MediaEngineSource::kMaxUniqueIdLength];
-        nsCString name;
-        nsCString uniqueId;
-        pid_t devicePid = 0;
-        int error = -1;
-        if (auto engine = self->EnsureInitialized(aCapEngine)) {
-          if (auto devInfo = engine->GetOrCreateVideoCaptureDeviceInfo()) {
-            error = devInfo->GetDeviceName(
-                aListNumber, deviceName, sizeof(deviceName), deviceUniqueId,
-                sizeof(deviceUniqueId), nullptr, 0, &devicePid);
+  RefPtr<Runnable> webrtc_runnable = NewRunnableFrom([self, aCapEngine,
+                                                      aListNumber]() {
+    char deviceName[MediaEngineSource::kMaxDeviceNameLength];
+    char deviceUniqueId[MediaEngineSource::kMaxUniqueIdLength];
+    nsCString name;
+    nsCString uniqueId;
+    pid_t devicePid = 0;
+    int error = -1;
+    if (auto engine = self->EnsureInitialized(aCapEngine)) {
+      if (auto devInfo = engine->GetOrCreateVideoCaptureDeviceInfo()) {
+        error = devInfo->GetDeviceName(
+            aListNumber, deviceName, sizeof(deviceName), deviceUniqueId,
+            sizeof(deviceUniqueId), nullptr, 0, &devicePid);
+      }
+    }
+    if (!error) {
+      name.Assign(deviceName);
+      uniqueId.Assign(deviceUniqueId);
+    }
+    RefPtr<nsIRunnable> ipc_runnable =
+        NewRunnableFrom([self, error, name, uniqueId, devicePid]() {
+          if (!self->mChildIsAlive) {
+            return NS_ERROR_FAILURE;
           }
-        }
-        if (!error) {
-          name.Assign(deviceName);
-          uniqueId.Assign(deviceUniqueId);
-        }
-        RefPtr<nsIRunnable> ipc_runnable =
-            media::NewRunnableFrom([self, error, name, uniqueId, devicePid]() {
-              if (!self->mChildIsAlive) {
-                return NS_ERROR_FAILURE;
-              }
-              if (error) {
-                LOG(("GetCaptureDevice failed: %d", error));
-                Unused << self->SendReplyFailure();
-                return NS_ERROR_FAILURE;
-              }
-              bool scary = (devicePid == getpid());
+          if (error) {
+            LOG(("GetCaptureDevice failed: %d", error));
+            Unused << self->SendReplyFailure();
+            return NS_ERROR_FAILURE;
+          }
+          bool scary = (devicePid == getpid());
 
-              LOG(("Returning %s name %s id (pid = %d)%s", name.get(),
-                   uniqueId.get(), devicePid, (scary ? " (scary)" : "")));
-              Unused << self->SendReplyGetCaptureDevice(name, uniqueId, scary);
-              return NS_OK;
-            });
-        self->mPBackgroundEventTarget->Dispatch(ipc_runnable,
-                                                NS_DISPATCH_NORMAL);
-        return NS_OK;
-      });
+          LOG(("Returning %s name %s id (pid = %d)%s", name.get(),
+               uniqueId.get(), devicePid, (scary ? " (scary)" : "")));
+          Unused << self->SendReplyGetCaptureDevice(name, uniqueId, scary);
+          return NS_OK;
+        });
+    self->mPBackgroundEventTarget->Dispatch(ipc_runnable, NS_DISPATCH_NORMAL);
+    return NS_OK;
+  });
   DispatchToVideoCaptureThread(webrtc_runnable);
   return IPC_OK();
 }
@@ -684,7 +655,8 @@ static bool HasCameraPermission(const ipc::PrincipalInfo& aPrincipalInfo) {
   }
 
   // Name used with nsIPermissionManager
-  static const char* cameraPermission = "MediaManagerVideo";
+  static const nsLiteralCString cameraPermission =
+      NS_LITERAL_CSTRING("MediaManagerVideo");
   nsCOMPtr<nsIPermissionManager> mgr =
       do_GetService(NS_PERMISSIONMANAGER_CONTRACTID, &rv);
   if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -713,63 +685,62 @@ mozilla::ipc::IPCResult CamerasParent::RecvAllocateCaptureDevice(
     const PrincipalInfo& aPrincipalInfo) {
   LOG(("%s: Verifying permissions", __PRETTY_FUNCTION__));
   RefPtr<CamerasParent> self(this);
-  RefPtr<Runnable> mainthread_runnable = media::NewRunnableFrom(
-      [self, aCapEngine, unique_id, aPrincipalInfo]() -> nsresult {
-        // Verify whether the claimed origin has received permission
-        // to use the camera, either persistently or this session (one shot).
-        bool allowed = HasCameraPermission(aPrincipalInfo);
-        if (!allowed) {
-          // Developer preference for turning off permission check.
-          if (Preferences::GetBool("media.navigator.permission.disabled",
-                                   false) ||
-              Preferences::GetBool("media.navigator.permission.fake")) {
-            allowed = true;
-            LOG(
-                ("No permission but checks are disabled or fake sources "
-                 "active"));
-          } else {
-            LOG(("No camera permission for this origin"));
-          }
-        }
-        // After retrieving the permission (or not) on the main thread,
-        // bounce to the WebRTC thread to allocate the device (or not),
-        // then bounce back to the IPC thread for the reply to content.
-        RefPtr<Runnable> webrtc_runnable = media::NewRunnableFrom(
-            [self, allowed, aCapEngine, unique_id]() -> nsresult {
-              int numdev = -1;
-              int error = -1;
-              if (allowed && self->EnsureInitialized(aCapEngine)) {
-                StaticRefPtr<VideoEngine>& engine = self->sEngines[aCapEngine];
-                engine->CreateVideoCapture(numdev, unique_id.get());
-                engine->WithEntry(numdev,
-                                  [&error](VideoEngine::CaptureEntry& cap) {
-                                    if (cap.VideoCapture()) {
-                                      error = 0;
-                                    }
-                                  });
+  RefPtr<Runnable> mainthread_runnable = NewRunnableFrom([self, aCapEngine,
+                                                          unique_id,
+                                                          aPrincipalInfo]() {
+    // Verify whether the claimed origin has received permission
+    // to use the camera, either persistently or this session (one shot).
+    bool allowed = HasCameraPermission(aPrincipalInfo);
+    if (!allowed) {
+      // Developer preference for turning off permission check.
+      if (Preferences::GetBool("media.navigator.permission.disabled", false) ||
+          Preferences::GetBool("media.navigator.permission.fake")) {
+        allowed = true;
+        LOG(
+            ("No permission but checks are disabled or fake sources "
+             "active"));
+      } else {
+        LOG(("No camera permission for this origin"));
+      }
+    }
+    // After retrieving the permission (or not) on the main thread,
+    // bounce to the WebRTC thread to allocate the device (or not),
+    // then bounce back to the IPC thread for the reply to content.
+    RefPtr<Runnable> webrtc_runnable =
+        NewRunnableFrom([self, allowed, aCapEngine, unique_id]() {
+          int numdev = -1;
+          int error = -1;
+          if (allowed && self->EnsureInitialized(aCapEngine)) {
+            StaticRefPtr<VideoEngine>& engine = self->sEngines[aCapEngine];
+            engine->CreateVideoCapture(numdev, unique_id.get());
+            engine->WithEntry(numdev, [&error](VideoEngine::CaptureEntry& cap) {
+              if (cap.VideoCapture()) {
+                error = 0;
               }
-              RefPtr<nsIRunnable> ipc_runnable =
-                  media::NewRunnableFrom([self, numdev, error]() -> nsresult {
-                    if (!self->mChildIsAlive) {
-                      return NS_ERROR_FAILURE;
-                    }
-
-                    if (error) {
-                      Unused << self->SendReplyFailure();
-                      return NS_ERROR_FAILURE;
-                    }
-
-                    LOG(("Allocated device nr %d", numdev));
-                    Unused << self->SendReplyAllocateCaptureDevice(numdev);
-                    return NS_OK;
-                  });
-              self->mPBackgroundEventTarget->Dispatch(ipc_runnable,
-                                                      NS_DISPATCH_NORMAL);
-              return NS_OK;
             });
-        self->DispatchToVideoCaptureThread(webrtc_runnable);
-        return NS_OK;
-      });
+          }
+          RefPtr<nsIRunnable> ipc_runnable =
+              NewRunnableFrom([self, numdev, error]() {
+                if (!self->mChildIsAlive) {
+                  return NS_ERROR_FAILURE;
+                }
+
+                if (error) {
+                  Unused << self->SendReplyFailure();
+                  return NS_ERROR_FAILURE;
+                }
+
+                LOG(("Allocated device nr %d", numdev));
+                Unused << self->SendReplyAllocateCaptureDevice(numdev);
+                return NS_OK;
+              });
+          self->mPBackgroundEventTarget->Dispatch(ipc_runnable,
+                                                  NS_DISPATCH_NORMAL);
+          return NS_OK;
+        });
+    self->DispatchToVideoCaptureThread(webrtc_runnable);
+    return NS_OK;
+  });
   NS_DispatchToMainThread(mainthread_runnable);
   return IPC_OK();
 }
@@ -789,29 +760,27 @@ mozilla::ipc::IPCResult CamerasParent::RecvReleaseCaptureDevice(
   LOG(("RecvReleaseCamera device nr %d", numdev));
 
   RefPtr<CamerasParent> self(this);
-  RefPtr<Runnable> webrtc_runnable =
-      media::NewRunnableFrom([self, aCapEngine, numdev]() -> nsresult {
-        int error = self->ReleaseCaptureDevice(aCapEngine, numdev);
-        RefPtr<nsIRunnable> ipc_runnable =
-            media::NewRunnableFrom([self, error, numdev]() -> nsresult {
-              if (!self->mChildIsAlive) {
-                return NS_ERROR_FAILURE;
-              }
+  RefPtr<Runnable> webrtc_runnable = NewRunnableFrom([self, aCapEngine,
+                                                      numdev]() {
+    int error = self->ReleaseCaptureDevice(aCapEngine, numdev);
+    RefPtr<nsIRunnable> ipc_runnable = NewRunnableFrom([self, error, numdev]() {
+      if (!self->mChildIsAlive) {
+        return NS_ERROR_FAILURE;
+      }
 
-              if (error) {
-                Unused << self->SendReplyFailure();
-                LOG(("Failed to free device nr %d", numdev));
-                return NS_ERROR_FAILURE;
-              }
+      if (error) {
+        Unused << self->SendReplyFailure();
+        LOG(("Failed to free device nr %d", numdev));
+        return NS_ERROR_FAILURE;
+      }
 
-              Unused << self->SendReplySuccess();
-              LOG(("Freed device nr %d", numdev));
-              return NS_OK;
-            });
-        self->mPBackgroundEventTarget->Dispatch(ipc_runnable,
-                                                NS_DISPATCH_NORMAL);
-        return NS_OK;
-      });
+      Unused << self->SendReplySuccess();
+      LOG(("Freed device nr %d", numdev));
+      return NS_OK;
+    });
+    self->mPBackgroundEventTarget->Dispatch(ipc_runnable, NS_DISPATCH_NORMAL);
+    return NS_OK;
+  });
   DispatchToVideoCaptureThread(webrtc_runnable);
   return IPC_OK();
 }
@@ -822,9 +791,8 @@ mozilla::ipc::IPCResult CamerasParent::RecvStartCapture(
   LOG((__PRETTY_FUNCTION__));
 
   RefPtr<CamerasParent> self(this);
-  RefPtr<Runnable> webrtc_runnable = media::NewRunnableFrom([self, aCapEngine,
-                                                             capnum, ipcCaps]()
-                                                                -> nsresult {
+  RefPtr<Runnable> webrtc_runnable = NewRunnableFrom([self, aCapEngine, capnum,
+                                                      ipcCaps]() {
     LOG((__PRETTY_FUNCTION__));
     CallbackHelper** cbh;
     int error = -1;
@@ -839,11 +807,8 @@ mozilla::ipc::IPCResult CamerasParent::RecvStartCapture(
             capability.width = ipcCaps.width();
             capability.height = ipcCaps.height();
             capability.maxFPS = ipcCaps.maxFPS();
-            capability.expectedCaptureDelay = ipcCaps.expectedCaptureDelay();
-            capability.rawType =
-                static_cast<webrtc::RawVideoType>(ipcCaps.rawType());
-            capability.codecType =
-                static_cast<webrtc::VideoCodecType>(ipcCaps.codecType());
+            capability.videoType =
+                static_cast<webrtc::VideoType>(ipcCaps.videoType());
             capability.interlaced = ipcCaps.interlaced();
 
             MOZ_DIAGNOSTIC_ASSERT(sDeviceUniqueIDs.find(capnum) ==
@@ -881,8 +846,8 @@ mozilla::ipc::IPCResult CamerasParent::RecvStartCapture(
 
                 for (auto& candidateCapability :
                      candidateCapabilities->second) {
-                  if (candidateCapability.second.rawType !=
-                      capability.rawType) {
+                  if (candidateCapability.second.videoType !=
+                      capability.videoType) {
                     continue;
                   }
                   // The first priority is finding a suitable resolution.
@@ -905,8 +870,7 @@ mozilla::ipc::IPCResult CamerasParent::RecvStartCapture(
                 capability = candidateCapabilities->second[minIdx];
               }
             } else if (aCapEngine == ScreenEngine ||
-                       aCapEngine == BrowserEngine || aCapEngine == WinEngine ||
-                       aCapEngine == AppEngine) {
+                       aCapEngine == BrowserEngine || aCapEngine == WinEngine) {
               for (const auto& it : sDeviceUniqueIDs) {
                 if (strcmp(it.second,
                            cap.VideoCapture()->CurrentDeviceName()) == 0) {
@@ -929,20 +893,19 @@ mozilla::ipc::IPCResult CamerasParent::RecvStartCapture(
             }
           });
     }
-    RefPtr<nsIRunnable> ipc_runnable =
-        media::NewRunnableFrom([self, error]() -> nsresult {
-          if (!self->mChildIsAlive) {
-            return NS_ERROR_FAILURE;
-          }
+    RefPtr<nsIRunnable> ipc_runnable = NewRunnableFrom([self, error]() {
+      if (!self->mChildIsAlive) {
+        return NS_ERROR_FAILURE;
+      }
 
-          if (!error) {
-            Unused << self->SendReplySuccess();
-            return NS_OK;
-          }
+      if (!error) {
+        Unused << self->SendReplySuccess();
+        return NS_OK;
+      }
 
-          Unused << self->SendReplyFailure();
-          return NS_ERROR_FAILURE;
-        });
+      Unused << self->SendReplyFailure();
+      return NS_ERROR_FAILURE;
+    });
     self->mPBackgroundEventTarget->Dispatch(ipc_runnable, NS_DISPATCH_NORMAL);
     return NS_OK;
   });
@@ -950,6 +913,40 @@ mozilla::ipc::IPCResult CamerasParent::RecvStartCapture(
   return IPC_OK();
 }
 
+mozilla::ipc::IPCResult CamerasParent::RecvFocusOnSelectedSource(
+    const CaptureEngine& aCapEngine, const int& aCapNum) {
+  LOG((__PRETTY_FUNCTION__));
+  RefPtr<Runnable> webrtc_runnable = NewRunnableFrom(
+      [self = RefPtr<CamerasParent>(this), aCapEngine, aCapNum]() {
+        if (auto engine = self->EnsureInitialized(aCapEngine)) {
+          engine->WithEntry(aCapNum, [self](VideoEngine::CaptureEntry& cap) {
+            if (cap.VideoCapture()) {
+              bool result = cap.VideoCapture()->FocusOnSelectedSource();
+              RefPtr<nsIRunnable> ipc_runnable =
+                  NewRunnableFrom([self, result]() {
+                    if (!self->mChildIsAlive) {
+                      return NS_ERROR_FAILURE;
+                    }
+
+                    if (result) {
+                      Unused << self->SendReplySuccess();
+                      return NS_OK;
+                    }
+
+                    Unused << self->SendReplyFailure();
+                    return NS_ERROR_FAILURE;
+                  });
+              self->mPBackgroundEventTarget->Dispatch(ipc_runnable,
+                                                      NS_DISPATCH_NORMAL);
+            }
+          });
+        }
+        return NS_ERROR_FAILURE;
+      });
+  DispatchToVideoCaptureThread(webrtc_runnable);
+  return IPC_OK();
+}
+
 void CamerasParent::StopCapture(const CaptureEngine& aCapEngine,
                                 const int& capnum) {
   if (auto engine = EnsureInitialized(aCapEngine)) {
@@ -984,7 +981,7 @@ mozilla::ipc::IPCResult CamerasParent::RecvStopCapture(
 
   RefPtr<CamerasParent> self(this);
   RefPtr<Runnable> webrtc_runnable =
-      media::NewRunnableFrom([self, aCapEngine, capnum]() -> nsresult {
+      NewRunnableFrom([self, aCapEngine, capnum]() {
         self->StopCapture(aCapEngine, capnum);
         return NS_OK;
       });
@@ -1037,8 +1034,21 @@ void CamerasParent::ActorDestroy(ActorDestroyReason aWhy) {
   StopVideoCapture();
 }
 
+nsString CamerasParent::GetNewName() {
+  static volatile uint64_t counter = 0;
+  nsString name(NS_LITERAL_STRING("CamerasParent "));
+  name.AppendInt(++counter);
+  return name;
+}
+
+NS_IMETHODIMP CamerasParent::BlockShutdown(nsIAsyncShutdownClient*) {
+  StopVideoCapture();
+  return NS_OK;
+}
+
 CamerasParent::CamerasParent()
-    : mShmemPool(CaptureEngine::MaxEngine),
+    : mName(GetNewName()),
+      mShmemPool(CaptureEngine::MaxEngine),
       mChildIsAlive(true),
       mDestroyed(false),
       mWebRTCAlive(true) {
@@ -1056,17 +1066,11 @@ CamerasParent::CamerasParent()
   LOG(("Spinning up WebRTC Cameras Thread"));
 
   RefPtr<CamerasParent> self(this);
-  RefPtr<Runnable> threadStart = media::NewRunnableFrom([self]() -> nsresult {
-    // Register thread shutdown observer
-    nsCOMPtr<nsIObserverService> obs = services::GetObserverService();
-    if (NS_WARN_IF(!obs)) {
-      return NS_ERROR_FAILURE;
-    }
-    nsresult rv =
-        obs->AddObserver(self, NS_XPCOM_WILL_SHUTDOWN_OBSERVER_ID, false);
-    if (NS_WARN_IF(NS_FAILED(rv))) {
-      return rv;
-    }
+  NS_DispatchToMainThread(NewRunnableFrom([self]() {
+    nsresult rv = GetShutdownBarrier()->AddBlocker(
+        self, NS_LITERAL_STRING(__FILE__), __LINE__, NS_LITERAL_STRING(""));
+    MOZ_RELEASE_ASSERT(NS_SUCCEEDED(rv));
+
     // Start the thread
     MonitorAutoLock lock(*(self->sThreadMonitor));
     if (self->sVideoCaptureThread == nullptr) {
@@ -1085,8 +1089,7 @@ CamerasParent::CamerasParent()
     sNumOfOpenCamerasParentEngines++;
     self->sThreadMonitor->NotifyAll();
     return NS_OK;
-  });
-  NS_DispatchToMainThread(threadStart);
+  }));
 }
 
 CamerasParent::~CamerasParent() {
