@@ -19,6 +19,7 @@
 #include "DecoderTraits.h"
 #include "AudioContext.h"
 #include "AudioBuffer.h"
+#include "js/MemoryFunctions.h"
 #include "MediaContainerType.h"
 #include "nsContentUtils.h"
 #include "nsIScriptObjectPrincipal.h"
@@ -84,6 +85,9 @@ class MediaDecodeTask final : public Runnable {
     MOZ_ASSERT(NS_IsMainThread());
   }
 
+  // MOZ_CAN_RUN_SCRIPT_BOUNDARY until Runnable::Run is MOZ_CAN_RUN_SCRIPT.  See
+  // bug 1535398.
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY
   NS_IMETHOD Run() override;
   bool CreateReader();
   MediaFormatReader* Reader() {
@@ -92,6 +96,7 @@ class MediaDecodeTask final : public Runnable {
   }
 
  private:
+  MOZ_CAN_RUN_SCRIPT
   void ReportFailureOnMainThread(WebAudioDecodeJob::ErrorCode aErrorCode) {
     if (NS_IsMainThread()) {
       Cleanup();
@@ -108,14 +113,14 @@ class MediaDecodeTask final : public Runnable {
   }
 
   void Decode();
-  void OnMetadataRead(MetadataHolder&& aMetadata);
-  void OnMetadataNotRead(const MediaResult& aError);
+  MOZ_CAN_RUN_SCRIPT void OnMetadataRead(MetadataHolder&& aMetadata);
+  MOZ_CAN_RUN_SCRIPT void OnMetadataNotRead(const MediaResult& aError);
   void RequestSample();
   void SampleDecoded(RefPtr<AudioData> aData);
-  void SampleNotDecoded(const MediaResult& aError);
-  void FinishDecode();
-  void AllocateBuffer();
-  void CallbackTheResult();
+  MOZ_CAN_RUN_SCRIPT void SampleNotDecoded(const MediaResult& aError);
+  MOZ_CAN_RUN_SCRIPT void FinishDecode();
+  MOZ_CAN_RUN_SCRIPT void AllocateBuffer();
+  MOZ_CAN_RUN_SCRIPT void CallbackTheResult();
 
   void Cleanup() {
     MOZ_ASSERT(NS_IsMainThread());
@@ -270,7 +275,7 @@ void MediaDecodeTask::SampleNotDecoded(const MediaResult& aError) {
 void MediaDecodeTask::FinishDecode() {
   mDecoderReader->Shutdown();
 
-  uint32_t frameCount = mAudioQueue.FrameCount();
+  uint32_t frameCount = mAudioQueue.AudioFramesCount();
   uint32_t channelCount = mMediaInfo.mAudio.mChannels;
   uint32_t sampleRate = mMediaInfo.mAudio.mRate;
 
@@ -300,7 +305,7 @@ void MediaDecodeTask::FinishDecode() {
   mDecodeJob.mBuffer.mChannelData.SetLength(channelCount);
 #if AUDIO_OUTPUT_FORMAT == AUDIO_FORMAT_FLOAT32
   // This buffer has separate channel arrays that could be transferred to
-  // JS_NewArrayBufferWithContents(), but AudioBuffer::RestoreJSChannelData()
+  // JS::NewArrayBufferWithContents(), but AudioBuffer::RestoreJSChannelData()
   // does not yet take advantage of this.
   RefPtr<ThreadSharedFloatArrayBufferList> buffer =
       ThreadSharedFloatArrayBufferList::Create(channelCount, resampledFrames,
@@ -340,20 +345,20 @@ void MediaDecodeTask::FinishDecode() {
       const uint32_t maxOutSamples = resampledFrames - writeIndex;
 
       for (uint32_t i = 0; i < audioData->mChannels; ++i) {
-        uint32_t inSamples = audioData->mFrames;
+        uint32_t inSamples = audioData->Frames();
         uint32_t outSamples = maxOutSamples;
         AudioDataValue* outData =
             mDecodeJob.mBuffer.ChannelDataForWrite<AudioDataValue>(i) +
             writeIndex;
 
         WebAudioUtils::SpeexResamplerProcess(
-            resampler, i, &bufferData[i * audioData->mFrames], &inSamples,
+            resampler, i, &bufferData[i * audioData->Frames()], &inSamples,
             outData, &outSamples);
 
         if (i == audioData->mChannels - 1) {
           writeIndex += outSamples;
           MOZ_ASSERT(writeIndex <= resampledFrames);
-          MOZ_ASSERT(inSamples == audioData->mFrames);
+          MOZ_ASSERT(inSamples == audioData->Frames());
         }
       }
     } else {
@@ -361,11 +366,11 @@ void MediaDecodeTask::FinishDecode() {
         AudioDataValue* outData =
             mDecodeJob.mBuffer.ChannelDataForWrite<AudioDataValue>(i) +
             writeIndex;
-        PodCopy(outData, &bufferData[i * audioData->mFrames],
-                audioData->mFrames);
+        PodCopy(outData, &bufferData[i * audioData->Frames()],
+                audioData->Frames());
 
         if (i == audioData->mChannels - 1) {
-          writeIndex += audioData->mFrames;
+          writeIndex += audioData->Frames();
         }
       }
     }
@@ -425,7 +430,7 @@ bool WebAudioDecodeJob::AllocateBuffer() {
 
   // Now create the AudioBuffer
   mOutput = AudioBuffer::Create(mContext->GetOwner(), mContext->SampleRate(),
-                                Move(mBuffer));
+                                std::move(mBuffer));
   return mOutput != nullptr;
 }
 
@@ -485,14 +490,14 @@ void WebAudioDecodeJob::OnSuccess(ErrorCode aErrorCode) {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(aErrorCode == NoError);
 
+  RefPtr<AudioBuffer> output(mOutput);
   if (mSuccessCallback) {
-    ErrorResult rv;
-    mSuccessCallback->Call(*mOutput, rv);
+    RefPtr<DecodeSuccessCallback> callback(mSuccessCallback);
     // Ignore errors in calling the callback, since there is not much that we
     // can do about it here.
-    rv.SuppressException();
+    callback->Call(*output);
   }
-  mPromise->MaybeResolve(mOutput);
+  mPromise->MaybeResolve(output);
 
   mContext->RemoveFromDecodeQueue(this);
 }
@@ -502,13 +507,6 @@ void WebAudioDecodeJob::OnFailure(ErrorCode aErrorCode) {
 
   const char* errorMessage;
   switch (aErrorCode) {
-    case NoError:
-      MOZ_FALLTHROUGH_ASSERT("Who passed NoError to OnFailure?");
-      // Fall through to get some sort of a sane error message if this actually
-      // happens at runtime.
-    case UnknownError:
-      errorMessage = "MediaDecodeAudioDataUnknownError";
-      break;
     case UnknownContent:
       errorMessage = "MediaDecodeAudioDataUnknownContentType";
       break;
@@ -518,9 +516,18 @@ void WebAudioDecodeJob::OnFailure(ErrorCode aErrorCode) {
     case NoAudio:
       errorMessage = "MediaDecodeAudioDataNoAudio";
       break;
+    case NoError:
+      MOZ_FALLTHROUGH_ASSERT("Who passed NoError to OnFailure?");
+      // Fall through to get some sort of a sane error message if this actually
+      // happens at runtime.
+    case UnknownError:
+      MOZ_FALLTHROUGH;
+    default:
+      errorMessage = "MediaDecodeAudioDataUnknownError";
+      break;
   }
 
-  nsIDocument* doc = nullptr;
+  Document* doc = nullptr;
   if (nsPIDOMWindowInner* pWindow = mContext->GetParentObject()) {
     doc = pWindow->GetExtantDoc();
   }
@@ -534,7 +541,8 @@ void WebAudioDecodeJob::OnFailure(ErrorCode aErrorCode) {
     nsAutoCString errorString(errorMessage);
     RefPtr<DOMException> exception = DOMException::Create(
         NS_ERROR_DOM_ENCODING_NOT_SUPPORTED_ERR, errorString);
-    mFailureCallback->Call(*exception);
+    RefPtr<DecodeErrorCallback> callback(mFailureCallback);
+    callback->Call(*exception);
   }
 
   mPromise->MaybeReject(NS_ERROR_DOM_ENCODING_NOT_SUPPORTED_ERR);
