@@ -107,9 +107,32 @@ class SharedBuffers final {
   }
 
   // main thread
+
+  // NotifyNodeIsConnected() may be called even when the state has not
+  // changed.
+  void NotifyNodeIsConnected(bool aIsConnected) {
+    MOZ_ASSERT(NS_IsMainThread());
+    if (!aIsConnected) {
+      // Reset main thread state for FinishProducingOutputBuffer().
+      mLatency = 0.0f;
+      mLastEventTime = TimeStamp();
+      mDroppingBuffers = false;
+      // Don't flush the output buffer here because the graph thread may be
+      // using it now.  The graph thread will flush when it knows it is
+      // disconnected.
+    }
+    mNodeIsConnected = aIsConnected;
+  }
+
   void FinishProducingOutputBuffer(const AudioChunk& aBuffer) {
     MOZ_ASSERT(NS_IsMainThread());
 
+    if (!mNodeIsConnected) {
+      // The output buffer is not used, and mLastEventTime will not be
+      // initialized until the node is re-connected.
+      return;
+    }
+
     TimeStamp now = TimeStamp::Now();
 
     if (mLastEventTime.IsNull()) {
@@ -152,6 +175,7 @@ class SharedBuffers final {
   }
 
   // graph thread
+
   AudioChunk GetOutputBuffer() {
     MOZ_ASSERT(!NS_IsMainThread());
     AudioChunk buffer;
@@ -181,15 +205,13 @@ class SharedBuffers final {
     return mDelaySoFar == STREAM_TIME_MAX ? 0 : mDelaySoFar;
   }
 
-  void Reset() {
+  void Flush() {
     MOZ_ASSERT(!NS_IsMainThread());
     mDelaySoFar = STREAM_TIME_MAX;
-    mLatency = 0.0f;
     {
       MutexAutoLock lock(mOutputQueue.Lock());
       mOutputQueue.Clear();
     }
-    mLastEventTime = TimeStamp();
   }
 
  private:
@@ -197,9 +219,11 @@ class SharedBuffers final {
   // How much delay we've seen so far.  This measures the amount of delay
   // caused by the main thread lagging behind in producing output buffers.
   // STREAM_TIME_MAX means that we have not received our first buffer yet.
+  // Graph thread only.
   StreamTime mDelaySoFar;
   // The samplerate of the context.
-  float mSampleRate;
+  const float mSampleRate;
+  // The remaining members are main thread only.
   // This is the latency caused by the buffering. If this grows too high, we
   // will drop buffers until it is acceptable.
   float mLatency;
@@ -208,6 +232,8 @@ class SharedBuffers final {
   TimeStamp mLastEventTime;
   // True if we should be dropping buffers.
   bool mDroppingBuffers;
+  // True iff the AudioNode has at least one input or output connected.
+  bool mNodeIsConnected;
 };
 
 class ScriptProcessorNodeEngine final : public AudioNodeEngine {
@@ -247,7 +273,7 @@ class ScriptProcessorNodeEngine final : public AudioNodeEngine {
     // buffer queue, and output a null buffer.
     if (!mIsConnected) {
       aOutput->SetNull(WEBAUDIO_BLOCK_SIZE);
-      mSharedBuffers->Reset();
+      mSharedBuffers->Flush();
       mInputWriteIndex = 0;
       return;
     }
@@ -489,7 +515,7 @@ void ScriptProcessorNode::EventListenerRemoved(nsAtom* aType) {
 
 JSObject* ScriptProcessorNode::WrapObject(JSContext* aCx,
                                           JS::Handle<JSObject*> aGivenProto) {
-  return ScriptProcessorNodeBinding::Wrap(aCx, this, aGivenProto);
+  return ScriptProcessorNode_Binding::Wrap(aCx, this, aGivenProto);
 }
 
 void ScriptProcessorNode::UpdateConnectedStatus() {
@@ -507,6 +533,9 @@ void ScriptProcessorNode::UpdateConnectedStatus() {
   } else {
     MarkInactive();
   }
+
+  auto engine = static_cast<ScriptProcessorNodeEngine*>(mStream->Engine());
+  engine->GetSharedBuffers()->NotifyNodeIsConnected(isConnected);
 }
 
 }  // namespace dom
