@@ -23,6 +23,11 @@ class Blob;
 class AllocationHandle;
 class MediaEngineSource;
 
+enum MediaSinkEnum {
+  Speaker,
+  Other,
+};
+
 enum { kVideoTrack = 1, kAudioTrack = 2, kTrackCount };
 
 class MediaEngine : public DeviceChangeCallback {
@@ -37,15 +42,15 @@ class MediaEngine : public DeviceChangeCallback {
    * Also include devices that are currently unavailable.
    */
   virtual void EnumerateDevices(uint64_t aWindowId, dom::MediaSourceEnum,
-                                nsTArray<RefPtr<MediaEngineSource>>*) = 0;
+                                MediaSinkEnum,
+                                nsTArray<RefPtr<MediaDevice>>*) = 0;
 
-  virtual void ReleaseResourcesForWindow(uint64_t aWindowId) = 0;
   virtual void Shutdown() = 0;
 
   virtual void SetFakeDeviceChangeEvents() {}
 
  protected:
-  virtual ~MediaEngine() {}
+  virtual ~MediaEngine() = default;
 };
 
 }  // namespace mozilla
