@@ -20,7 +20,7 @@ typedef SVGPolyElement SVGPolylineElementBase;
 class SVGPolylineElement final : public SVGPolylineElementBase {
  protected:
   explicit SVGPolylineElement(
-      already_AddRefed<mozilla::dom::NodeInfo>& aNodeInfo);
+      already_AddRefed<mozilla::dom::NodeInfo>&& aNodeInfo);
   virtual JSObject* WrapNode(JSContext* cx,
                              JS::Handle<JSObject*> aGivenProto) override;
   friend nsresult(::NS_NewSVGPolylineElement(
@@ -32,8 +32,7 @@ class SVGPolylineElement final : public SVGPolylineElementBase {
 
  public:
   // nsIContent interface
-  virtual nsresult Clone(mozilla::dom::NodeInfo* aNodeInfo, nsINode** aResult,
-                         bool aPreallocateChildren) const override;
+  virtual nsresult Clone(dom::NodeInfo*, nsINode** aResult) const override;
 };
 
 }  // namespace dom
