@@ -11,39 +11,47 @@
 
 #include "prsystem.h"
 
+// Short macro to get the size of a member of a
+// given struct at compile time.
+// t is the type of struct, m the name of the
+// member:
+// DOM_SIZEOF_MEMBER(struct mystruct, myint)
+// will give you the size of the type of myint.
+#define DOM_SIZEOF_MEMBER(t, m) sizeof(((t*)0)->m)
+
 #if defined(XP_UNIX)
-#include "unistd.h"
-#include "dirent.h"
-#include "poll.h"
-#include "sys/stat.h"
-#if defined(XP_LINUX)
-#include <sys/vfs.h>
-#define statvfs statfs
-#define f_frsize f_bsize
-#else
-#include "sys/statvfs.h"
-#endif  // defined(XP_LINUX)
-#if !defined(ANDROID)
-#include "sys/wait.h"
-#include <spawn.h>
-#endif  // !defined(ANDROID)
-#endif  // defined(XP_UNIX)
+#  include "unistd.h"
+#  include "dirent.h"
+#  include "poll.h"
+#  include "sys/stat.h"
+#  if defined(XP_LINUX)
+#    include <sys/vfs.h>
+#    define statvfs statfs
+#    define f_frsize f_bsize
+#  else
+#    include "sys/statvfs.h"
+#  endif  // defined(XP_LINUX)
+#  if !defined(ANDROID)
+#    include "sys/wait.h"
+#    include <spawn.h>
+#  endif  // !defined(ANDROID)
+#endif    // defined(XP_UNIX)
 
 #if defined(XP_LINUX)
-#include <linux/fadvise.h>
+#  include <linux/fadvise.h>
 #endif  // defined(XP_LINUX)
 
 #if defined(XP_MACOSX)
-#include "copyfile.h"
+#  include "copyfile.h"
 #endif  // defined(XP_MACOSX)
 
 #if defined(XP_WIN)
-#include <windows.h>
-#include <accctrl.h>
+#  include <windows.h>
+#  include <accctrl.h>
 
-#ifndef PATH_MAX
-#define PATH_MAX MAX_PATH
-#endif
+#  ifndef PATH_MAX
+#    define PATH_MAX MAX_PATH
+#  endif
 
 #endif  // defined(XP_WIN)
 
@@ -78,7 +86,7 @@
 
 #if defined(__DragonFly__) || defined(__FreeBSD__) || defined(__NetBSD__) || \
     defined(__OpenBSD__)
-#define __dd_fd dd_fd
+#  define __dd_fd dd_fd
 #endif
 
 /**
@@ -103,44 +111,12 @@ struct OSFileConstantsService::Paths {
   nsString tmpDir;
   nsString profileDir;
   nsString localProfileDir;
-  /**
-   * The user's home directory
-   */
-  nsString homeDir;
-  /**
-   * The user's 'application data' directory.
-   * Windows:
-   *   HOME = Documents and Settings\$USER\Application Data
-   *   UAppData = $HOME[\$vendor]\$name
-   *
-   * Unix:
-   *   HOME = ~
-   *   UAppData = $HOME/.[$vendor/]$name
-   *
-   * Mac:
-   *   HOME = ~
-   *   UAppData = $HOME/Library/Application Support/$name
-   */
-  nsString userApplicationDataDir;
-
-#if defined(XP_MACOSX)
-  /**
-   * The user's Library directory.
-   */
-  nsString macUserLibDir;
-#endif  // defined(XP_MACOSX)
 
   Paths() {
     libDir.SetIsVoid(true);
     tmpDir.SetIsVoid(true);
     profileDir.SetIsVoid(true);
     localProfileDir.SetIsVoid(true);
-    homeDir.SetIsVoid(true);
-    userApplicationDataDir.SetIsVoid(true);
-
-#if defined(XP_MACOSX)
-    macUserLibDir.SetIsVoid(true);
-#endif  // defined(XP_MACOSX)
   }
 };
 
@@ -151,7 +127,7 @@ struct OSFileConstantsService::Paths {
  * @param aOutPath The path to the special directory. In case of error,
  * the string is set to void.
  */
-nsresult GetPathToSpecialDir(const char *aKey, nsString &aOutPath) {
+nsresult GetPathToSpecialDir(const char* aKey, nsString& aOutPath) {
   nsCOMPtr<nsIFile> file;
   nsresult rv = NS_GetSpecialDirectory(aKey, getter_AddRefs(file));
   if (NS_FAILED(rv) || !file) {
@@ -172,8 +148,8 @@ nsresult GetPathToSpecialDir(const char *aKey, nsString &aOutPath) {
  * and |mPaths->localProfileDir| once the profile is setup.
  */
 NS_IMETHODIMP
-OSFileConstantsService::Observe(nsISupports *, const char *aTopic,
-                                const char16_t *) {
+OSFileConstantsService::Observe(nsISupports*, const char* aTopic,
+                                const char16_t*) {
   if (!mInitialized) {
     // Initialization has not taken place, something is wrong,
     // don't make things worse.
@@ -248,23 +224,14 @@ nsresult OSFileConstantsService::InitOSFileConstants() {
     }
   }
 
-  // For other directories, ignore errors (they may be undefined on
-  // some platforms or in non-Firefox embeddings of Gecko).
-
   GetPathToSpecialDir(NS_OS_TEMP_DIR, paths->tmpDir);
-  GetPathToSpecialDir(NS_OS_HOME_DIR, paths->homeDir);
-  GetPathToSpecialDir(XRE_USER_APP_DATA_DIR, paths->userApplicationDataDir);
 
-#if defined(XP_MACOSX)
-  GetPathToSpecialDir(NS_MAC_USER_LIB_DIR, paths->macUserLibDir);
-#endif  // defined(XP_MACOSX)
-
-  mPaths = Move(paths);
+  mPaths = std::move(paths);
 
   // Get the umask from the system-info service.
   // The property will always be present, but it will be zero on
   // non-Unix systems.
-  // nsSystemInfo::gUserUmask is initialized by NS_InitXPCOM2 so we don't need
+  // nsSystemInfo::gUserUmask is initialized by NS_InitXPCOM so we don't need
   // to initialize the service.
   mUserUmask = nsSystemInfo::gUserUmask;
 
@@ -302,18 +269,18 @@ nsresult OSFileConstantsService::InitOSFileConstants() {
 
 // Define missing constants for Android
 #if !defined(S_IRGRP)
-#define S_IXOTH 0001
-#define S_IWOTH 0002
-#define S_IROTH 0004
-#define S_IRWXO 0007
-#define S_IXGRP 0010
-#define S_IWGRP 0020
-#define S_IRGRP 0040
-#define S_IRWXG 0070
-#define S_IXUSR 0100
-#define S_IWUSR 0200
-#define S_IRUSR 0400
-#define S_IRWXU 0700
+#  define S_IXOTH 0001
+#  define S_IWOTH 0002
+#  define S_IROTH 0004
+#  define S_IRWXO 0007
+#  define S_IXGRP 0010
+#  define S_IWGRP 0020
+#  define S_IRGRP 0040
+#  define S_IRWXG 0070
+#  define S_IXUSR 0100
+#  define S_IWUSR 0200
+#  define S_IRUSR 0400
+#  define S_IRWXU 0700
 #endif  // !defined(S_IRGRP)
 
 /**
@@ -421,9 +388,9 @@ static const dom::ConstantSpec gLibcProperties[] = {
     INT_CONSTANT(POLLOUT),
 
 // wait
-#if defined(WNOHANG)
+#  if defined(WNOHANG)
     INT_CONSTANT(WNOHANG),
-#endif  // defined(WNOHANG)
+#  endif  // defined(WNOHANG)
 
     // fcntl command values
     INT_CONSTANT(F_GETLK),
@@ -438,19 +405,19 @@ static const dom::ConstantSpec gLibcProperties[] = {
     INT_CONSTANT(F_UNLCK),
 
 // splice
-#if defined(SPLICE_F_MOVE)
+#  if defined(SPLICE_F_MOVE)
     INT_CONSTANT(SPLICE_F_MOVE),
-#endif  // defined(SPLICE_F_MOVE)
-#if defined(SPLICE_F_NONBLOCK)
+#  endif  // defined(SPLICE_F_MOVE)
+#  if defined(SPLICE_F_NONBLOCK)
     INT_CONSTANT(SPLICE_F_NONBLOCK),
-#endif  // defined(SPLICE_F_NONBLOCK)
-#if defined(SPLICE_F_MORE)
+#  endif  // defined(SPLICE_F_NONBLOCK)
+#  if defined(SPLICE_F_MORE)
     INT_CONSTANT(SPLICE_F_MORE),
-#endif  // defined(SPLICE_F_MORE)
-#if defined(SPLICE_F_GIFT)
+#  endif  // defined(SPLICE_F_MORE)
+#  if defined(SPLICE_F_GIFT)
     INT_CONSTANT(SPLICE_F_GIFT),
-#endif  // defined(SPLICE_F_GIFT)
-#endif  // defined(XP_UNIX)
+#  endif  // defined(SPLICE_F_GIFT)
+#endif    // defined(XP_UNIX)
 // copyfile
 #if defined(COPYFILE_DATA)
     INT_CONSTANT(COPYFILE_DATA),
@@ -549,18 +516,18 @@ static const dom::ConstantSpec gLibcProperties[] = {
     // The size of |fsblkcnt_t|.
     {"OSFILE_SIZEOF_FSBLKCNT_T", JS::Int32Value(sizeof(fsblkcnt_t))},
 
-#if !defined(ANDROID)
+#  if !defined(ANDROID)
     // The size of |posix_spawn_file_actions_t|.
     {"OSFILE_SIZEOF_POSIX_SPAWN_FILE_ACTIONS_T",
      JS::Int32Value(sizeof(posix_spawn_file_actions_t))},
-#endif  // !defined(ANDROID)
+#  endif  // !defined(ANDROID)
 
     // Defining |dirent|.
     // Size
     {"OSFILE_SIZEOF_DIRENT", JS::Int32Value(sizeof(dirent))},
 
 // Defining |flock|.
-#if defined(XP_UNIX)
+#  if defined(XP_UNIX)
     {"OSFILE_SIZEOF_FLOCK", JS::Int32Value(sizeof(struct flock))},
     {"OSFILE_OFFSETOF_FLOCK_L_START",
      JS::Int32Value(offsetof(struct flock, l_start))},
@@ -572,7 +539,7 @@ static const dom::ConstantSpec gLibcProperties[] = {
      JS::Int32Value(offsetof(struct flock, l_type))},
     {"OSFILE_OFFSETOF_FLOCK_L_WHENCE",
      JS::Int32Value(offsetof(struct flock, l_whence))},
-#endif  // defined(XP_UNIX)
+#  endif  // defined(XP_UNIX)
     // Offset of field |d_name|.
     {"OSFILE_OFFSETOF_DIRENT_D_NAME",
      JS::Int32Value(offsetof(struct dirent, d_name))},
@@ -588,21 +555,21 @@ static const dom::ConstantSpec gLibcProperties[] = {
     {"OSFILE_OFFSETOF_TIMEVAL_TV_USEC",
      JS::Int32Value(offsetof(struct timeval, tv_usec))},
 
-#if defined(DT_UNKNOWN)
+#  if defined(DT_UNKNOWN)
     // Position of field |d_type| in |dirent|
     // Not strictly posix, but seems defined on all platforms
     // except mingw32.
     {"OSFILE_OFFSETOF_DIRENT_D_TYPE",
      JS::Int32Value(offsetof(struct dirent, d_type))},
-#endif  // defined(DT_UNKNOWN)
+#  endif  // defined(DT_UNKNOWN)
 
 // Under MacOS X and BSDs, |dirfd| is a macro rather than a
 // function, so we need a little help to get it to work
-#if defined(dirfd)
+#  if defined(dirfd)
     {"OSFILE_SIZEOF_DIR", JS::Int32Value(sizeof(DIR))},
 
     {"OSFILE_OFFSETOF_DIR_DD_FD", JS::Int32Value(offsetof(DIR, __dd_fd))},
-#endif
+#  endif
 
     // Defining |stat|
 
@@ -617,32 +584,35 @@ static const dom::ConstantSpec gLibcProperties[] = {
     {"OSFILE_OFFSETOF_STAT_ST_SIZE",
      JS::Int32Value(offsetof(struct stat, st_size))},
 
-#if defined(HAVE_ST_ATIMESPEC)
+#  if defined(HAVE_ST_ATIMESPEC)
     {"OSFILE_OFFSETOF_STAT_ST_ATIME",
      JS::Int32Value(offsetof(struct stat, st_atimespec))},
     {"OSFILE_OFFSETOF_STAT_ST_MTIME",
      JS::Int32Value(offsetof(struct stat, st_mtimespec))},
     {"OSFILE_OFFSETOF_STAT_ST_CTIME",
      JS::Int32Value(offsetof(struct stat, st_ctimespec))},
-#else
+#  else
     {"OSFILE_OFFSETOF_STAT_ST_ATIME",
      JS::Int32Value(offsetof(struct stat, st_atime))},
     {"OSFILE_OFFSETOF_STAT_ST_MTIME",
      JS::Int32Value(offsetof(struct stat, st_mtime))},
     {"OSFILE_OFFSETOF_STAT_ST_CTIME",
      JS::Int32Value(offsetof(struct stat, st_ctime))},
-#endif  // defined(HAVE_ST_ATIME)
+#  endif  // defined(HAVE_ST_ATIME)
 
 // Several OSes have a birthtime field. For the moment, supporting only Darwin.
-#if defined(_DARWIN_FEATURE_64_BIT_INODE)
+#  if defined(_DARWIN_FEATURE_64_BIT_INODE)
     {"OSFILE_OFFSETOF_STAT_ST_BIRTHTIME",
      JS::Int32Value(offsetof(struct stat, st_birthtime))},
-#endif  // defined(_DARWIN_FEATURE_64_BIT_INODE)
+#  endif  // defined(_DARWIN_FEATURE_64_BIT_INODE)
 
     // Defining |statvfs|
 
     {"OSFILE_SIZEOF_STATVFS", JS::Int32Value(sizeof(struct statvfs))},
 
+    // We have no guarantee how big "f_frsize" is, so we have to calculate that.
+    {"OSFILE_SIZEOF_STATVFS_F_FRSIZE",
+     JS::Int32Value(DOM_SIZEOF_MEMBER(struct statvfs, f_frsize))},
     {"OSFILE_OFFSETOF_STATVFS_F_FRSIZE",
      JS::Int32Value(offsetof(struct statvfs, f_frsize))},
     {"OSFILE_OFFSETOF_STATVFS_F_BAVAIL",
@@ -766,9 +736,9 @@ static const dom::ConstantSpec gWinProperties[] = {
  * If the field does not exist, create it. If it exists but is not an
  * object, throw a JS error.
  */
-JSObject *GetOrCreateObjectProperty(JSContext *cx,
-                                    JS::Handle<JSObject *> aObject,
-                                    const char *aProperty) {
+JSObject* GetOrCreateObjectProperty(JSContext* cx,
+                                    JS::Handle<JSObject*> aObject,
+                                    const char* aProperty) {
   JS::Rooted<JS::Value> val(cx);
   if (!JS_GetProperty(cx, aObject, aProperty, &val)) {
     return nullptr;
@@ -791,12 +761,12 @@ JSObject *GetOrCreateObjectProperty(JSContext *cx,
  *
  * If the nsString is void (i.e. IsVoid is true), do nothing.
  */
-bool SetStringProperty(JSContext *cx, JS::Handle<JSObject *> aObject,
-                       const char *aProperty, const nsString aValue) {
+bool SetStringProperty(JSContext* cx, JS::Handle<JSObject*> aObject,
+                       const char* aProperty, const nsString aValue) {
   if (aValue.IsVoid()) {
     return true;
   }
-  JSString *strValue = JS_NewUCStringCopyZ(cx, aValue.get());
+  JSString* strValue = JS_NewUCStringCopyZ(cx, aValue.get());
   NS_ENSURE_TRUE(strValue, false);
   JS::Rooted<JS::Value> valValue(cx, JS::StringValue(strValue));
   return JS_SetProperty(cx, aObject, aProperty, valValue);
@@ -809,7 +779,7 @@ bool SetStringProperty(JSContext *cx, JS::Handle<JSObject *> aObject,
  * all its constants.
  */
 bool OSFileConstantsService::DefineOSFileConstants(
-    JSContext *aCx, JS::Handle<JSObject *> aGlobal) {
+    JSContext* aCx, JS::Handle<JSObject*> aGlobal) {
   if (!mInitialized) {
     JS_ReportErrorNumberASCII(aCx, js::GetErrorMessage, nullptr,
                               JSMSG_CANT_OPEN, "OSFileConstants",
@@ -817,18 +787,18 @@ bool OSFileConstantsService::DefineOSFileConstants(
     return false;
   }
 
-  JS::Rooted<JSObject *> objOS(aCx);
+  JS::Rooted<JSObject*> objOS(aCx);
   if (!(objOS = GetOrCreateObjectProperty(aCx, aGlobal, "OS"))) {
     return false;
   }
-  JS::Rooted<JSObject *> objConstants(aCx);
+  JS::Rooted<JSObject*> objConstants(aCx);
   if (!(objConstants = GetOrCreateObjectProperty(aCx, objOS, "Constants"))) {
     return false;
   }
 
   // Build OS.Constants.libc
 
-  JS::Rooted<JSObject *> objLibc(aCx);
+  JS::Rooted<JSObject*> objLibc(aCx);
   if (!(objLibc = GetOrCreateObjectProperty(aCx, objConstants, "libc"))) {
     return false;
   }
@@ -839,7 +809,7 @@ bool OSFileConstantsService::DefineOSFileConstants(
 #if defined(XP_WIN)
   // Build OS.Constants.Win
 
-  JS::Rooted<JSObject *> objWin(aCx);
+  JS::Rooted<JSObject*> objWin(aCx);
   if (!(objWin = GetOrCreateObjectProperty(aCx, objConstants, "Win"))) {
     return false;
   }
@@ -850,7 +820,7 @@ bool OSFileConstantsService::DefineOSFileConstants(
 
   // Build OS.Constants.Sys
 
-  JS::Rooted<JSObject *> objSys(aCx);
+  JS::Rooted<JSObject*> objSys(aCx);
   if (!(objSys = GetOrCreateObjectProperty(aCx, objConstants, "Sys"))) {
     return false;
   }
@@ -862,7 +832,7 @@ bool OSFileConstantsService::DefineOSFileConstants(
     DebugOnly<nsresult> rv = runtime->GetOS(os);
     MOZ_ASSERT(NS_SUCCEEDED(rv));
 
-    JSString *strVersion = JS_NewStringCopyZ(aCx, os.get());
+    JSString* strVersion = JS_NewStringCopyZ(aCx, os.get());
     if (!strVersion) {
       return false;
     }
@@ -897,14 +867,14 @@ bool OSFileConstantsService::DefineOSFileConstants(
 
   // Build OS.Constants.Path
 
-  JS::Rooted<JSObject *> objPath(aCx);
+  JS::Rooted<JSObject*> objPath(aCx);
   if (!(objPath = GetOrCreateObjectProperty(aCx, objConstants, "Path"))) {
     return false;
   }
 
-    // Locate libxul
-    // Note that we don't actually provide the full path, only the name of the
-    // library, which is sufficient to link to the library using js-ctypes.
+  // Locate libxul
+  // Note that we don't actually provide the full path, only the name of the
+  // library, which is sufficient to link to the library using js-ctypes.
 
 #if defined(XP_MACOSX)
   // Under MacOS X, for some reason, libxul is called simply "XUL",
@@ -916,9 +886,9 @@ bool OSFileConstantsService::DefineOSFileConstants(
   // On other platforms, libxul is a library "xul" with regular
   // library prefix/suffix.
   nsAutoString libxul;
-  libxul.AppendLiteral(DLL_PREFIX);
+  libxul.AppendLiteral(MOZ_DLL_PREFIX);
   libxul.AppendLiteral("xul");
-  libxul.AppendLiteral(DLL_SUFFIX);
+  libxul.AppendLiteral(MOZ_DLL_SUFFIX);
 #endif  // defined(XP_MACOSX)
 
   if (!SetStringProperty(aCx, objPath, "libxul", libxul)) {
@@ -946,34 +916,18 @@ bool OSFileConstantsService::DefineOSFileConstants(
     return false;
   }
 
-  if (!SetStringProperty(aCx, objPath, "homeDir", mPaths->homeDir)) {
-    return false;
-  }
-
-  if (!SetStringProperty(aCx, objPath, "userApplicationDataDir",
-                         mPaths->userApplicationDataDir)) {
-    return false;
-  }
-
-#if defined(XP_MACOSX)
-  if (!SetStringProperty(aCx, objPath, "macUserLibDir",
-                         mPaths->macUserLibDir)) {
-    return false;
-  }
-#endif  // defined(XP_MACOSX)
-
   // sqlite3 is linked from different places depending on the platform
   nsAutoString libsqlite3;
 #if defined(ANDROID)
   // On Android, we use the system's libsqlite3
-  libsqlite3.AppendLiteral(DLL_PREFIX);
+  libsqlite3.AppendLiteral(MOZ_DLL_PREFIX);
   libsqlite3.AppendLiteral("sqlite3");
-  libsqlite3.AppendLiteral(DLL_SUFFIX);
+  libsqlite3.AppendLiteral(MOZ_DLL_SUFFIX);
 #elif defined(XP_WIN)
   // On Windows, for some reason, this is part of nss3.dll
-  libsqlite3.AppendLiteral(DLL_PREFIX);
+  libsqlite3.AppendLiteral(MOZ_DLL_PREFIX);
   libsqlite3.AppendLiteral("nss3");
-  libsqlite3.AppendLiteral(DLL_SUFFIX);
+  libsqlite3.AppendLiteral(MOZ_DLL_SUFFIX);
 #else
   // On other platforms, we link sqlite3 into libxul
   libsqlite3 = libxul;
@@ -989,8 +943,8 @@ bool OSFileConstantsService::DefineOSFileConstants(
 NS_IMPL_ISUPPORTS(OSFileConstantsService, nsIOSFileConstantsService,
                   nsIObserver)
 
-/* static */ already_AddRefed<OSFileConstantsService>
-OSFileConstantsService::GetOrCreate() {
+/* static */
+already_AddRefed<OSFileConstantsService> OSFileConstantsService::GetOrCreate() {
   if (!gInstance) {
     MOZ_ASSERT(NS_IsMainThread());
 
@@ -1018,7 +972,7 @@ OSFileConstantsService::~OSFileConstantsService() {
 }
 
 NS_IMETHODIMP
-OSFileConstantsService::Init(JSContext *aCx) {
+OSFileConstantsService::Init(JSContext* aCx) {
   MOZ_ASSERT(NS_IsMainThread());
 
   nsresult rv = InitOSFileConstants();
@@ -1026,8 +980,8 @@ OSFileConstantsService::Init(JSContext *aCx) {
     return rv;
   }
 
-  mozJSComponentLoader *loader = mozJSComponentLoader::Get();
-  JS::Rooted<JSObject *> targetObj(aCx);
+  mozJSComponentLoader* loader = mozJSComponentLoader::Get();
+  JS::Rooted<JSObject*> targetObj(aCx);
   loader->FindTargetObject(aCx, &targetObj);
 
   if (!DefineOSFileConstants(aCx, targetObj)) {
