@@ -3,12 +3,11 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsHtml5PlainTextUtils.h"
-#include "mozilla/Preferences.h"
+#include "mozilla/StaticPrefs.h"
 #include "nsHtml5AttributeName.h"
 #include "nsHtml5Portability.h"
 #include "nsHtml5String.h"
-#include "nsIServiceManager.h"
-#include "nsIStringBundle.h"
+#include "nsGkAtoms.h"
 
 // static
 nsHtml5HtmlAttributes* nsHtml5PlainTextUtils::NewLinkAttributes() {
@@ -21,3 +20,15 @@ nsHtml5HtmlAttributes* nsHtml5PlainTextUtils::NewLinkAttributes() {
   linkAttrs->addAttribute(nsHtml5AttributeName::ATTR_HREF, href, -1);
   return linkAttrs;
 }
+
+// static
+nsHtml5HtmlAttributes* nsHtml5PlainTextUtils::NewBodyAttributes() {
+  if (mozilla::StaticPrefs::plain_text_wrap_long_lines()) {
+    return nsHtml5HtmlAttributes::EMPTY_ATTRIBUTES;
+  }
+  nsHtml5HtmlAttributes* bodyAttrs = new nsHtml5HtmlAttributes(0);
+  RefPtr<nsAtom> nowrap = nsGkAtoms::nowrap;
+  bodyAttrs->addAttribute(nsHtml5AttributeName::ATTR_CLASS,
+                          nsHtml5String::FromAtom(nowrap.forget()), -1);
+  return bodyAttrs;
+}
