@@ -14,16 +14,16 @@
 
 #include "prio.h"
 #if defined(XP_WIN)
-#include <windows.h>
+#  include <windows.h>
 #elif defined(MOZ_WIDGET_COCOA)
-#include <CoreServices/CoreServices.h>
-#include "nsCocoaFeatures.h"
+#  include <CoreServices/CoreServices.h>
+#  include "nsCocoaFeatures.h"
 #elif defined(MOZ_WIDGET_GTK)
-#include <gtk/gtk.h>
+#  include <gtk/gtk.h>
 #endif
 
 #ifdef MOZ_WIDGET_ANDROID
-#include "AndroidBridge.h"
+#  include "AndroidBridge.h"
 #endif
 
 #include "mozilla/Services.h"
@@ -41,107 +41,76 @@
 
 using namespace mozilla;
 
-struct ManifestDirective
-{
+struct ManifestDirective {
   const char* directive;
   int argc;
 
-  // Binary components are only allowed for APP locations.
-  bool apponly;
-
-  // Some directives should only be delivered for APP or EXTENSION locations.
-  bool componentonly;
-
   bool ischrome;
 
-  bool allowbootstrap;
-
   // The contentaccessible flags only apply to content/resource directives.
   bool contentflags;
 
   // Function to handle this directive. This isn't a union because C++ still
   // hasn't learned how to initialize unions in a sane way.
   void (nsComponentManagerImpl::*mgrfunc)(
-    nsComponentManagerImpl::ManifestProcessingContext& aCx,
-    int aLineNo, char* const* aArgv);
+      nsComponentManagerImpl::ManifestProcessingContext& aCx, int aLineNo,
+      char* const* aArgv);
   void (nsChromeRegistry::*regfunc)(
-    nsChromeRegistry::ManifestProcessingContext& aCx,
-    int aLineNo, char* const* aArgv, int aFlags);
-
-  bool isContract;
+      nsChromeRegistry::ManifestProcessingContext& aCx, int aLineNo,
+      char* const* aArgv, int aFlags);
 };
 static const ManifestDirective kParsingTable[] = {
+    // clang-format off
   {
-    "manifest",         1, false, false, true, true, false,
+    "manifest",         1, true, false,
     &nsComponentManagerImpl::ManifestManifest, nullptr,
   },
   {
-    "binary-component", 1, true, true, false, false, false,
-    &nsComponentManagerImpl::ManifestBinaryComponent, nullptr,
-  },
-  {
-    "interfaces",       1, false, true, false, false, false,
-    &nsComponentManagerImpl::ManifestXPT, nullptr,
-  },
-  {
-    "component",        2, false, true, false, false, false,
+    "component",        2, false, false,
     &nsComponentManagerImpl::ManifestComponent, nullptr,
   },
   {
-    "contract",         2, false, true, false, false, false,
+    "contract",         2, false, false,
     &nsComponentManagerImpl::ManifestContract, nullptr,
   },
   {
-    "category",         3, false, true, false, false, false,
+    "category",         3, false, false,
     &nsComponentManagerImpl::ManifestCategory, nullptr,
   },
   {
-    "content",          2, false, true, true, true,  true,
+    "content",          2, true,  true,
     nullptr, &nsChromeRegistry::ManifestContent,
   },
   {
-    "locale",           3, false, true, true, true, false,
+    "locale",           3, true, false,
     nullptr, &nsChromeRegistry::ManifestLocale,
   },
   {
-    "skin",             3, false, false, true, true, false,
+    "skin",             3, true, false,
     nullptr, &nsChromeRegistry::ManifestSkin,
   },
-  {
-    "overlay",          2, false, true, true, false, false,
-    nullptr, &nsChromeRegistry::ManifestOverlay,
-  },
-  {
-    "style",            2, false, false, true, false, false,
-    nullptr, &nsChromeRegistry::ManifestStyle,
-  },
   {
     // NB: note that while skin manifests can use this, they are only allowed
     // to use it for chrome://../skin/ URLs
-    "override",         2, false, false, true, true, false,
+    "override",         2, true, false,
     nullptr, &nsChromeRegistry::ManifestOverride,
   },
   {
-    "resource",         2, false, true, true, true, true,
+    "resource",         2, false, true,
     nullptr, &nsChromeRegistry::ManifestResource,
   }
+    // clang-format on
 };
 
 static const char kWhitespace[] = "\t ";
 
-static bool
-IsNewline(char aChar)
-{
-  return aChar == '\n' || aChar == '\r';
-}
+static bool IsNewline(char aChar) { return aChar == '\n' || aChar == '\r'; }
 
-void
-LogMessage(const char* aMsg, ...)
-{
+void LogMessage(const char* aMsg, ...) {
   MOZ_ASSERT(nsComponentManagerImpl::gComponentManager);
 
   nsCOMPtr<nsIConsoleService> console =
-    do_GetService(NS_CONSOLESERVICE_CONTRACTID);
+      do_GetService(NS_CONSOLESERVICE_CONTRACTID);
   if (!console) {
     return;
   }
@@ -152,14 +121,12 @@ LogMessage(const char* aMsg, ...)
   va_end(args);
 
   nsCOMPtr<nsIConsoleMessage> error =
-    new nsConsoleMessage(NS_ConvertUTF8toUTF16(formatted.get()).get());
+      new nsConsoleMessage(NS_ConvertUTF8toUTF16(formatted.get()).get());
   console->LogMessage(error);
 }
 
-void
-LogMessageWithContext(FileLocation& aFile,
-                      uint32_t aLineNumber, const char* aMsg, ...)
-{
+void LogMessageWithContext(FileLocation& aFile, uint32_t aLineNumber,
+                           const char* aMsg, ...) {
   va_list args;
   va_start(args, aMsg);
   SmprintfPointer formatted(mozilla::Vsmprintf(aMsg, args));
@@ -173,26 +140,26 @@ LogMessageWithContext(FileLocation& aFile,
   nsCString file;
   aFile.GetURIString(file);
 
-  nsCOMPtr<nsIScriptError> error =
-    do_CreateInstance(NS_SCRIPTERROR_CONTRACTID);
+  nsCOMPtr<nsIScriptError> error = do_CreateInstance(NS_SCRIPTERROR_CONTRACTID);
   if (!error) {
     // This can happen early in component registration. Fall back to a
     // generic console message.
-    LogMessage("Warning: in '%s', line %i: %s", file.get(),
-               aLineNumber, formatted.get());
+    LogMessage("Warning: in '%s', line %i: %s", file.get(), aLineNumber,
+               formatted.get());
     return;
   }
 
   nsCOMPtr<nsIConsoleService> console =
-    do_GetService(NS_CONSOLESERVICE_CONTRACTID);
+      do_GetService(NS_CONSOLESERVICE_CONTRACTID);
   if (!console) {
     return;
   }
 
-  nsresult rv = error->Init(NS_ConvertUTF8toUTF16(formatted.get()),
-                            NS_ConvertUTF8toUTF16(file), EmptyString(),
-                            aLineNumber, 0, nsIScriptError::warningFlag,
-                            "chrome registration");
+  nsresult rv = error->Init(
+      NS_ConvertUTF8toUTF16(formatted.get()), NS_ConvertUTF8toUTF16(file),
+      EmptyString(), aLineNumber, 0, nsIScriptError::warningFlag,
+      "chrome registration", false /* from private window */,
+      true /* from chrome context */);
   if (NS_FAILED(rv)) {
     return;
   }
@@ -211,9 +178,8 @@ LogMessageWithContext(FileLocation& aFile,
  * @param aResult If the flag is found, the value is assigned here.
  * @return Whether the flag was handled.
  */
-static bool
-CheckFlag(const nsAString& aFlag, const nsAString& aData, bool& aResult)
-{
+static bool CheckFlag(const nsAString& aFlag, const nsAString& aData,
+                      bool& aResult) {
   if (!StringBeginsWith(aData, aFlag)) {
     return false;
   }
@@ -236,14 +202,14 @@ CheckFlag(const nsAString& aFlag, const nsAString& aData, bool& aResult)
 
   switch (aData.CharAt(aFlag.Length() + 1)) {
     case '1':
-    case 't': //true
-    case 'y': //yes
+    case 't':  // true
+    case 'y':  // yes
       aResult = true;
       return true;
 
     case '0':
-    case 'f': //false
-    case 'n': //no
+    case 'f':  // false
+    case 'n':  // no
       aResult = false;
       return true;
   }
@@ -251,12 +217,7 @@ CheckFlag(const nsAString& aFlag, const nsAString& aData, bool& aResult)
   return false;
 }
 
-enum TriState
-{
-  eUnspecified,
-  eBad,
-  eOK
-};
+enum TriState { eUnspecified, eBad, eOK };
 
 /**
  * Check for a modifier flag of the following form:
@@ -270,10 +231,8 @@ enum TriState
  *                Otherwise if the flag is found it is set to eBad or eOK.
  * @return Whether the flag was handled.
  */
-static bool
-CheckStringFlag(const nsAString& aFlag, const nsAString& aData,
-                const nsAString& aValue, TriState& aResult)
-{
+static bool CheckStringFlag(const nsAString& aFlag, const nsAString& aData,
+                            const nsAString& aValue, TriState& aResult) {
   if (aData.Length() < aFlag.Length() + 1) {
     return false;
   }
@@ -284,8 +243,7 @@ CheckStringFlag(const nsAString& aFlag, const nsAString& aData,
 
   bool comparison = true;
   if (aData[aFlag.Length()] != '=') {
-    if (aData[aFlag.Length()] == '!' &&
-        aData.Length() >= aFlag.Length() + 2 &&
+    if (aData[aFlag.Length()] == '!' && aData.Length() >= aFlag.Length() + 2 &&
         aData[aFlag.Length() + 1] == '=') {
       comparison = false;
     } else {
@@ -295,7 +253,7 @@ CheckStringFlag(const nsAString& aFlag, const nsAString& aData,
 
   if (aResult != eOK) {
     nsDependentSubstring testdata =
-      Substring(aData, aFlag.Length() + (comparison ? 1 : 2));
+        Substring(aData, aFlag.Length() + (comparison ? 1 : 2));
     if (testdata.Equals(aValue)) {
       aResult = comparison ? eOK : eBad;
     } else {
@@ -306,14 +264,13 @@ CheckStringFlag(const nsAString& aFlag, const nsAString& aData,
   return true;
 }
 
-static bool
-CheckOsFlag(const nsAString& aFlag, const nsAString& aData,
-            const nsAString& aValue, TriState& aResult)
-{
+static bool CheckOsFlag(const nsAString& aFlag, const nsAString& aData,
+                        const nsAString& aValue, TriState& aResult) {
   bool result = CheckStringFlag(aFlag, aData, aValue, aResult);
 #if defined(XP_UNIX) && !defined(XP_DARWIN) && !defined(ANDROID)
   if (result && aResult == eBad) {
-    result = CheckStringFlag(aFlag, aData, NS_LITERAL_STRING("likeunix"), aResult);
+    result =
+        CheckStringFlag(aFlag, aData, NS_LITERAL_STRING("likeunix"), aResult);
   }
 #endif
   return result;
@@ -336,14 +293,12 @@ CheckOsFlag(const nsAString& aFlag, const nsAString& aData,
  * @return Whether the flag was handled.
  */
 
-#define COMPARE_EQ    1 << 0
-#define COMPARE_LT    1 << 1
-#define COMPARE_GT    1 << 2
+#define COMPARE_EQ 1 << 0
+#define COMPARE_LT 1 << 1
+#define COMPARE_GT 1 << 2
 
-static bool
-CheckVersionFlag(const nsString& aFlag, const nsString& aData,
-                 const nsString& aValue, TriState& aResult)
-{
+static bool CheckVersionFlag(const nsString& aFlag, const nsString& aData,
+                             const nsString& aValue, TriState& aResult) {
   if (aData.Length() < aFlag.Length() + 2) {
     return false;
   }
@@ -412,9 +367,7 @@ CheckVersionFlag(const nsString& aFlag, const nsString& aData,
 }
 
 // In-place conversion of ascii characters to lower case
-static void
-ToLowerCase(char* aToken)
-{
+static void ToLowerCase(char* aToken) {
   for (; *aToken; ++aToken) {
     *aToken = NS_ToLower(*aToken);
   }
@@ -422,19 +375,15 @@ ToLowerCase(char* aToken)
 
 namespace {
 
-struct CachedDirective
-{
+struct CachedDirective {
   int lineno;
   char* argv[4];
 };
 
-} // namespace
+}  // namespace
 
-
-void
-ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
-              bool aChromeOnly)
-{
+void ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
+                   bool aChromeOnly) {
   nsComponentManagerImpl::ManifestProcessingContext mgrcx(aType, aFile,
                                                           aChromeOnly);
   nsChromeRegistry::ManifestProcessingContext chromecx(aType, aFile);
@@ -505,31 +454,26 @@ ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
 
   nsAutoString osVersion;
 #if defined(XP_WIN)
-#pragma warning(push)
-#pragma warning(disable:4996) // VC12+ deprecates GetVersionEx
-  OSVERSIONINFO info = { sizeof(OSVERSIONINFO) };
+#  pragma warning(push)
+#  pragma warning(disable : 4996)  // VC12+ deprecates GetVersionEx
+  OSVERSIONINFO info = {sizeof(OSVERSIONINFO)};
   if (GetVersionEx(&info)) {
-    nsTextFormatter::ssprintf(osVersion, u"%ld.%ld",
-                              info.dwMajorVersion,
+    nsTextFormatter::ssprintf(osVersion, u"%ld.%ld", info.dwMajorVersion,
                               info.dwMinorVersion);
   }
-#pragma warning(pop)
+#  pragma warning(pop)
 #elif defined(MOZ_WIDGET_COCOA)
   SInt32 majorVersion = nsCocoaFeatures::OSXVersionMajor();
   SInt32 minorVersion = nsCocoaFeatures::OSXVersionMinor();
-  nsTextFormatter::ssprintf(osVersion, u"%ld.%ld",
-                            majorVersion,
-                            minorVersion);
+  nsTextFormatter::ssprintf(osVersion, u"%ld.%ld", majorVersion, minorVersion);
 #elif defined(MOZ_WIDGET_GTK)
-  nsTextFormatter::ssprintf(osVersion, u"%ld.%ld",
-                            gtk_major_version,
+  nsTextFormatter::ssprintf(osVersion, u"%ld.%ld", gtk_major_version,
                             gtk_minor_version);
 #elif defined(MOZ_WIDGET_ANDROID)
   bool isTablet = false;
   if (mozilla::AndroidBridge::Bridge()) {
-    mozilla::AndroidBridge::Bridge()->GetStaticStringField("android/os/Build$VERSION",
-                                                           "RELEASE",
-                                                           osVersion);
+    mozilla::AndroidBridge::Bridge()->GetStaticStringField(
+        "android/os/Build$VERSION", "RELEASE", osVersion);
     isTablet = java::GeckoAppShell::IsTablet();
   }
 #endif
@@ -569,7 +513,7 @@ ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
     }
     ++line;
 
-    if (*token == '#') { // ignore lines that begin with # as comments
+    if (*token == '#') {  // ignore lines that begin with # as comments
       continue;
     }
 
@@ -581,8 +525,7 @@ ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
 
     const ManifestDirective* directive = nullptr;
     for (const ManifestDirective* d = kParsingTable;
-         d < ArrayEnd(kParsingTable);
-         ++d) {
+         d < ArrayEnd(kParsingTable); ++d) {
       if (!strcmp(d->directive, token)) {
         directive = d;
         break;
@@ -590,31 +533,16 @@ ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
     }
 
     if (!directive) {
-      LogMessageWithContext(aFile, line,
-                            "Ignoring unrecognized chrome manifest directive '%s'.",
-                            token);
+      LogMessageWithContext(
+          aFile, line, "Ignoring unrecognized chrome manifest directive '%s'.",
+          token);
       continue;
     }
 
-    if (!directive->allowbootstrap && NS_BOOTSTRAPPED_LOCATION == aType) {
-      LogMessageWithContext(aFile, line,
-                            "Bootstrapped manifest not allowed to use '%s' directive.",
-                            token);
-      continue;
-    }
-
-#ifndef MOZ_BINARY_EXTENSIONS
-    if (directive->apponly && NS_APP_LOCATION != aType) {
-      LogMessageWithContext(aFile, line,
-                            "Only application manifests may use the '%s' directive.", token);
-      continue;
-    }
-#endif
-
-    if (directive->componentonly && NS_SKIN_LOCATION == aType) {
-      LogMessageWithContext(aFile, line,
-                            "Skin manifest not allowed to use '%s' directive.",
-                            token);
+    if (!directive->ischrome && NS_BOOTSTRAPPED_LOCATION == aType) {
+      LogMessageWithContext(
+          aFile, line,
+          "Bootstrapped manifest not allowed to use '%s' directive.", token);
       continue;
     }
 
@@ -626,7 +554,8 @@ ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
 
     if (!argv[directive->argc - 1]) {
       LogMessageWithContext(aFile, line,
-                            "Not enough arguments for chrome manifest directive '%s', expected %i.",
+                            "Not enough arguments for chrome manifest "
+                            "directive '%s', expected %i.",
                             token, directive->argc);
       continue;
     }
@@ -655,7 +584,8 @@ ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
           CheckStringFlag(kProcess, wtoken, process, stProcess) ||
           CheckVersionFlag(kOsVersion, wtoken, osVersion, stOsVersion) ||
           CheckVersionFlag(kAppVersion, wtoken, appVersion, stAppVersion) ||
-          CheckVersionFlag(kGeckoVersion, wtoken, geckoVersion, stGeckoVersion)) {
+          CheckVersionFlag(kGeckoVersion, wtoken, geckoVersion,
+                           stGeckoVersion)) {
         continue;
       }
 
@@ -670,47 +600,38 @@ ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
       if (directive->contentflags) {
         bool flag;
         if (CheckFlag(kContentAccessible, wtoken, flag)) {
-          if (flag)
-            flags |= nsChromeRegistry::CONTENT_ACCESSIBLE;
+          if (flag) flags |= nsChromeRegistry::CONTENT_ACCESSIBLE;
           continue;
         }
         if (CheckFlag(kRemoteEnabled, wtoken, flag)) {
-          if (flag)
-            flags |= nsChromeRegistry::REMOTE_ALLOWED;
+          if (flag) flags |= nsChromeRegistry::REMOTE_ALLOWED;
           continue;
         }
         if (CheckFlag(kRemoteRequired, wtoken, flag)) {
-          if (flag)
-            flags |= nsChromeRegistry::REMOTE_REQUIRED;
+          if (flag) flags |= nsChromeRegistry::REMOTE_REQUIRED;
           continue;
         }
       }
 
-      bool xpcNativeWrappers = true; // Dummy for CheckFlag.
+      bool xpcNativeWrappers = true;  // Dummy for CheckFlag.
       if (CheckFlag(kXPCNativeWrappers, wtoken, xpcNativeWrappers)) {
-        LogMessageWithContext(aFile, line,
-                              "Ignoring obsolete chrome registration modifier '%s'.",
-                              token);
+        LogMessageWithContext(
+            aFile, line, "Ignoring obsolete chrome registration modifier '%s'.",
+            token);
         continue;
       }
 
-      LogMessageWithContext(aFile, line,
-                            "Unrecognized chrome manifest modifier '%s'.",
-                            token);
+      LogMessageWithContext(
+          aFile, line, "Unrecognized chrome manifest modifier '%s'.", token);
       ok = false;
     }
 
-    if (!ok ||
-        stApp == eBad ||
-        stAppVersion == eBad ||
-        stGeckoVersion == eBad ||
-        stOs == eBad ||
-        stOsVersion == eBad ||
+    if (!ok || stApp == eBad || stAppVersion == eBad ||
+        stGeckoVersion == eBad || stOs == eBad || stOsVersion == eBad ||
 #ifdef MOZ_WIDGET_ANDROID
         stTablet == eBad ||
 #endif
-        stABI == eBad ||
-        stProcess == eBad) {
+        stABI == eBad || stProcess == eBad) {
       continue;
     }
 
@@ -721,7 +642,7 @@ ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
 
       if (!nsChromeRegistry::gChromeRegistry) {
         nsCOMPtr<nsIChromeRegistry> cr =
-          mozilla::services::GetChromeRegistryService();
+            mozilla::services::GetChromeRegistryService();
         if (!nsChromeRegistry::gChromeRegistry) {
           LogMessageWithContext(aFile, line,
                                 "Chrome registry isn't available yet.");
@@ -729,25 +650,17 @@ ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
         }
       }
 
-      (nsChromeRegistry::gChromeRegistry->*(directive->regfunc))(
-        chromecx, line, argv, flags);
+      (nsChromeRegistry::gChromeRegistry->*(directive->regfunc))(chromecx, line,
+                                                                 argv, flags);
     } else if (directive->ischrome || !aChromeOnly) {
-      if (directive->isContract) {
-        CachedDirective* cd = contracts.AppendElement();
-        cd->lineno = line;
-        cd->argv[0] = argv[0];
-        cd->argv[1] = argv[1];
-      } else {
-        (nsComponentManagerImpl::gComponentManager->*(directive->mgrfunc))(
+      (nsComponentManagerImpl::gComponentManager->*(directive->mgrfunc))(
           mgrcx, line, argv);
-      }
     }
   }
 
   for (uint32_t i = 0; i < contracts.Length(); ++i) {
     CachedDirective& d = contracts[i];
-    nsComponentManagerImpl::gComponentManager->ManifestContract(mgrcx,
-                                                                d.lineno,
+    nsComponentManagerImpl::gComponentManager->ManifestContract(mgrcx, d.lineno,
                                                                 d.argv);
   }
 }
