@@ -4,18 +4,24 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+// See the comment at the top of mfbt/HashTable.h for a comparison between
+// PLDHashTable and mozilla::HashTable.
+
 #ifndef PLDHashTable_h
 #define PLDHashTable_h
 
 #include "mozilla/Atomics.h"
-#include "mozilla/Attributes.h" // for MOZ_ALWAYS_INLINE
+#include "mozilla/Attributes.h"  // for MOZ_ALWAYS_INLINE
 #include "mozilla/fallible.h"
+#include "mozilla/FunctionTypeTraits.h"
+#include "mozilla/HashFunctions.h"
 #include "mozilla/MemoryReporting.h"
 #include "mozilla/Move.h"
 #include "mozilla/Types.h"
 #include "nscore.h"
 
-typedef uint32_t PLDHashNumber;
+using PLDHashNumber = mozilla::HashNumber;
+static const uint32_t kPLDHashNumberBits = mozilla::kHashNumberBits;
 
 class PLDHashTable;
 struct PLDHashTableOps;
@@ -31,17 +37,19 @@ struct PLDHashTableOps;
 // structure, for single static initialization per hash table sub-type.
 //
 // Each hash table sub-type should make its entry type a subclass of
-// PLDHashEntryHdr. The mKeyHash member contains the result of multiplying the
-// hash code returned from the hashKey callback (see below) by kGoldenRatio,
-// then constraining the result to avoid the magic 0 and 1 values. The stored
-// mKeyHash value is table size invariant, and it is maintained automatically
-// -- users need never access it.
-struct PLDHashEntryHdr
-{
-private:
-  friend class PLDHashTable;
+// PLDHashEntryHdr. PLDHashEntryHdr is merely a common superclass to present a
+// uniform interface to PLDHashTable clients. The zero-sized base class
+// optimization, employed by all of our supported C++ compilers, will ensure
+// that this abstraction does not make objects needlessly larger.
+struct PLDHashEntryHdr {
+  PLDHashEntryHdr() = default;
+  PLDHashEntryHdr(const PLDHashEntryHdr&) = delete;
+  PLDHashEntryHdr& operator=(const PLDHashEntryHdr&) = delete;
+  PLDHashEntryHdr(PLDHashEntryHdr&&) = default;
+  PLDHashEntryHdr& operator=(PLDHashEntryHdr&&) = default;
 
-  PLDHashNumber mKeyHash;
+ private:
+  friend class PLDHashTable;
 };
 
 #ifdef DEBUG
@@ -72,9 +80,8 @@ private:
 //
 // - mIsWritable, which indicates if the table is mutable.
 //
-class Checker
-{
-public:
+class Checker {
+ public:
   constexpr Checker() : mState(kIdle), mIsWritable(1) {}
 
   Checker& operator=(Checker&& aOther) {
@@ -87,9 +94,10 @@ public:
     return *this;
   }
 
-  static bool IsIdle(uint32_t aState)  { return aState == kIdle; }
-  static bool IsRead(uint32_t aState)  { return kRead1 <= aState &&
-                                                aState <= kReadMax; }
+  static bool IsIdle(uint32_t aState) { return aState == kIdle; }
+  static bool IsRead(uint32_t aState) {
+    return kRead1 <= aState && aState <= kReadMax;
+  }
   static bool IsRead1(uint32_t aState) { return aState == kRead1; }
   static bool IsWrite(uint32_t aState) { return aState == kWrite; }
 
@@ -112,28 +120,24 @@ public:
   // For |mIsWritable| we don't need to be as careful because it can only in
   // transition in one direction (from writable to non-writable).
 
-  void StartReadOp()
-  {
-    uint32_t oldState = mState++;     // this is an atomic increment
+  void StartReadOp() {
+    uint32_t oldState = mState++;  // this is an atomic increment
     MOZ_ASSERT(IsIdle(oldState) || IsRead(oldState));
     MOZ_ASSERT(oldState < kReadMax);  // check for overflow
   }
 
-  void EndReadOp()
-  {
-    uint32_t oldState = mState--;     // this is an atomic decrement
+  void EndReadOp() {
+    uint32_t oldState = mState--;  // this is an atomic decrement
     MOZ_ASSERT(IsRead(oldState));
   }
 
-  void StartWriteOp()
-  {
+  void StartWriteOp() {
     MOZ_ASSERT(IsWritable());
     uint32_t oldState = mState.exchange(kWrite);
     MOZ_ASSERT(IsIdle(oldState));
   }
 
-  void EndWriteOp()
-  {
+  void EndWriteOp() {
     // Check again that the table is writable, in case it was marked as
     // non-writable just after the IsWritable() assertion in StartWriteOp()
     // occurred.
@@ -142,8 +146,7 @@ public:
     MOZ_ASSERT(IsWrite(oldState));
   }
 
-  void StartIteratorRemovalOp()
-  {
+  void StartIteratorRemovalOp() {
     // When doing removals at the end of iteration, we go from Read1 state to
     // Write and then back.
     MOZ_ASSERT(IsWritable());
@@ -151,8 +154,7 @@ public:
     MOZ_ASSERT(IsRead1(oldState));
   }
 
-  void EndIteratorRemovalOp()
-  {
+  void EndIteratorRemovalOp() {
     // Check again that the table is writable, in case it was marked as
     // non-writable just after the IsWritable() assertion in
     // StartIteratorRemovalOp() occurred.
@@ -161,33 +163,35 @@ public:
     MOZ_ASSERT(IsWrite(oldState));
   }
 
-  void StartDestructorOp()
-  {
+  void StartDestructorOp() {
     // A destructor op is like a write, but the table doesn't need to be
     // writable.
     uint32_t oldState = mState.exchange(kWrite);
     MOZ_ASSERT(IsIdle(oldState));
   }
 
-  void EndDestructorOp()
-  {
+  void EndDestructorOp() {
     uint32_t oldState = mState.exchange(kIdle);
     MOZ_ASSERT(IsWrite(oldState));
   }
 
-private:
+ private:
   // Things of note about the representation of |mState|.
   // - The values between kRead1..kReadMax represent valid Read(n) values.
   // - kIdle and kRead1 are deliberately chosen so that incrementing the -
   //   former gives the latter.
   // - 9999 concurrent readers should be enough for anybody.
-  static const uint32_t kIdle    = 0;
-  static const uint32_t kRead1   = 1;
+  static const uint32_t kIdle = 0;
+  static const uint32_t kRead1 = 1;
   static const uint32_t kReadMax = 9999;
-  static const uint32_t kWrite   = 10000;
+  static const uint32_t kWrite = 10000;
 
-  mutable mozilla::Atomic<uint32_t> mState;
-  mutable mozilla::Atomic<uint32_t> mIsWritable;
+  mutable mozilla::Atomic<uint32_t, mozilla::SequentiallyConsistent,
+                          mozilla::recordreplay::Behavior::DontPreserve>
+      mState;
+  mutable mozilla::Atomic<uint32_t, mozilla::SequentiallyConsistent,
+                          mozilla::recordreplay::Behavior::DontPreserve>
+      mIsWritable;
 };
 #endif
 
@@ -204,35 +208,160 @@ private:
 // and use it after an add or remove operation, unless you sample Generation()
 // before adding or removing, and compare the sample after, dereferencing the
 // entry pointer only if Generation() has not changed.
-class PLDHashTable
-{
-private:
+class PLDHashTable {
+ private:
+  // A slot represents a cached hash value and its associated entry stored in
+  // the hash table. The hash value and the entry are not stored contiguously.
+  struct Slot {
+    Slot(PLDHashEntryHdr* aEntry, PLDHashNumber* aKeyHash)
+        : mEntry(aEntry), mKeyHash(aKeyHash) {}
+
+    Slot(const Slot&) = default;
+    Slot(Slot&& aOther) = default;
+
+    Slot& operator=(Slot&& aOther) {
+      this->~Slot();
+      new (this) Slot(std::move(aOther));
+      return *this;
+    }
+
+    bool operator==(const Slot& aOther) { return mEntry == aOther.mEntry; }
+
+    PLDHashNumber KeyHash() const { return *HashPtr(); }
+    void SetKeyHash(PLDHashNumber aHash) { *HashPtr() = aHash; }
+
+    PLDHashEntryHdr* ToEntry() const { return mEntry; }
+
+    bool IsFree() const { return KeyHash() == 0; }
+    bool IsRemoved() const { return KeyHash() == 1; }
+    bool IsLive() const { return IsLiveHash(KeyHash()); }
+    static bool IsLiveHash(uint32_t aHash) { return aHash >= 2; }
+
+    void MarkFree() { *HashPtr() = 0; }
+    void MarkRemoved() { *HashPtr() = 1; }
+    void MarkColliding() { *HashPtr() |= kCollisionFlag; }
+
+    void Next(uint32_t aEntrySize) {
+      char* p = reinterpret_cast<char*>(mEntry);
+      p += aEntrySize;
+      mEntry = reinterpret_cast<PLDHashEntryHdr*>(p);
+      mKeyHash++;
+    }
+    PLDHashNumber* HashPtr() const { return mKeyHash; }
+
+   private:
+    PLDHashEntryHdr* mEntry;
+    PLDHashNumber* mKeyHash;
+  };
+
   // This class maintains the invariant that every time the entry store is
   // changed, the generation is updated.
   //
+  // The data layout separates the cached hashes of entries and the entries
+  // themselves to save space. We could store the entries thusly:
+  //
+  // +--------+--------+---------+
+  // | entry0 | entry1 | ...     |
+  // +--------+--------+---------+
+  //
+  // where the entries themselves contain the cached hash stored as their
+  // first member. PLDHashTable did this for a long time, with entries looking
+  // like:
+  //
+  // class PLDHashEntryHdr
+  // {
+  //   PLDHashNumber mKeyHash;
+  // };
+  //
+  // class MyEntry : public PLDHashEntryHdr
+  // {
+  //   ...
+  // };
+  //
+  // The problem with this setup is that, depending on the layout of
+  // `MyEntry`, there may be platform ABI-mandated padding between `mKeyHash`
+  // and the first member of `MyEntry`. This ABI-mandated padding is wasted
+  // space, and was surprisingly common, e.g. when MyEntry contained a single
+  // pointer on 64-bit platforms.
+  //
+  // As previously alluded to, the current setup stores things thusly:
+  //
+  // +-------+-------+-------+-------+--------+--------+---------+
+  // | hash0 | hash1 | ..... | hashN | entry0 | entry1 | ...     |
+  // +-------+-------+-------+-------+--------+--------+---------+
+  //
+  // which contains no wasted space between the hashes themselves, and no
+  // wasted space between the entries themselves. malloc is guaranteed to
+  // return blocks of memory with at least word alignment on all of our major
+  // platforms. PLDHashTable mandates that the size of the hash table is
+  // always a power of two, so the alignment of the memory containing the
+  // first entry is always at least the alignment of the entire entry store.
+  // That means the alignment of `entry0` should be its natural alignment.
+  // Entries may have problems if they contain over-aligned members such as
+  // SIMD vector types, but this has not been a problem in practice.
+  //
   // Note: It would be natural to store the generation within this class, but
   // we can't do that without bloating sizeof(PLDHashTable) on 64-bit machines.
   // So instead we store it outside this class, and Set() takes a pointer to it
   // and ensures it is updated as necessary.
-  class EntryStore
-  {
-  private:
+  class EntryStore {
+   private:
     char* mEntryStore;
 
-  public:
+    static char* Entries(char* aStore, uint32_t aCapacity) {
+      return aStore + aCapacity * sizeof(PLDHashNumber);
+    }
+
+    char* Entries(uint32_t aCapacity) const {
+      return Entries(Get(), aCapacity);
+    }
+
+   public:
     EntryStore() : mEntryStore(nullptr) {}
 
-    ~EntryStore()
-    {
+    ~EntryStore() {
       free(mEntryStore);
       mEntryStore = nullptr;
     }
 
-    char* Get() { return mEntryStore; }
-    const char* Get() const { return mEntryStore; }
+    char* Get() const { return mEntryStore; }
 
-    void Set(char* aEntryStore, uint16_t* aGeneration)
-    {
+    Slot SlotForIndex(uint32_t aIndex, uint32_t aEntrySize,
+                      uint32_t aCapacity) const {
+      char* entries = Entries(aCapacity);
+      auto entry =
+          reinterpret_cast<PLDHashEntryHdr*>(entries + aIndex * aEntrySize);
+      auto hashes = reinterpret_cast<PLDHashNumber*>(Get());
+      return Slot(entry, &hashes[aIndex]);
+    }
+
+    Slot SlotForPLDHashEntry(PLDHashEntryHdr* aEntry, uint32_t aCapacity,
+                             uint32_t aEntrySize) {
+      char* entries = Entries(aCapacity);
+      char* entry = reinterpret_cast<char*>(aEntry);
+      uint32_t entryOffset = entry - entries;
+      uint32_t slotIndex = entryOffset / aEntrySize;
+      return SlotForIndex(slotIndex, aEntrySize, aCapacity);
+    }
+
+    template <typename F>
+    void ForEachSlot(uint32_t aCapacity, uint32_t aEntrySize, F&& aFunc) {
+      ForEachSlot(Get(), aCapacity, aEntrySize, std::move(aFunc));
+    }
+
+    template <typename F>
+    static void ForEachSlot(char* aStore, uint32_t aCapacity,
+                            uint32_t aEntrySize, F&& aFunc) {
+      char* entries = Entries(aStore, aCapacity);
+      Slot slot(reinterpret_cast<PLDHashEntryHdr*>(entries),
+                reinterpret_cast<PLDHashNumber*>(aStore));
+      for (size_t i = 0; i < aCapacity; ++i) {
+        aFunc(slot);
+        slot.Next(aEntrySize);
+      }
+    }
+
+    void Set(char* aEntryStore, uint16_t* aGeneration) {
       mEntryStore = aEntryStore;
       *aGeneration += 1;
     }
@@ -242,18 +371,18 @@ private:
   // sizeof(PLDHashTable) is 20. On 64-bit platforms, sizeof(PLDHashTable) is
   // 32; 28 bytes of data followed by 4 bytes of padding for alignment.
   const PLDHashTableOps* const mOps;  // Virtual operations; see below.
-  EntryStore          mEntryStore;    // (Lazy) entry storage and generation.
-  uint16_t            mGeneration;    // The storage generation.
-  uint8_t             mHashShift;     // Multiplicative hash shift.
-  const uint8_t       mEntrySize;     // Number of bytes in an entry.
-  uint32_t            mEntryCount;    // Number of entries in table.
-  uint32_t            mRemovedCount;  // Removed entry sentinels in table.
+  EntryStore mEntryStore;             // (Lazy) entry storage and generation.
+  uint16_t mGeneration;               // The storage generation.
+  uint8_t mHashShift;                 // Multiplicative hash shift.
+  const uint8_t mEntrySize;           // Number of bytes in an entry.
+  uint32_t mEntryCount;               // Number of entries in table.
+  uint32_t mRemovedCount;             // Removed entry sentinels in table.
 
 #ifdef DEBUG
   mutable Checker mChecker;
 #endif
 
-public:
+ public:
   // Table capacity limit; do not exceed. The max capacity used to be 1<<23 but
   // that occasionally that wasn't enough. Making it much bigger than 1<<26
   // probably isn't worthwhile -- tables that big are kind of ridiculous.
@@ -283,21 +412,18 @@ public:
                uint32_t aLength = kDefaultInitialLength);
 
   PLDHashTable(PLDHashTable&& aOther)
-      // We initialize mOps and mEntrySize here because they are |const|, and
-      // the move assignment operator cannot modify them.
-      // We initialize mEntryStore because it is required for a safe call to
-      // the destructor, which the move assignment operator does.
-      // We initialize mGeneration because it is modified by the move
-      // assignment operator.
-    : mOps(aOther.mOps)
-    , mEntryStore()
-    , mGeneration(0)
-    , mEntrySize(aOther.mEntrySize)
+      // Initialize fields which are checked by the move assignment operator
+      // and the destructor (which the move assignment operator calls).
+      : mOps(nullptr),
+        mEntryStore(),
+        mGeneration(0),
+        mEntrySize(0)
 #ifdef DEBUG
-    , mChecker()
+        ,
+        mChecker()
 #endif
   {
-    *this = mozilla::Move(aOther);
+    *this = std::move(aOther);
   }
 
   PLDHashTable& operator=(PLDHashTable&& aOther);
@@ -305,17 +431,21 @@ public:
   ~PLDHashTable();
 
   // This should be used rarely.
-  const PLDHashTableOps* Ops() const { return mOps; }
+  const PLDHashTableOps* Ops() const {
+    return mozilla::recordreplay::UnwrapPLDHashTableCallbacks(mOps);
+  }
+
+  // Provide access to the raw ops to internal record/replay structures.
+  const PLDHashTableOps* RecordReplayWrappedOps() const { return mOps; }
 
   // Size in entries (gross, not net of free and removed sentinels) for table.
   // This can be zero if no elements have been added yet, in which case the
   // entry storage will not have yet been allocated.
-  uint32_t Capacity() const
-  {
+  uint32_t Capacity() const {
     return mEntryStore.Get() ? CapacityFromHashShift() : 0;
   }
 
-  uint32_t EntrySize()  const { return mEntrySize; }
+  uint32_t EntrySize() const { return mEntrySize; }
   uint32_t EntryCount() const { return mEntryCount; }
   uint32_t Generation() const { return mGeneration; }
 
@@ -325,7 +455,7 @@ public:
   //
   // If |entry| is non-null, |key| was found. If |entry| is null, key was not
   // found.
-  PLDHashEntryHdr* Search(const void* aKey);
+  PLDHashEntryHdr* Search(const void* aKey) const;
 
   // To add an entry identified by |key| to table, call:
   //
@@ -334,11 +464,9 @@ public:
   // If |entry| is null upon return, then the table is severely overloaded and
   // memory can't be allocated for entry storage.
   //
-  // Otherwise, |aEntry->mKeyHash| has been set so that
-  // PLDHashTable::EntryIsFree(entry) is false, and it is up to the caller to
-  // initialize the key and value parts of the entry sub-type, if they have not
-  // been set already (i.e. if entry was not already in the table, and if the
-  // optional initEntry hook was not used).
+  // Otherwise, if the initEntry hook was provided, |entry| will be
+  // initialized.  If the initEntry hook was not provided, the caller
+  // should initialize |entry| as appropriate.
   PLDHashEntryHdr* Add(const void* aKey, const mozilla::fallible_t&);
 
   // This is like the other Add() function, but infallible, and so never
@@ -440,9 +568,8 @@ public:
   // The latter form is more verbose but is easier to work with when
   // making subclasses of Iterator.
   //
-  class Iterator
-  {
-  public:
+  class Iterator {
+   public:
     explicit Iterator(PLDHashTable* aTable);
     Iterator(Iterator&& aOther);
     ~Iterator();
@@ -451,13 +578,10 @@ public:
     bool Done() const { return mNexts == mNextsLimit; }
 
     // Get the current entry.
-    PLDHashEntryHdr* Get() const
-    {
+    PLDHashEntryHdr* Get() const {
       MOZ_ASSERT(!Done());
-
-      PLDHashEntryHdr* entry = reinterpret_cast<PLDHashEntryHdr*>(mCurrent);
-      MOZ_ASSERT(EntryIsLive(entry));
-      return entry;
+      MOZ_ASSERT(mCurrent.IsLive());
+      return mCurrent.ToEntry();
     }
 
     // Advance to the next entry.
@@ -467,20 +591,20 @@ public:
     // must not be called on that entry afterwards.
     void Remove();
 
-  protected:
-    PLDHashTable* mTable;             // Main table pointer.
+   protected:
+    PLDHashTable* mTable;  // Main table pointer.
 
-  private:
-    char* mStart;                     // The first entry.
-    char* mLimit;                     // One past the last entry.
-    char* mCurrent;                   // Pointer to the current entry.
-    uint32_t mNexts;                  // Number of Next() calls.
-    uint32_t mNextsLimit;             // Next() call limit.
+   private:
+    Slot mCurrent;         // Pointer to the current entry.
+    uint32_t mNexts;       // Number of Next() calls.
+    uint32_t mNextsLimit;  // Next() call limit.
 
-    bool mHaveRemoved;                // Have any elements been removed?
+    bool mHaveRemoved;   // Have any elements been removed?
+    uint8_t mEntrySize;  // Size of entries.
 
     bool IsOnNonLiveEntry() const;
-    void MoveToNextEntry();
+
+    void MoveToNextLiveEntry();
 
     Iterator() = delete;
     Iterator(const Iterator&) = delete;
@@ -492,68 +616,43 @@ public:
 
   // Use this if you need to initialize an Iterator in a const method. If you
   // use this case, you should not call Remove() on the iterator.
-  Iterator ConstIter() const
-  {
+  Iterator ConstIter() const {
     return Iterator(const_cast<PLDHashTable*>(this));
   }
 
-private:
-  // Multiplicative hash uses an unsigned 32 bit integer and the golden ratio,
-  // expressed as a fixed-point 32-bit fraction.
-  static const uint32_t kHashBits = 32;
-  static const uint32_t kGoldenRatio = 0x9E3779B9U;
-
+ private:
   static uint32_t HashShift(uint32_t aEntrySize, uint32_t aLength);
 
   static const PLDHashNumber kCollisionFlag = 1;
 
-  static bool EntryIsFree(PLDHashEntryHdr* aEntry)
-  {
-    return aEntry->mKeyHash == 0;
-  }
-  static bool EntryIsRemoved(PLDHashEntryHdr* aEntry)
-  {
-    return aEntry->mKeyHash == 1;
-  }
-  static bool EntryIsLive(PLDHashEntryHdr* aEntry)
-  {
-    return aEntry->mKeyHash >= 2;
-  }
+  PLDHashNumber Hash1(PLDHashNumber aHash0) const;
+  void Hash2(PLDHashNumber aHash, uint32_t& aHash2Out,
+             uint32_t& aSizeMaskOut) const;
 
-  static void MarkEntryFree(PLDHashEntryHdr* aEntry)
-  {
-    aEntry->mKeyHash = 0;
-  }
-  static void MarkEntryRemoved(PLDHashEntryHdr* aEntry)
-  {
-    aEntry->mKeyHash = 1;
-  }
-
-  PLDHashNumber Hash1(PLDHashNumber aHash0);
-  void Hash2(PLDHashNumber aHash, uint32_t& aHash2Out, uint32_t& aSizeMaskOut);
-
-  static bool MatchEntryKeyhash(PLDHashEntryHdr* aEntry, PLDHashNumber aHash);
-  PLDHashEntryHdr* AddressEntry(uint32_t aIndex);
+  static bool MatchSlotKeyhash(Slot& aSlot, const PLDHashNumber aHash);
+  Slot SlotForIndex(uint32_t aIndex) const;
 
   // We store mHashShift rather than sizeLog2 to optimize the collision-free
   // case in SearchTable.
-  uint32_t CapacityFromHashShift() const
-  {
-    return ((uint32_t)1 << (kHashBits - mHashShift));
+  uint32_t CapacityFromHashShift() const {
+    return ((uint32_t)1 << (kPLDHashNumberBits - mHashShift));
   }
 
-  PLDHashNumber ComputeKeyHash(const void* aKey);
+  PLDHashNumber ComputeKeyHash(const void* aKey) const;
 
   enum SearchReason { ForSearchOrRemove, ForAdd };
 
-  template <SearchReason Reason>
-  PLDHashEntryHdr* NS_FASTCALL
-    SearchTable(const void* aKey, PLDHashNumber aKeyHash);
+  // Avoid using bare `Success` and `Failure`, as those names are commonly
+  // defined as macros.
+  template <SearchReason Reason, typename PLDSuccess, typename PLDFailure>
+  auto SearchTable(const void* aKey, PLDHashNumber aKeyHash,
+                   PLDSuccess&& aSucess, PLDFailure&& aFailure) const;
 
-  PLDHashEntryHdr* FindFreeEntry(PLDHashNumber aKeyHash);
+  Slot FindFreeSlot(PLDHashNumber aKeyHash) const;
 
   bool ChangeTable(int aDeltaLog2);
 
+  void RawRemove(Slot& aSlot);
   void ShrinkIfAppropriate();
 
   PLDHashTable(const PLDHashTable& aOther) = delete;
@@ -582,10 +681,8 @@ typedef void (*PLDHashMoveEntry)(PLDHashTable* aTable,
 typedef void (*PLDHashClearEntry)(PLDHashTable* aTable,
                                   PLDHashEntryHdr* aEntry);
 
-// Initialize a new entry, apart from mKeyHash. This function is called when
-// Add() finds no existing entry for the given key, and must add a new one. At
-// that point, |aEntry->mKeyHash| is not set yet, to avoid claiming the last
-// free entry in a severely overloaded table.
+// Initialize a new entry. This function is called when
+// Add() finds no existing entry for the given key, and must add a new one.
 typedef void (*PLDHashInitEntry)(PLDHashEntryHdr* aEntry, const void* aKey);
 
 // Finally, the "vtable" structure for PLDHashTable. The first four hooks
@@ -599,30 +696,27 @@ typedef void (*PLDHashInitEntry)(PLDHashEntryHdr* aEntry, const void* aKey);
 //  clearEntry          Run dtor on entry.
 //
 // Note the reason why initEntry is optional: the default hooks (stubs) clear
-// entry storage:  On successful Add(tbl, key), the returned entry pointer
-// addresses an entry struct whose mKeyHash member has been set non-zero, but
-// all other entry members are still clear (null). Add() callers can test such
-// members to see whether the entry was newly created by the Add() call that
-// just succeeded. If placement new or similar initialization is required,
-// define an |initEntry| hook. Of course, the |clearEntry| hook must zero or
-// null appropriately.
+// entry storage. On a successful Add(tbl, key), the returned entry pointer
+// addresses an entry struct whose entry members are still clear (null). Add()
+// callers can test such members to see whether the entry was newly created by
+// the Add() call that just succeeded. If placement new or similar
+// initialization is required, define an |initEntry| hook. Of course, the
+// |clearEntry| hook must zero or null appropriately.
 //
 // XXX assumes 0 is null for pointer types.
-struct PLDHashTableOps
-{
+struct PLDHashTableOps {
   // Mandatory hooks. All implementations must provide these.
-  PLDHashHashKey      hashKey;
-  PLDHashMatchEntry   matchEntry;
-  PLDHashMoveEntry    moveEntry;
-  PLDHashClearEntry   clearEntry;
+  PLDHashHashKey hashKey;
+  PLDHashMatchEntry matchEntry;
+  PLDHashMoveEntry moveEntry;
+  PLDHashClearEntry clearEntry;
 
   // Optional hooks start here. If null, these are not called.
-  PLDHashInitEntry    initEntry;
+  PLDHashInitEntry initEntry;
 };
 
 // A minimal entry is a subclass of PLDHashEntryHdr and has a void* key pointer.
-struct PLDHashEntryStub : public PLDHashEntryHdr
-{
+struct PLDHashEntryStub : public PLDHashEntryHdr {
   const void* key;
 };
 
