@@ -34,25 +34,17 @@ static NS_DEFINE_CID(kStreamTransportServiceCID, NS_STREAMTRANSPORTSERVICE_CID);
 // This is a nsICancelableRunnable because we can dispatch it to Workers and
 // those can be shut down at any time, and in these cases, Cancel() is called
 // instead of Run().
-class nsInputStreamReadyEvent final
-  : public CancelableRunnable
-  , public nsIInputStreamCallback
-{
-public:
+class nsInputStreamReadyEvent final : public CancelableRunnable,
+                                      public nsIInputStreamCallback {
+ public:
   NS_DECL_ISUPPORTS_INHERITED
 
-    nsInputStreamReadyEvent(const char* aName,
-                            nsIInputStreamCallback* aCallback,
-                            nsIEventTarget* aTarget)
-    : CancelableRunnable(aName)
-    , mCallback(aCallback)
-    , mTarget(aTarget)
-  {
-  }
+  nsInputStreamReadyEvent(const char* aName, nsIInputStreamCallback* aCallback,
+                          nsIEventTarget* aTarget)
+      : CancelableRunnable(aName), mCallback(aCallback), mTarget(aTarget) {}
 
-private:
-  ~nsInputStreamReadyEvent()
-  {
+ private:
+  ~nsInputStreamReadyEvent() {
     if (!mCallback) {
       return;
     }
@@ -66,13 +58,13 @@ private:
     bool val;
     nsresult rv = mTarget->IsOnCurrentThread(&val);
     if (NS_FAILED(rv) || !val) {
-      nsCOMPtr<nsIInputStreamCallback> event =
-        NS_NewInputStreamReadyEvent("~nsInputStreamReadyEvent", mCallback, mTarget);
+      nsCOMPtr<nsIInputStreamCallback> event = NS_NewInputStreamReadyEvent(
+          "~nsInputStreamReadyEvent", mCallback, mTarget);
       mCallback = nullptr;
       if (event) {
         rv = event->OnInputStreamReady(nullptr);
         if (NS_FAILED(rv)) {
-          NS_NOTREACHED("leaking stream event");
+          MOZ_ASSERT_UNREACHABLE("leaking stream event");
           nsISupports* sup = event;
           NS_ADDREF(sup);
         }
@@ -80,13 +72,11 @@ private:
     }
   }
 
-public:
-  NS_IMETHOD OnInputStreamReady(nsIAsyncInputStream* aStream) override
-  {
+ public:
+  NS_IMETHOD OnInputStreamReady(nsIAsyncInputStream* aStream) override {
     mStream = aStream;
 
-    nsresult rv =
-      mTarget->Dispatch(this, NS_DISPATCH_NORMAL);
+    nsresult rv = mTarget->Dispatch(this, NS_DISPATCH_NORMAL);
     if (NS_FAILED(rv)) {
       NS_WARNING("Dispatch failed");
       return NS_ERROR_FAILURE;
@@ -95,8 +85,7 @@ public:
     return NS_OK;
   }
 
-  NS_IMETHOD Run() override
-  {
+  NS_IMETHOD Run() override {
     if (mCallback) {
       if (mStream) {
         mCallback->OnInputStreamReady(mStream);
@@ -106,16 +95,15 @@ public:
     return NS_OK;
   }
 
-  nsresult Cancel() override
-  {
+  nsresult Cancel() override {
     mCallback = nullptr;
     return NS_OK;
   }
 
-private:
-  nsCOMPtr<nsIAsyncInputStream>    mStream;
+ private:
+  nsCOMPtr<nsIAsyncInputStream> mStream;
   nsCOMPtr<nsIInputStreamCallback> mCallback;
-  nsCOMPtr<nsIEventTarget>         mTarget;
+  nsCOMPtr<nsIEventTarget> mTarget;
 };
 
 NS_IMPL_ISUPPORTS_INHERITED(nsInputStreamReadyEvent, CancelableRunnable,
@@ -126,24 +114,19 @@ NS_IMPL_ISUPPORTS_INHERITED(nsInputStreamReadyEvent, CancelableRunnable,
 // This is a nsICancelableRunnable because we can dispatch it to Workers and
 // those can be shut down at any time, and in these cases, Cancel() is called
 // instead of Run().
-class nsOutputStreamReadyEvent final
-  : public CancelableRunnable
-  , public nsIOutputStreamCallback
-{
-public:
+class nsOutputStreamReadyEvent final : public CancelableRunnable,
+                                       public nsIOutputStreamCallback {
+ public:
   NS_DECL_ISUPPORTS_INHERITED
 
   nsOutputStreamReadyEvent(nsIOutputStreamCallback* aCallback,
                            nsIEventTarget* aTarget)
-    : CancelableRunnable("nsOutputStreamReadyEvent")
-    , mCallback(aCallback)
-    , mTarget(aTarget)
-  {
-  }
+      : CancelableRunnable("nsOutputStreamReadyEvent"),
+        mCallback(aCallback),
+        mTarget(aTarget) {}
 
-private:
-  ~nsOutputStreamReadyEvent()
-  {
+ private:
+  ~nsOutputStreamReadyEvent() {
     if (!mCallback) {
       return;
     }
@@ -158,12 +141,12 @@ private:
     nsresult rv = mTarget->IsOnCurrentThread(&val);
     if (NS_FAILED(rv) || !val) {
       nsCOMPtr<nsIOutputStreamCallback> event =
-        NS_NewOutputStreamReadyEvent(mCallback, mTarget);
+          NS_NewOutputStreamReadyEvent(mCallback, mTarget);
       mCallback = nullptr;
       if (event) {
         rv = event->OnOutputStreamReady(nullptr);
         if (NS_FAILED(rv)) {
-          NS_NOTREACHED("leaking stream event");
+          MOZ_ASSERT_UNREACHABLE("leaking stream event");
           nsISupports* sup = event;
           NS_ADDREF(sup);
         }
@@ -171,13 +154,11 @@ private:
     }
   }
 
-public:
-  NS_IMETHOD OnOutputStreamReady(nsIAsyncOutputStream* aStream) override
-  {
+ public:
+  NS_IMETHOD OnOutputStreamReady(nsIAsyncOutputStream* aStream) override {
     mStream = aStream;
 
-    nsresult rv =
-      mTarget->Dispatch(this, NS_DISPATCH_NORMAL);
+    nsresult rv = mTarget->Dispatch(this, NS_DISPATCH_NORMAL);
     if (NS_FAILED(rv)) {
       NS_WARNING("PostEvent failed");
       return NS_ERROR_FAILURE;
@@ -186,8 +167,7 @@ public:
     return NS_OK;
   }
 
-  NS_IMETHOD Run() override
-  {
+  NS_IMETHOD Run() override {
     if (mCallback) {
       if (mStream) {
         mCallback->OnOutputStreamReady(mStream);
@@ -197,16 +177,15 @@ public:
     return NS_OK;
   }
 
-  nsresult Cancel() override
-  {
+  nsresult Cancel() override {
     mCallback = nullptr;
     return NS_OK;
   }
 
-private:
-  nsCOMPtr<nsIAsyncOutputStream>    mStream;
+ private:
+  nsCOMPtr<nsIAsyncOutputStream> mStream;
   nsCOMPtr<nsIOutputStreamCallback> mCallback;
-  nsCOMPtr<nsIEventTarget>          mTarget;
+  nsCOMPtr<nsIEventTarget> mTarget;
 };
 
 NS_IMPL_ISUPPORTS_INHERITED(nsOutputStreamReadyEvent, CancelableRunnable,
@@ -214,26 +193,22 @@ NS_IMPL_ISUPPORTS_INHERITED(nsOutputStreamReadyEvent, CancelableRunnable,
 
 //-----------------------------------------------------------------------------
 
-already_AddRefed<nsIInputStreamCallback>
-NS_NewInputStreamReadyEvent(const char* aName,
-                            nsIInputStreamCallback* aCallback,
-                            nsIEventTarget* aTarget)
-{
+already_AddRefed<nsIInputStreamCallback> NS_NewInputStreamReadyEvent(
+    const char* aName, nsIInputStreamCallback* aCallback,
+    nsIEventTarget* aTarget) {
   NS_ASSERTION(aCallback, "null callback");
   NS_ASSERTION(aTarget, "null target");
   RefPtr<nsInputStreamReadyEvent> ev =
-    new nsInputStreamReadyEvent(aName, aCallback, aTarget);
+      new nsInputStreamReadyEvent(aName, aCallback, aTarget);
   return ev.forget();
 }
 
-already_AddRefed<nsIOutputStreamCallback>
-NS_NewOutputStreamReadyEvent(nsIOutputStreamCallback* aCallback,
-                             nsIEventTarget* aTarget)
-{
+already_AddRefed<nsIOutputStreamCallback> NS_NewOutputStreamReadyEvent(
+    nsIOutputStreamCallback* aCallback, nsIEventTarget* aTarget) {
   NS_ASSERTION(aCallback, "null callback");
   NS_ASSERTION(aTarget, "null target");
   RefPtr<nsOutputStreamReadyEvent> ev =
-    new nsOutputStreamReadyEvent(aCallback, aTarget);
+      new nsOutputStreamReadyEvent(aCallback, aTarget);
   return ev.forget();
 }
 
@@ -241,41 +216,31 @@ NS_NewOutputStreamReadyEvent(nsIOutputStreamCallback* aCallback,
 // NS_AsyncCopy implementation
 
 // abstract stream copier...
-class nsAStreamCopier
-  : public nsIInputStreamCallback
-  , public nsIOutputStreamCallback
-  , public CancelableRunnable
-{
-public:
+class nsAStreamCopier : public nsIInputStreamCallback,
+                        public nsIOutputStreamCallback,
+                        public CancelableRunnable {
+ public:
   NS_DECL_ISUPPORTS_INHERITED
 
   nsAStreamCopier()
-    : CancelableRunnable("nsAStreamCopier")
-    , mLock("nsAStreamCopier.mLock")
-    , mCallback(nullptr)
-    , mProgressCallback(nullptr)
-    , mClosure(nullptr)
-    , mChunkSize(0)
-    , mEventInProcess(false)
-    , mEventIsPending(false)
-    , mCloseSource(true)
-    , mCloseSink(true)
-    , mCanceled(false)
-    , mCancelStatus(NS_OK)
-  {
-  }
+      : CancelableRunnable("nsAStreamCopier"),
+        mLock("nsAStreamCopier.mLock"),
+        mCallback(nullptr),
+        mProgressCallback(nullptr),
+        mClosure(nullptr),
+        mChunkSize(0),
+        mEventInProcess(false),
+        mEventIsPending(false),
+        mCloseSource(true),
+        mCloseSink(true),
+        mCanceled(false),
+        mCancelStatus(NS_OK) {}
 
   // kick off the async copy...
-  nsresult Start(nsIInputStream* aSource,
-                 nsIOutputStream* aSink,
-                 nsIEventTarget* aTarget,
-                 nsAsyncCopyCallbackFun aCallback,
-                 void* aClosure,
-                 uint32_t aChunksize,
-                 bool aCloseSource,
-                 bool aCloseSink,
-                 nsAsyncCopyProgressFun aProgressCallback)
-  {
+  nsresult Start(nsIInputStream* aSource, nsIOutputStream* aSink,
+                 nsIEventTarget* aTarget, nsAsyncCopyCallbackFun aCallback,
+                 void* aClosure, uint32_t aChunksize, bool aCloseSource,
+                 bool aCloseSink, nsAsyncCopyProgressFun aProgressCallback) {
     mSource = aSource;
     mSink = aSink;
     mTarget = aTarget;
@@ -297,8 +262,7 @@ public:
   virtual uint32_t DoCopy(nsresult* aSourceCondition,
                           nsresult* aSinkCondition) = 0;
 
-  void Process()
-  {
+  void Process() {
     if (!mSource || !mSink) {
       return;
     }
@@ -330,8 +294,8 @@ public:
         if (n > 0 && mProgressCallback) {
           mProgressCallback(mClosure, n);
         }
-        copyFailed = NS_FAILED(sourceCondition) ||
-                     NS_FAILED(sinkCondition) || n == 0;
+        copyFailed =
+            NS_FAILED(sourceCondition) || NS_FAILED(sinkCondition) || n == 0;
 
         MutexAutoLock lock(mLock);
         canceled = mCanceled;
@@ -344,8 +308,7 @@ public:
           mAsyncSource->AsyncWait(this, 0, 0, nullptr);
 
           if (mAsyncSink)
-            mAsyncSink->AsyncWait(this,
-                                  nsIAsyncOutputStream::WAIT_CLOSURE_ONLY,
+            mAsyncSink->AsyncWait(this, nsIAsyncOutputStream::WAIT_CLOSURE_ONLY,
                                   0, nullptr);
           break;
         } else if (sinkCondition == NS_BASE_STREAM_WOULD_BLOCK && mAsyncSink) {
@@ -355,9 +318,8 @@ public:
           mAsyncSink->AsyncWait(this, 0, 0, nullptr);
 
           if (mAsyncSource)
-            mAsyncSource->AsyncWait(this,
-                                    nsIAsyncInputStream::WAIT_CLOSURE_ONLY,
-                                    0, nullptr);
+            mAsyncSource->AsyncWait(
+                this, nsIAsyncInputStream::WAIT_CLOSURE_ONLY, 0, nullptr);
           break;
         }
       }
@@ -365,8 +327,8 @@ public:
         if (mCloseSource) {
           // close source
           if (mAsyncSource)
-            mAsyncSource->CloseWithStatus(
-              canceled ? cancelStatus : sinkCondition);
+            mAsyncSource->CloseWithStatus(canceled ? cancelStatus
+                                                   : sinkCondition);
           else {
             mSource->Close();
           }
@@ -377,14 +339,13 @@ public:
         if (mCloseSink) {
           // close sink
           if (mAsyncSink)
-            mAsyncSink->CloseWithStatus(
-              canceled ? cancelStatus : sourceCondition);
+            mAsyncSink->CloseWithStatus(canceled ? cancelStatus
+                                                 : sourceCondition);
           else {
             // If we have an nsISafeOutputStream, and our
             // sourceCondition and sinkCondition are not set to a
             // failure state, finish writing.
-            nsCOMPtr<nsISafeOutputStream> sostream =
-              do_QueryInterface(mSink);
+            nsCOMPtr<nsISafeOutputStream> sostream = do_QueryInterface(mSink);
             if (sostream && NS_SUCCEEDED(sourceCondition) &&
                 NS_SUCCEEDED(sinkCondition)) {
               sostream->Finish();
@@ -417,8 +378,7 @@ public:
     }
   }
 
-  nsresult Cancel(nsresult aReason)
-  {
+  nsresult Cancel(nsresult aReason) {
     MutexAutoLock lock(mLock);
     if (mCanceled) {
       return NS_ERROR_FAILURE;
@@ -434,21 +394,18 @@ public:
     return NS_OK;
   }
 
-  NS_IMETHOD OnInputStreamReady(nsIAsyncInputStream* aSource) override
-  {
+  NS_IMETHOD OnInputStreamReady(nsIAsyncInputStream* aSource) override {
     PostContinuationEvent();
     return NS_OK;
   }
 
-  NS_IMETHOD OnOutputStreamReady(nsIAsyncOutputStream* aSink) override
-  {
+  NS_IMETHOD OnOutputStreamReady(nsIAsyncOutputStream* aSink) override {
     PostContinuationEvent();
     return NS_OK;
   }
 
   // continuation event handler
-  NS_IMETHOD Run() override
-  {
+  NS_IMETHOD Run() override {
     Process();
 
     // clear "in process" flag and post any pending continuation event
@@ -464,8 +421,7 @@ public:
 
   nsresult Cancel() MOZ_MUST_OVERRIDE override = 0;
 
-  nsresult PostContinuationEvent()
-  {
+  nsresult PostContinuationEvent() {
     // we cannot post a continuation event if there is currently
     // an event in process.  doing so could result in Process being
     // run simultaneously on multiple threads, so we mark the event
@@ -477,8 +433,7 @@ public:
     return PostContinuationEvent_Locked();
   }
 
-  nsresult PostContinuationEvent_Locked()
-  {
+  nsresult PostContinuationEvent_Locked() {
     nsresult rv = NS_OK;
     if (mEventInProcess) {
       mEventIsPending = true;
@@ -493,59 +448,45 @@ public:
     return rv;
   }
 
-protected:
-  nsCOMPtr<nsIInputStream>       mSource;
-  nsCOMPtr<nsIOutputStream>      mSink;
-  nsCOMPtr<nsIAsyncInputStream>  mAsyncSource;
+ protected:
+  nsCOMPtr<nsIInputStream> mSource;
+  nsCOMPtr<nsIOutputStream> mSink;
+  nsCOMPtr<nsIAsyncInputStream> mAsyncSource;
   nsCOMPtr<nsIAsyncOutputStream> mAsyncSink;
-  nsCOMPtr<nsIEventTarget>       mTarget;
-  Mutex                          mLock;
-  nsAsyncCopyCallbackFun         mCallback;
-  nsAsyncCopyProgressFun         mProgressCallback;
-  void*                          mClosure;
-  uint32_t                       mChunkSize;
-  bool                           mEventInProcess;
-  bool                           mEventIsPending;
-  bool                           mCloseSource;
-  bool                           mCloseSink;
-  bool                           mCanceled;
-  nsresult                       mCancelStatus;
+  nsCOMPtr<nsIEventTarget> mTarget;
+  Mutex mLock;
+  nsAsyncCopyCallbackFun mCallback;
+  nsAsyncCopyProgressFun mProgressCallback;
+  void* mClosure;
+  uint32_t mChunkSize;
+  bool mEventInProcess;
+  bool mEventIsPending;
+  bool mCloseSource;
+  bool mCloseSink;
+  bool mCanceled;
+  nsresult mCancelStatus;
 
   // virtual since subclasses call superclass Release()
-  virtual ~nsAStreamCopier()
-  {
-  }
+  virtual ~nsAStreamCopier() {}
 };
 
-NS_IMPL_ISUPPORTS_INHERITED(nsAStreamCopier,
-                            CancelableRunnable,
-                            nsIInputStreamCallback,
-                            nsIOutputStreamCallback)
+NS_IMPL_ISUPPORTS_INHERITED(nsAStreamCopier, CancelableRunnable,
+                            nsIInputStreamCallback, nsIOutputStreamCallback)
 
-class nsStreamCopierIB final : public nsAStreamCopier
-{
-public:
-  nsStreamCopierIB() : nsAStreamCopier()
-  {
-  }
-  virtual ~nsStreamCopierIB()
-  {
-  }
+class nsStreamCopierIB final : public nsAStreamCopier {
+ public:
+  nsStreamCopierIB() : nsAStreamCopier() {}
+  virtual ~nsStreamCopierIB() {}
 
-  struct MOZ_STACK_CLASS ReadSegmentsState
-  {
+  struct MOZ_STACK_CLASS ReadSegmentsState {
     // the nsIOutputStream will outlive the ReadSegmentsState on the stack
     nsIOutputStream* MOZ_NON_OWNING_REF mSink;
-    nsresult         mSinkCondition;
+    nsresult mSinkCondition;
   };
 
-  static nsresult ConsumeInputBuffer(nsIInputStream* aInStr,
-                                     void* aClosure,
-                                     const char* aBuffer,
-                                     uint32_t aOffset,
-                                     uint32_t aCount,
-                                     uint32_t* aCountWritten)
-  {
+  static nsresult ConsumeInputBuffer(nsIInputStream* aInStr, void* aClosure,
+                                     const char* aBuffer, uint32_t aOffset,
+                                     uint32_t aCount, uint32_t* aCountWritten) {
     ReadSegmentsState* state = (ReadSegmentsState*)aClosure;
 
     nsresult rv = state->mSink->Write(aBuffer, aCount, aCountWritten);
@@ -559,49 +500,35 @@ public:
   }
 
   uint32_t DoCopy(nsresult* aSourceCondition,
-                  nsresult* aSinkCondition) override
-  {
+                  nsresult* aSinkCondition) override {
     ReadSegmentsState state;
     state.mSink = mSink;
     state.mSinkCondition = NS_OK;
 
     uint32_t n;
     *aSourceCondition =
-      mSource->ReadSegments(ConsumeInputBuffer, &state, mChunkSize, &n);
+        mSource->ReadSegments(ConsumeInputBuffer, &state, mChunkSize, &n);
     *aSinkCondition = state.mSinkCondition;
     return n;
   }
 
-  nsresult Cancel() override
-  {
-    return NS_OK;
-  }
+  nsresult Cancel() override { return NS_OK; }
 };
 
-class nsStreamCopierOB final : public nsAStreamCopier
-{
-public:
-  nsStreamCopierOB() : nsAStreamCopier()
-  {
-  }
-  virtual ~nsStreamCopierOB()
-  {
-  }
+class nsStreamCopierOB final : public nsAStreamCopier {
+ public:
+  nsStreamCopierOB() : nsAStreamCopier() {}
+  virtual ~nsStreamCopierOB() {}
 
-  struct MOZ_STACK_CLASS WriteSegmentsState
-  {
+  struct MOZ_STACK_CLASS WriteSegmentsState {
     // the nsIInputStream will outlive the WriteSegmentsState on the stack
     nsIInputStream* MOZ_NON_OWNING_REF mSource;
-    nsresult        mSourceCondition;
+    nsresult mSourceCondition;
   };
 
-  static nsresult FillOutputBuffer(nsIOutputStream* aOutStr,
-                                   void* aClosure,
-                                   char* aBuffer,
-                                   uint32_t aOffset,
-                                   uint32_t aCount,
-                                   uint32_t* aCountRead)
-  {
+  static nsresult FillOutputBuffer(nsIOutputStream* aOutStr, void* aClosure,
+                                   char* aBuffer, uint32_t aOffset,
+                                   uint32_t aCount, uint32_t* aCountRead) {
     WriteSegmentsState* state = (WriteSegmentsState*)aClosure;
 
     nsresult rv = state->mSource->Read(aBuffer, aCount, aCountRead);
@@ -615,40 +542,29 @@ public:
   }
 
   uint32_t DoCopy(nsresult* aSourceCondition,
-                  nsresult* aSinkCondition) override
-  {
+                  nsresult* aSinkCondition) override {
     WriteSegmentsState state;
     state.mSource = mSource;
     state.mSourceCondition = NS_OK;
 
     uint32_t n;
     *aSinkCondition =
-      mSink->WriteSegments(FillOutputBuffer, &state, mChunkSize, &n);
+        mSink->WriteSegments(FillOutputBuffer, &state, mChunkSize, &n);
     *aSourceCondition = state.mSourceCondition;
     return n;
   }
 
-  nsresult Cancel() override
-  {
-    return NS_OK;
-  }
+  nsresult Cancel() override { return NS_OK; }
 };
 
 //-----------------------------------------------------------------------------
 
-nsresult
-NS_AsyncCopy(nsIInputStream*         aSource,
-             nsIOutputStream*        aSink,
-             nsIEventTarget*         aTarget,
-             nsAsyncCopyMode         aMode,
-             uint32_t                aChunkSize,
-             nsAsyncCopyCallbackFun  aCallback,
-             void*                   aClosure,
-             bool                    aCloseSource,
-             bool                    aCloseSink,
-             nsISupports**           aCopierCtx,
-             nsAsyncCopyProgressFun  aProgressCallback)
-{
+nsresult NS_AsyncCopy(nsIInputStream* aSource, nsIOutputStream* aSink,
+                      nsIEventTarget* aTarget, nsAsyncCopyMode aMode,
+                      uint32_t aChunkSize, nsAsyncCopyCallbackFun aCallback,
+                      void* aClosure, bool aCloseSource, bool aCloseSink,
+                      nsISupports** aCopierCtx,
+                      nsAsyncCopyProgressFun aProgressCallback) {
   NS_ASSERTION(aTarget, "non-null target required");
 
   nsresult rv;
@@ -676,22 +592,42 @@ NS_AsyncCopy(nsIInputStream*         aSource,
 
 //-----------------------------------------------------------------------------
 
-nsresult
-NS_CancelAsyncCopy(nsISupports* aCopierCtx, nsresult aReason)
-{
+nsresult NS_CancelAsyncCopy(nsISupports* aCopierCtx, nsresult aReason) {
   nsAStreamCopier* copier =
-    static_cast<nsAStreamCopier*>(static_cast<nsIRunnable *>(aCopierCtx));
+      static_cast<nsAStreamCopier*>(static_cast<nsIRunnable*>(aCopierCtx));
   return copier->Cancel(aReason);
 }
 
 //-----------------------------------------------------------------------------
 
-nsresult
-NS_ConsumeStream(nsIInputStream* aStream, uint32_t aMaxCount,
-                 nsACString& aResult)
-{
+namespace {
+template <typename T>
+struct ResultTraits {};
+
+template <>
+struct ResultTraits<nsACString> {
+  static void Clear(nsACString& aString) { aString.Truncate(); }
+
+  static char* GetStorage(nsACString& aString) {
+    return aString.BeginWriting();
+  }
+};
+
+template <>
+struct ResultTraits<nsTArray<uint8_t>> {
+  static void Clear(nsTArray<uint8_t>& aArray) { aArray.Clear(); }
+
+  static char* GetStorage(nsTArray<uint8_t>& aArray) {
+    return reinterpret_cast<char*>(aArray.Elements());
+  }
+};
+}  // namespace
+
+template <typename T>
+nsresult DoConsumeStream(nsIInputStream* aStream, uint32_t aMaxCount,
+                         T& aResult) {
   nsresult rv = NS_OK;
-  aResult.Truncate();
+  ResultTraits<T>::Clear(aResult);
 
   while (aMaxCount) {
     uint64_t avail64;
@@ -710,15 +646,15 @@ NS_ConsumeStream(nsIInputStream* aStream, uint32_t aMaxCount,
 
     // resize aResult buffer
     uint32_t length = aResult.Length();
-    if (avail > UINT32_MAX - length) {
+    CheckedInt<uint32_t> newLength = CheckedInt<uint32_t>(length) + avail;
+    if (!newLength.isValid()) {
       return NS_ERROR_FILE_TOO_BIG;
     }
 
-    aResult.SetLength(length + avail);
-    if (aResult.Length() != (length + avail)) {
+    if (!aResult.SetLength(newLength.value(), fallible)) {
       return NS_ERROR_OUT_OF_MEMORY;
     }
-    char* buf = aResult.BeginWriting() + length;
+    char* buf = ResultTraits<T>::GetStorage(aResult) + length;
 
     uint32_t n;
     rv = aStream->Read(buf, avail, &n);
@@ -726,6 +662,7 @@ NS_ConsumeStream(nsIInputStream* aStream, uint32_t aMaxCount,
       break;
     }
     if (n != avail) {
+      MOZ_ASSERT(n < avail, "What happened there???");
       aResult.SetLength(length + n);
     }
     if (n == 0) {
@@ -737,25 +674,28 @@ NS_ConsumeStream(nsIInputStream* aStream, uint32_t aMaxCount,
   return rv;
 }
 
+nsresult NS_ConsumeStream(nsIInputStream* aStream, uint32_t aMaxCount,
+                          nsACString& aResult) {
+  return DoConsumeStream(aStream, aMaxCount, aResult);
+}
+
+nsresult NS_ConsumeStream(nsIInputStream* aStream, uint32_t aMaxCount,
+                          nsTArray<uint8_t>& aResult) {
+  return DoConsumeStream(aStream, aMaxCount, aResult);
+}
+
 //-----------------------------------------------------------------------------
 
-static nsresult
-TestInputStream(nsIInputStream* aInStr,
-                void* aClosure,
-                const char* aBuffer,
-                uint32_t aOffset,
-                uint32_t aCount,
-                uint32_t* aCountWritten)
-{
+static nsresult TestInputStream(nsIInputStream* aInStr, void* aClosure,
+                                const char* aBuffer, uint32_t aOffset,
+                                uint32_t aCount, uint32_t* aCountWritten) {
   bool* result = static_cast<bool*>(aClosure);
   *result = true;
   *aCountWritten = 0;
-  return NS_ERROR_ABORT; // don't call me anymore
+  return NS_ERROR_ABORT;  // don't call me anymore
 }
 
-bool
-NS_InputStreamIsBuffered(nsIInputStream* aStream)
-{
+bool NS_InputStreamIsBuffered(nsIInputStream* aStream) {
   nsCOMPtr<nsIBufferedInputStream> bufferedIn = do_QueryInterface(aStream);
   if (bufferedIn) {
     return true;
@@ -767,23 +707,16 @@ NS_InputStreamIsBuffered(nsIInputStream* aStream)
   return result || NS_SUCCEEDED(rv);
 }
 
-static nsresult
-TestOutputStream(nsIOutputStream* aOutStr,
-                 void* aClosure,
-                 char* aBuffer,
-                 uint32_t aOffset,
-                 uint32_t aCount,
-                 uint32_t* aCountRead)
-{
+static nsresult TestOutputStream(nsIOutputStream* aOutStr, void* aClosure,
+                                 char* aBuffer, uint32_t aOffset,
+                                 uint32_t aCount, uint32_t* aCountRead) {
   bool* result = static_cast<bool*>(aClosure);
   *result = true;
   *aCountRead = 0;
-  return NS_ERROR_ABORT; // don't call me anymore
+  return NS_ERROR_ABORT;  // don't call me anymore
 }
 
-bool
-NS_OutputStreamIsBuffered(nsIOutputStream* aStream)
-{
+bool NS_OutputStreamIsBuffered(nsIOutputStream* aStream) {
   nsCOMPtr<nsIBufferedOutputStream> bufferedOut = do_QueryInterface(aStream);
   if (bufferedOut) {
     return true;
@@ -797,14 +730,9 @@ NS_OutputStreamIsBuffered(nsIOutputStream* aStream)
 
 //-----------------------------------------------------------------------------
 
-nsresult
-NS_CopySegmentToStream(nsIInputStream* aInStr,
-                       void* aClosure,
-                       const char* aBuffer,
-                       uint32_t aOffset,
-                       uint32_t aCount,
-                       uint32_t* aCountWritten)
-{
+nsresult NS_CopySegmentToStream(nsIInputStream* aInStr, void* aClosure,
+                                const char* aBuffer, uint32_t aOffset,
+                                uint32_t aCount, uint32_t* aCountWritten) {
   nsIOutputStream* outStr = static_cast<nsIOutputStream*>(aClosure);
   *aCountWritten = 0;
   while (aCount) {
@@ -820,65 +748,43 @@ NS_CopySegmentToStream(nsIInputStream* aInStr,
   return NS_OK;
 }
 
-nsresult
-NS_CopySegmentToBuffer(nsIInputStream* aInStr,
-                       void* aClosure,
-                       const char* aBuffer,
-                       uint32_t aOffset,
-                       uint32_t aCount,
-                       uint32_t* aCountWritten)
-{
+nsresult NS_CopySegmentToBuffer(nsIInputStream* aInStr, void* aClosure,
+                                const char* aBuffer, uint32_t aOffset,
+                                uint32_t aCount, uint32_t* aCountWritten) {
   char* toBuf = static_cast<char*>(aClosure);
   memcpy(&toBuf[aOffset], aBuffer, aCount);
   *aCountWritten = aCount;
   return NS_OK;
 }
 
-nsresult
-NS_CopySegmentToBuffer(nsIOutputStream* aOutStr,
-                       void* aClosure,
-                       char* aBuffer,
-                       uint32_t aOffset,
-                       uint32_t aCount,
-                       uint32_t* aCountRead)
-{
+nsresult NS_CopySegmentToBuffer(nsIOutputStream* aOutStr, void* aClosure,
+                                char* aBuffer, uint32_t aOffset,
+                                uint32_t aCount, uint32_t* aCountRead) {
   const char* fromBuf = static_cast<const char*>(aClosure);
   memcpy(aBuffer, &fromBuf[aOffset], aCount);
   *aCountRead = aCount;
   return NS_OK;
 }
 
-nsresult
-NS_DiscardSegment(nsIInputStream* aInStr,
-                  void* aClosure,
-                  const char* aBuffer,
-                  uint32_t aOffset,
-                  uint32_t aCount,
-                  uint32_t* aCountWritten)
-{
+nsresult NS_DiscardSegment(nsIInputStream* aInStr, void* aClosure,
+                           const char* aBuffer, uint32_t aOffset,
+                           uint32_t aCount, uint32_t* aCountWritten) {
   *aCountWritten = aCount;
   return NS_OK;
 }
 
 //-----------------------------------------------------------------------------
 
-nsresult
-NS_WriteSegmentThunk(nsIInputStream* aInStr,
-                     void* aClosure,
-                     const char* aBuffer,
-                     uint32_t aOffset,
-                     uint32_t aCount,
-                     uint32_t* aCountWritten)
-{
+nsresult NS_WriteSegmentThunk(nsIInputStream* aInStr, void* aClosure,
+                              const char* aBuffer, uint32_t aOffset,
+                              uint32_t aCount, uint32_t* aCountWritten) {
   nsWriteSegmentThunk* thunk = static_cast<nsWriteSegmentThunk*>(aClosure);
   return thunk->mFun(thunk->mStream, thunk->mClosure, aBuffer, aOffset, aCount,
                      aCountWritten);
 }
 
-nsresult
-NS_FillArray(FallibleTArray<char>& aDest, nsIInputStream* aInput,
-             uint32_t aKeep, uint32_t* aNewBytes)
-{
+nsresult NS_FillArray(FallibleTArray<char>& aDest, nsIInputStream* aInput,
+                      uint32_t aKeep, uint32_t* aNewBytes) {
   MOZ_ASSERT(aInput, "null stream");
   MOZ_ASSERT(aKeep <= aDest.Length(), "illegal keep count");
 
@@ -889,7 +795,7 @@ NS_FillArray(FallibleTArray<char>& aDest, nsIInputStream* aInput,
   }
 
   nsresult rv =
-    aInput->Read(aBuffer + aKeep, aDest.Capacity() - aKeep, aNewBytes);
+      aInput->Read(aBuffer + aKeep, aDest.Capacity() - aKeep, aNewBytes);
   if (NS_FAILED(rv)) {
     *aNewBytes = 0;
   }
@@ -902,9 +808,7 @@ NS_FillArray(FallibleTArray<char>& aDest, nsIInputStream* aInput,
   return rv;
 }
 
-bool
-NS_InputStreamIsCloneable(nsIInputStream* aSource)
-{
+bool NS_InputStreamIsCloneable(nsIInputStream* aSource) {
   if (!aSource) {
     return false;
   }
@@ -913,10 +817,9 @@ NS_InputStreamIsCloneable(nsIInputStream* aSource)
   return cloneable && cloneable->GetCloneable();
 }
 
-nsresult
-NS_CloneInputStream(nsIInputStream* aSource, nsIInputStream** aCloneOut,
-                    nsIInputStream** aReplacementOut)
-{
+nsresult NS_CloneInputStream(nsIInputStream* aSource,
+                             nsIInputStream** aCloneOut,
+                             nsIInputStream** aReplacementOut) {
   if (NS_WARN_IF(!aSource)) {
     return NS_ERROR_FAILURE;
   }
@@ -944,23 +847,31 @@ NS_CloneInputStream(nsIInputStream* aSource, nsIInputStream** aCloneOut,
   nsCOMPtr<nsIInputStream> readerClone;
   nsCOMPtr<nsIOutputStream> writer;
 
-  nsresult rv = NS_NewPipe(getter_AddRefs(reader), getter_AddRefs(writer),
-                           0, 0,        // default segment size and max size
-                           true, true); // non-blocking
-  if (NS_WARN_IF(NS_FAILED(rv))) { return rv; }
+  nsresult rv = NS_NewPipe(getter_AddRefs(reader), getter_AddRefs(writer), 0,
+                           0,            // default segment size and max size
+                           true, true);  // non-blocking
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
 
   cloneable = do_QueryInterface(reader);
   MOZ_ASSERT(cloneable && cloneable->GetCloneable());
 
   rv = cloneable->Clone(getter_AddRefs(readerClone));
-  if (NS_WARN_IF(NS_FAILED(rv))) { return rv; }
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
 
   nsCOMPtr<nsIEventTarget> target =
-    do_GetService(NS_STREAMTRANSPORTSERVICE_CONTRACTID, &rv);
-  if (NS_WARN_IF(NS_FAILED(rv))) { return rv; }
+      do_GetService(NS_STREAMTRANSPORTSERVICE_CONTRACTID, &rv);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
 
   rv = NS_AsyncCopy(aSource, writer, target, NS_ASYNCCOPY_VIA_WRITESEGMENTS);
-  if (NS_WARN_IF(NS_FAILED(rv))) { return rv; }
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
 
   readerClone.forget(aCloneOut);
   reader.forget(aReplacementOut);
@@ -968,11 +879,10 @@ NS_CloneInputStream(nsIInputStream* aSource, nsIInputStream** aCloneOut,
   return NS_OK;
 }
 
-nsresult
-NS_MakeAsyncNonBlockingInputStream(already_AddRefed<nsIInputStream> aSource,
-                                   nsIAsyncInputStream** aAsyncInputStream)
-{
-  nsCOMPtr<nsIInputStream> source = Move(aSource);
+nsresult NS_MakeAsyncNonBlockingInputStream(
+    already_AddRefed<nsIInputStream> aSource,
+    nsIAsyncInputStream** aAsyncInputStream) {
+  nsCOMPtr<nsIInputStream> source = std::move(aSource);
   if (NS_WARN_IF(!aAsyncInputStream)) {
     return NS_ERROR_FAILURE;
   }
@@ -998,7 +908,7 @@ NS_MakeAsyncNonBlockingInputStream(already_AddRefed<nsIInputStream> aSource,
   }
 
   nsCOMPtr<nsIStreamTransportService> sts =
-    do_GetService(kStreamTransportServiceCID, &rv);
+      do_GetService(kStreamTransportServiceCID, &rv);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
