@@ -12,19 +12,24 @@
 #include "nsComponentManagerUtils.h"
 #include "nsExceptionHandler.h"
 #include "nsITimer.h"
+#include "prsystem.h"
 
 #ifdef MOZILLA_INTERNAL_API
-# include "nsThreadManager.h"
+#  include "nsThreadManager.h"
 #else
-# include "nsXPCOMCIDInternal.h"
-# include "nsIThreadManager.h"
-# include "nsServiceManagerUtils.h"
+#  include "nsXPCOMCIDInternal.h"
+#  include "nsIThreadManager.h"
+#  include "nsServiceManagerUtils.h"
 #endif
 
 #ifdef XP_WIN
-#include <windows.h>
+#  include <windows.h>
 #elif defined(XP_MACOSX)
-#include <sys/resource.h>
+#  include <sys/resource.h>
+#endif
+
+#if defined(ANDROID)
+#  include <sys/prctl.h>
 #endif
 
 using namespace mozilla;
@@ -34,8 +39,7 @@ using namespace mozilla;
 NS_IMPL_ISUPPORTS(IdlePeriod, nsIIdlePeriod)
 
 NS_IMETHODIMP
-IdlePeriod::GetIdlePeriodHint(TimeStamp* aIdleDeadline)
-{
+IdlePeriod::GetIdlePeriodHint(TimeStamp* aIdleDeadline) {
   *aIdleDeadline = TimeStamp();
   return NS_OK;
 }
@@ -43,68 +47,59 @@ IdlePeriod::GetIdlePeriodHint(TimeStamp* aIdleDeadline)
 // NS_IMPL_NAMED_* relies on the mName field, which is not present on
 // release or beta. Instead, fall back to using "Runnable" for all
 // runnables.
-#ifdef RELEASE_OR_BETA
-NS_IMPL_ISUPPORTS(Runnable, nsIRunnable, nsINamed)
-#else
+#  ifndef MOZ_COLLECTING_RUNNABLE_TELEMETRY
+NS_IMPL_ISUPPORTS(Runnable, nsIRunnable)
+#  else
 NS_IMPL_NAMED_ADDREF(Runnable, mName)
 NS_IMPL_NAMED_RELEASE(Runnable, mName)
 NS_IMPL_QUERY_INTERFACE(Runnable, nsIRunnable, nsINamed)
-#endif
+#  endif
 
 NS_IMETHODIMP
-Runnable::Run()
-{
+Runnable::Run() {
   // Do nothing
   return NS_OK;
 }
 
+#  ifdef MOZ_COLLECTING_RUNNABLE_TELEMETRY
 NS_IMETHODIMP
-Runnable::GetName(nsACString& aName)
-{
-#ifdef RELEASE_OR_BETA
-  aName.Truncate();
-#else
+Runnable::GetName(nsACString& aName) {
   if (mName) {
     aName.AssignASCII(mName);
   } else {
     aName.Truncate();
   }
-#endif
   return NS_OK;
 }
+#  endif
 
-NS_IMPL_ISUPPORTS_INHERITED(CancelableRunnable, Runnable,
-                            nsICancelableRunnable)
+NS_IMPL_ISUPPORTS_INHERITED(CancelableRunnable, Runnable, nsICancelableRunnable)
 
-nsresult
-CancelableRunnable::Cancel()
-{
+nsresult CancelableRunnable::Cancel() {
   // Do nothing
   return NS_OK;
 }
 
-NS_IMPL_ISUPPORTS_INHERITED(IdleRunnable, CancelableRunnable,
-                            nsIIdleRunnable)
+NS_IMPL_ISUPPORTS_INHERITED(IdleRunnable, CancelableRunnable, nsIIdleRunnable)
 
 NS_IMPL_ISUPPORTS_INHERITED(PrioritizableRunnable, Runnable,
                             nsIRunnablePriority)
 
-PrioritizableRunnable::PrioritizableRunnable(already_AddRefed<nsIRunnable>&& aRunnable,
-                                             uint32_t aPriority)
- // Real runnable name is managed by overridding the GetName function.
- : Runnable("PrioritizableRunnable")
- , mRunnable(Move(aRunnable))
- , mPriority(aPriority)
-{
-#if DEBUG
+PrioritizableRunnable::PrioritizableRunnable(
+    already_AddRefed<nsIRunnable>&& aRunnable, uint32_t aPriority)
+    // Real runnable name is managed by overridding the GetName function.
+    : Runnable("PrioritizableRunnable"),
+      mRunnable(std::move(aRunnable)),
+      mPriority(aPriority) {
+#  if DEBUG
   nsCOMPtr<nsIRunnablePriority> runnablePrio = do_QueryInterface(mRunnable);
   MOZ_ASSERT(!runnablePrio);
-#endif
+#  endif
 }
 
+#  ifdef MOZ_COLLECTING_RUNNABLE_TELEMETRY
 NS_IMETHODIMP
-PrioritizableRunnable::GetName(nsACString& aName)
-{
+PrioritizableRunnable::GetName(nsACString& aName) {
   // Try to get a name from the underlying runnable.
   nsCOMPtr<nsINamed> named = do_QueryInterface(mRunnable);
   if (named) {
@@ -112,40 +107,41 @@ PrioritizableRunnable::GetName(nsACString& aName)
   }
   return NS_OK;
 }
+#  endif
 
 NS_IMETHODIMP
-PrioritizableRunnable::Run()
-{
+PrioritizableRunnable::Run() {
   MOZ_RELEASE_ASSERT(NS_IsMainThread());
   return mRunnable->Run();
 }
 
 NS_IMETHODIMP
-PrioritizableRunnable::GetPriority(uint32_t* aPriority)
-{
+PrioritizableRunnable::GetPriority(uint32_t* aPriority) {
   *aPriority = mPriority;
   return NS_OK;
 }
 
+already_AddRefed<nsIRunnable> mozilla::CreateMediumHighRunnable(
+    already_AddRefed<nsIRunnable>&& aRunnable) {
+  nsCOMPtr<nsIRunnable> runnable = new PrioritizableRunnable(
+      std::move(aRunnable), nsIRunnablePriority::PRIORITY_MEDIUMHIGH);
+  return runnable.forget();
+}
+
 #endif  // XPCOM_GLUE_AVOID_NSPR
 
 //-----------------------------------------------------------------------------
 
-nsresult
-NS_NewNamedThread(const nsACString& aName,
-                  nsIThread** aResult,
-                  nsIRunnable* aEvent,
-                  uint32_t aStackSize)
-{
+nsresult NS_NewNamedThread(const nsACString& aName, nsIThread** aResult,
+                           nsIRunnable* aEvent, uint32_t aStackSize) {
   nsCOMPtr<nsIThread> thread;
 #ifdef MOZILLA_INTERNAL_API
-  nsresult rv =
-    nsThreadManager::get().nsThreadManager::NewNamedThread(aName, aStackSize,
-                                                           getter_AddRefs(thread));
+  nsresult rv = nsThreadManager::get().nsThreadManager::NewNamedThread(
+      aName, aStackSize, getter_AddRefs(thread));
 #else
   nsresult rv;
   nsCOMPtr<nsIThreadManager> mgr =
-    do_GetService(NS_THREADMANAGER_CONTRACTID, &rv);
+      do_GetService(NS_THREADMANAGER_CONTRACTID, &rv);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
@@ -168,21 +164,18 @@ NS_NewNamedThread(const nsACString& aName,
   return NS_OK;
 }
 
-nsresult
-NS_NewThread(nsIThread** aResult, nsIRunnable* aEvent, uint32_t aStackSize)
-{
+nsresult NS_NewThread(nsIThread** aResult, nsIRunnable* aEvent,
+                      uint32_t aStackSize) {
   return NS_NewNamedThread(NS_LITERAL_CSTRING(""), aResult, aEvent, aStackSize);
 }
 
-nsresult
-NS_GetCurrentThread(nsIThread** aResult)
-{
+nsresult NS_GetCurrentThread(nsIThread** aResult) {
 #ifdef MOZILLA_INTERNAL_API
   return nsThreadManager::get().nsThreadManager::GetCurrentThread(aResult);
 #else
   nsresult rv;
   nsCOMPtr<nsIThreadManager> mgr =
-    do_GetService(NS_THREADMANAGER_CONTRACTID, &rv);
+      do_GetService(NS_THREADMANAGER_CONTRACTID, &rv);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
@@ -190,15 +183,13 @@ NS_GetCurrentThread(nsIThread** aResult)
 #endif
 }
 
-nsresult
-NS_GetMainThread(nsIThread** aResult)
-{
+nsresult NS_GetMainThread(nsIThread** aResult) {
 #ifdef MOZILLA_INTERNAL_API
   return nsThreadManager::get().nsThreadManager::GetMainThread(aResult);
 #else
   nsresult rv;
   nsCOMPtr<nsIThreadManager> mgr =
-    do_GetService(NS_THREADMANAGER_CONTRACTID, &rv);
+      do_GetService(NS_THREADMANAGER_CONTRACTID, &rv);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
@@ -206,9 +197,7 @@ NS_GetMainThread(nsIThread** aResult)
 #endif
 }
 
-nsresult
-NS_DispatchToCurrentThread(already_AddRefed<nsIRunnable>&& aEvent)
-{
+nsresult NS_DispatchToCurrentThread(already_AddRefed<nsIRunnable>&& aEvent) {
   nsresult rv;
   nsCOMPtr<nsIRunnable> event(aEvent);
 #ifdef MOZILLA_INTERNAL_API
@@ -239,21 +228,19 @@ NS_DispatchToCurrentThread(already_AddRefed<nsIRunnable>&& aEvent)
 // It is common to call NS_DispatchToCurrentThread with a newly
 // allocated runnable with a refcount of zero. To keep us from leaking
 // the runnable if the dispatch method fails, we take a death grip.
-nsresult
-NS_DispatchToCurrentThread(nsIRunnable* aEvent)
-{
+nsresult NS_DispatchToCurrentThread(nsIRunnable* aEvent) {
   nsCOMPtr<nsIRunnable> event(aEvent);
   return NS_DispatchToCurrentThread(event.forget());
 }
 
-nsresult
-NS_DispatchToMainThread(already_AddRefed<nsIRunnable>&& aEvent, uint32_t aDispatchFlags)
-{
-  LeakRefPtr<nsIRunnable> event(Move(aEvent));
+nsresult NS_DispatchToMainThread(already_AddRefed<nsIRunnable>&& aEvent,
+                                 uint32_t aDispatchFlags) {
+  LeakRefPtr<nsIRunnable> event(std::move(aEvent));
   nsCOMPtr<nsIThread> thread;
   nsresult rv = NS_GetMainThread(getter_AddRefs(thread));
   if (NS_WARN_IF(NS_FAILED(rv))) {
-    NS_ASSERTION(false, "Failed NS_DispatchToMainThread() in shutdown; leaking");
+    NS_ASSERTION(false,
+                 "Failed NS_DispatchToMainThread() in shutdown; leaking");
     // NOTE: if you stop leaking here, adjust Promise::MaybeReportRejected(),
     // which assumes a leak here, or split into leaks and no-leaks versions
     return rv;
@@ -266,16 +253,13 @@ NS_DispatchToMainThread(already_AddRefed<nsIRunnable>&& aEvent, uint32_t aDispat
 // likely that the runnable is being dispatched to the main thread
 // because it owns main thread only objects, so it is not safe to
 // release them here.
-nsresult
-NS_DispatchToMainThread(nsIRunnable* aEvent, uint32_t aDispatchFlags)
-{
+nsresult NS_DispatchToMainThread(nsIRunnable* aEvent, uint32_t aDispatchFlags) {
   nsCOMPtr<nsIRunnable> event(aEvent);
   return NS_DispatchToMainThread(event.forget(), aDispatchFlags);
 }
 
-nsresult
-NS_DelayedDispatchToCurrentThread(already_AddRefed<nsIRunnable>&& aEvent, uint32_t aDelayMs)
-{
+nsresult NS_DelayedDispatchToCurrentThread(
+    already_AddRefed<nsIRunnable>&& aEvent, uint32_t aDelayMs) {
   nsCOMPtr<nsIRunnable> event(aEvent);
 #ifdef MOZILLA_INTERNAL_API
   nsIEventTarget* thread = GetCurrentThreadEventTarget();
@@ -294,10 +278,9 @@ NS_DelayedDispatchToCurrentThread(already_AddRefed<nsIRunnable>&& aEvent, uint32
   return thread->DelayedDispatch(event.forget(), aDelayMs);
 }
 
-nsresult
-NS_IdleDispatchToThread(already_AddRefed<nsIRunnable>&& aEvent,
-                        nsIThread* aThread)
-{
+nsresult NS_DispatchToThreadQueue(already_AddRefed<nsIRunnable>&& aEvent,
+                                  nsIThread* aThread,
+                                  EventQueuePriority aQueue) {
   nsresult rv;
   nsCOMPtr<nsIRunnable> event(aEvent);
   NS_ENSURE_TRUE(event, NS_ERROR_INVALID_ARG);
@@ -307,7 +290,7 @@ NS_IdleDispatchToThread(already_AddRefed<nsIRunnable>&& aEvent,
   // To keep us from leaking the runnable if dispatch method fails,
   // we grab the reference on failures and release it.
   nsIRunnable* temp = event.get();
-  rv = aThread->IdleDispatch(event.forget());
+  rv = aThread->DispatchToQueue(event.forget(), aQueue);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     // Dispatch() leaked the reference to the event, but due to caller's
     // assumptions, we shouldn't leak here. And given we are on the same
@@ -318,23 +301,28 @@ NS_IdleDispatchToThread(already_AddRefed<nsIRunnable>&& aEvent,
   return rv;
 }
 
-nsresult
-NS_IdleDispatchToCurrentThread(already_AddRefed<nsIRunnable>&& aEvent)
-{
-  return NS_IdleDispatchToThread(Move(aEvent),
-                                 NS_GetCurrentThread());
+nsresult NS_DispatchToCurrentThreadQueue(already_AddRefed<nsIRunnable>&& aEvent,
+                                         EventQueuePriority aQueue) {
+  return NS_DispatchToThreadQueue(std::move(aEvent), NS_GetCurrentThread(),
+                                  aQueue);
 }
 
-class IdleRunnableWrapper : public IdleRunnable
-{
-public:
-  explicit IdleRunnableWrapper(already_AddRefed<nsIRunnable>&& aEvent)
-    : mRunnable(Move(aEvent))
-  {
+extern nsresult NS_DispatchToMainThreadQueue(
+    already_AddRefed<nsIRunnable>&& aEvent, EventQueuePriority aQueue) {
+  nsCOMPtr<nsIThread> mainThread;
+  nsresult rv = NS_GetMainThread(getter_AddRefs(mainThread));
+  if (NS_SUCCEEDED(rv)) {
+    return NS_DispatchToThreadQueue(std::move(aEvent), mainThread, aQueue);
   }
+  return rv;
+}
 
-  NS_IMETHOD Run() override
-  {
+class IdleRunnableWrapper final : public IdleRunnable {
+ public:
+  explicit IdleRunnableWrapper(already_AddRefed<nsIRunnable>&& aEvent)
+      : mRunnable(std::move(aEvent)) {}
+
+  NS_IMETHOD Run() override {
     if (!mRunnable) {
       return NS_OK;
     }
@@ -343,29 +331,22 @@ public:
     return runnable->Run();
   }
 
-  static void
-  TimedOut(nsITimer* aTimer, void* aClosure)
-  {
+  static void TimedOut(nsITimer* aTimer, void* aClosure) {
     RefPtr<IdleRunnableWrapper> runnable =
-      static_cast<IdleRunnableWrapper*>(aClosure);
+        static_cast<IdleRunnableWrapper*>(aClosure);
     runnable->Run();
   }
 
-  void SetTimer(uint32_t aDelay, nsIEventTarget* aTarget) override
-  {
+  void SetTimer(uint32_t aDelay, nsIEventTarget* aTarget) override {
     MOZ_ASSERT(aTarget);
     MOZ_ASSERT(!mTimer);
-    NS_NewTimerWithFuncCallback(getter_AddRefs(mTimer),
-                                TimedOut,
-                                this,
-                                aDelay,
+    NS_NewTimerWithFuncCallback(getter_AddRefs(mTimer), TimedOut, this, aDelay,
                                 nsITimer::TYPE_ONE_SHOT,
-                                "IdleRunnableWrapper::SetTimer",
-                                aTarget);
+                                "IdleRunnableWrapper::SetTimer", aTarget);
   }
 
-  NS_IMETHOD GetName(nsACString& aName) override
-  {
+#ifdef MOZ_COLLECTING_RUNNABLE_TELEMETRY
+  NS_IMETHOD GetName(nsACString& aName) override {
     aName.AssignLiteral("IdleRunnableWrapper");
     if (nsCOMPtr<nsINamed> named = do_QueryInterface(mRunnable)) {
       nsAutoCString name;
@@ -377,15 +358,12 @@ public:
     }
     return NS_OK;
   }
+#endif
 
-private:
-  ~IdleRunnableWrapper()
-  {
-    CancelTimer();
-  }
+ private:
+  ~IdleRunnableWrapper() { CancelTimer(); }
 
-  void CancelTimer()
-  {
+  void CancelTimer() {
     if (mTimer) {
       mTimer->Cancel();
     }
@@ -395,15 +373,15 @@ private:
   nsCOMPtr<nsIRunnable> mRunnable;
 };
 
-extern nsresult
-NS_IdleDispatchToThread(already_AddRefed<nsIRunnable>&& aEvent,
-                        uint32_t aTimeout,
-                        nsIThread* aThread)
-{
-  nsCOMPtr<nsIRunnable> event(Move(aEvent));
+extern nsresult NS_DispatchToThreadQueue(already_AddRefed<nsIRunnable>&& aEvent,
+                                         uint32_t aTimeout, nsIThread* aThread,
+                                         EventQueuePriority aQueue) {
+  nsCOMPtr<nsIRunnable> event(std::move(aEvent));
   NS_ENSURE_TRUE(event, NS_ERROR_INVALID_ARG);
+  MOZ_ASSERT(aQueue == EventQueuePriority::Idle ||
+             aQueue == EventQueuePriority::DeferredTimers);
 
-  //XXX Using current thread for now as the nsIEventTarget.
+  // XXX Using current thread for now as the nsIEventTarget.
   nsIEventTarget* target = mozilla::GetCurrentThreadEventTarget();
   if (!target) {
     return NS_ERROR_UNEXPECTED;
@@ -418,31 +396,28 @@ NS_IdleDispatchToThread(already_AddRefed<nsIRunnable>&& aEvent,
   }
   idleEvent->SetTimer(aTimeout, target);
 
-  return NS_IdleDispatchToThread(event.forget(), aThread);
+  return NS_DispatchToThreadQueue(event.forget(), aThread, aQueue);
 }
 
-extern nsresult
-NS_IdleDispatchToCurrentThread(already_AddRefed<nsIRunnable>&& aEvent,
-                               uint32_t aTimeout)
-{
-  return NS_IdleDispatchToThread(Move(aEvent), aTimeout,
-                                 NS_GetCurrentThread());
+extern nsresult NS_DispatchToCurrentThreadQueue(
+    already_AddRefed<nsIRunnable>&& aEvent, uint32_t aTimeout,
+    EventQueuePriority aQueue) {
+  return NS_DispatchToThreadQueue(std::move(aEvent), aTimeout,
+                                  NS_GetCurrentThread(), aQueue);
 }
 
 #ifndef XPCOM_GLUE_AVOID_NSPR
-nsresult
-NS_ProcessPendingEvents(nsIThread* aThread, PRIntervalTime aTimeout)
-{
+nsresult NS_ProcessPendingEvents(nsIThread* aThread, PRIntervalTime aTimeout) {
   nsresult rv = NS_OK;
 
-#ifdef MOZILLA_INTERNAL_API
+#  ifdef MOZILLA_INTERNAL_API
   if (!aThread) {
     aThread = NS_GetCurrentThread();
     if (NS_WARN_IF(!aThread)) {
       return NS_ERROR_UNEXPECTED;
     }
   }
-#else
+#  else
   nsCOMPtr<nsIThread> current;
   if (!aThread) {
     rv = NS_GetCurrentThread(getter_AddRefs(current));
@@ -451,7 +426,7 @@ NS_ProcessPendingEvents(nsIThread* aThread, PRIntervalTime aTimeout)
     }
     aThread = current.get();
   }
-#endif
+#  endif
 
   PRIntervalTime start = PR_IntervalNow();
   for (;;) {
@@ -466,18 +441,14 @@ NS_ProcessPendingEvents(nsIThread* aThread, PRIntervalTime aTimeout)
   }
   return rv;
 }
-#endif // XPCOM_GLUE_AVOID_NSPR
+#endif  // XPCOM_GLUE_AVOID_NSPR
 
-inline bool
-hasPendingEvents(nsIThread* aThread)
-{
+inline bool hasPendingEvents(nsIThread* aThread) {
   bool val;
   return NS_SUCCEEDED(aThread->HasPendingEvents(&val)) && val;
 }
 
-bool
-NS_HasPendingEvents(nsIThread* aThread)
-{
+bool NS_HasPendingEvents(nsIThread* aThread) {
   if (!aThread) {
 #ifndef MOZILLA_INTERNAL_API
     nsCOMPtr<nsIThread> current;
@@ -493,9 +464,7 @@ NS_HasPendingEvents(nsIThread* aThread)
   return hasPendingEvents(aThread);
 }
 
-bool
-NS_ProcessNextEvent(nsIThread* aThread, bool aMayWait)
-{
+bool NS_ProcessNextEvent(nsIThread* aThread, bool aMayWait) {
 #ifdef MOZILLA_INTERNAL_API
   if (!aThread) {
     aThread = NS_GetCurrentThread();
@@ -517,24 +486,23 @@ NS_ProcessNextEvent(nsIThread* aThread, bool aMayWait)
   return NS_SUCCEEDED(aThread->ProcessNextEvent(aMayWait, &val)) && val;
 }
 
-void
-NS_SetCurrentThreadName(const char* aName)
-{
+void NS_SetCurrentThreadName(const char* aName) {
+#if defined(ANDROID)
+  // Workaround for Bug 1541216 - PR_SetCurrentThreadName() Fails to set the
+  // thread name on Android.
+  prctl(PR_SET_NAME, reinterpret_cast<unsigned long>(aName));
+#else
   PR_SetCurrentThreadName(aName);
+#endif
   CrashReporter::SetCurrentThreadName(aName);
 }
 
 #ifdef MOZILLA_INTERNAL_API
-nsIThread*
-NS_GetCurrentThread()
-{
+nsIThread* NS_GetCurrentThread() {
   return nsThreadManager::get().GetCurrentThread();
 }
 
-
-nsIThread*
-NS_GetCurrentThreadNoCreate()
-{
+nsIThread* NS_GetCurrentThreadNoCreate() {
   if (nsThreadManager::get().IsNSThread()) {
     return NS_GetCurrentThread();
   }
@@ -543,34 +511,29 @@ NS_GetCurrentThreadNoCreate()
 #endif
 
 // nsThreadPoolNaming
-nsCString
-nsThreadPoolNaming::GetNextThreadName(const nsACString& aPoolName)
-{
+nsCString nsThreadPoolNaming::GetNextThreadName(const nsACString& aPoolName) {
   nsCString name(aPoolName);
   name.AppendLiteral(" #");
-  name.AppendInt(++mCounter, 10); // The counter is declared as atomic
+  name.AppendInt(++mCounter, 10);  // The counter is declared as atomic
   return name;
 }
 
 // nsAutoLowPriorityIO
-nsAutoLowPriorityIO::nsAutoLowPriorityIO()
-{
+nsAutoLowPriorityIO::nsAutoLowPriorityIO() {
 #if defined(XP_WIN)
-  lowIOPrioritySet = SetThreadPriority(GetCurrentThread(),
-                                       THREAD_MODE_BACKGROUND_BEGIN);
+  lowIOPrioritySet =
+      SetThreadPriority(GetCurrentThread(), THREAD_MODE_BACKGROUND_BEGIN);
 #elif defined(XP_MACOSX)
   oldPriority = getiopolicy_np(IOPOL_TYPE_DISK, IOPOL_SCOPE_THREAD);
-  lowIOPrioritySet = oldPriority != -1 &&
-                     setiopolicy_np(IOPOL_TYPE_DISK,
-                                    IOPOL_SCOPE_THREAD,
-                                    IOPOL_THROTTLE) != -1;
+  lowIOPrioritySet =
+      oldPriority != -1 &&
+      setiopolicy_np(IOPOL_TYPE_DISK, IOPOL_SCOPE_THREAD, IOPOL_THROTTLE) != -1;
 #else
   lowIOPrioritySet = false;
 #endif
 }
 
-nsAutoLowPriorityIO::~nsAutoLowPriorityIO()
-{
+nsAutoLowPriorityIO::~nsAutoLowPriorityIO() {
 #if defined(XP_WIN)
   if (MOZ_LIKELY(lowIOPrioritySet)) {
     // On Windows the old thread priority is automatically restored
@@ -585,9 +548,7 @@ nsAutoLowPriorityIO::~nsAutoLowPriorityIO()
 
 namespace mozilla {
 
-nsIEventTarget*
-GetCurrentThreadEventTarget()
-{
+nsIEventTarget* GetCurrentThreadEventTarget() {
   nsCOMPtr<nsIThread> thread;
   nsresult rv = NS_GetCurrentThread(getter_AddRefs(thread));
   if (NS_FAILED(rv)) {
@@ -597,9 +558,7 @@ GetCurrentThreadEventTarget()
   return thread->EventTarget();
 }
 
-nsIEventTarget*
-GetMainThreadEventTarget()
-{
+nsIEventTarget* GetMainThreadEventTarget() {
   nsCOMPtr<nsIThread> thread;
   nsresult rv = NS_GetMainThread(getter_AddRefs(thread));
   if (NS_FAILED(rv)) {
@@ -609,9 +568,7 @@ GetMainThreadEventTarget()
   return thread->EventTarget();
 }
 
-nsISerialEventTarget*
-GetCurrentThreadSerialEventTarget()
-{
+nsISerialEventTarget* GetCurrentThreadSerialEventTarget() {
   nsCOMPtr<nsIThread> thread;
   nsresult rv = NS_GetCurrentThread(getter_AddRefs(thread));
   if (NS_FAILED(rv)) {
@@ -621,9 +578,7 @@ GetCurrentThreadSerialEventTarget()
   return thread->SerialEventTarget();
 }
 
-nsISerialEventTarget*
-GetMainThreadSerialEventTarget()
-{
+nsISerialEventTarget* GetMainThreadSerialEventTarget() {
   nsCOMPtr<nsIThread> thread;
   nsresult rv = NS_GetMainThread(getter_AddRefs(thread));
   if (NS_FAILED(rv)) {
@@ -633,13 +588,62 @@ GetMainThreadSerialEventTarget()
   return thread->SerialEventTarget();
 }
 
-} // namespace mozilla
+size_t GetNumberOfProcessors() {
+#if defined(XP_LINUX) && defined(MOZ_SANDBOX)
+  static const PRInt32 procs = PR_GetNumberOfProcessors();
+#else
+  PRInt32 procs = PR_GetNumberOfProcessors();
+#endif
+  MOZ_ASSERT(procs > 0);
+  return static_cast<size_t>(procs);
+}
 
-bool
-nsIEventTarget::IsOnCurrentThread()
-{
+}  // namespace mozilla
+
+bool nsIEventTarget::IsOnCurrentThread() {
   if (mVirtualThread) {
     return mVirtualThread == GetCurrentVirtualThread();
   }
   return IsOnCurrentThreadInfallible();
 }
+
+extern "C" {
+// These functions use the C language linkage because they're exposed to Rust
+// via the xpcom/rust/moz_task crate, which wraps them in safe Rust functions
+// that enable Rust code to get/create threads and dispatch runnables on them.
+
+nsresult NS_GetCurrentThreadEventTarget(nsIEventTarget** aResult) {
+  nsCOMPtr<nsIEventTarget> target = mozilla::GetCurrentThreadEventTarget();
+  if (!target) {
+    return NS_ERROR_UNEXPECTED;
+  }
+  target.forget(aResult);
+  return NS_OK;
+}
+
+nsresult NS_GetMainThreadEventTarget(nsIEventTarget** aResult) {
+  nsCOMPtr<nsIEventTarget> target = mozilla::GetMainThreadEventTarget();
+  if (!target) {
+    return NS_ERROR_UNEXPECTED;
+  }
+  target.forget(aResult);
+  return NS_OK;
+}
+
+// NS_NewNamedThread's aStackSize parameter has the default argument
+// nsIThreadManager::DEFAULT_STACK_SIZE, but we can't omit default arguments
+// when calling a C++ function from Rust, and we can't access
+// nsIThreadManager::DEFAULT_STACK_SIZE in Rust to pass it explicitly,
+// since it is defined in a %{C++ ... %} block within nsIThreadManager.idl.
+// So we indirect through this function.
+nsresult NS_NewNamedThreadWithDefaultStackSize(const nsACString& aName,
+                                               nsIThread** aResult,
+                                               nsIRunnable* aEvent) {
+  return NS_NewNamedThread(aName, aResult, aEvent);
+}
+
+bool NS_IsCurrentThread(nsIEventTarget* aThread) {
+  return aThread->IsOnCurrentThread();
+}
+
+}  // extern "C"
